@@ -15,6 +15,10 @@ import (
 )
 
 func TestACL_Legacy_Disabled_Response(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := NewTestAgent(t, "")
 	defer a.Shutdown()
@@ -70,6 +74,10 @@ func makeTestACL(t *testing.T, srv *HTTPHandlers) string {
 }
 
 func TestACL_Legacy_Update(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := NewTestAgent(t, TestACLConfig())
 	defer a.Shutdown()
@@ -100,6 +108,10 @@ func TestACL_Legacy_Update(t *testing.T) {
 }
 
 func TestACL_Legacy_UpdateUpsert(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := NewTestAgent(t, TestACLConfig())
 	defer a.Shutdown()
@@ -129,6 +141,10 @@ func TestACL_Legacy_UpdateUpsert(t *testing.T) {
 }
 
 func TestACL_Legacy_Destroy(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := NewTestAgent(t, TestACLConfig())
 	defer a.Shutdown()
@@ -161,6 +177,10 @@ func TestACL_Legacy_Destroy(t *testing.T) {
 }
 
 func TestACL_Legacy_Clone(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := NewTestAgent(t, TestACLConfig())
 	defer a.Shutdown()
@@ -205,6 +225,10 @@ func TestACL_Legacy_Clone(t *testing.T) {
 }
 
 func TestACL_Legacy_Get(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	t.Run("wrong id", func(t *testing.T) {
 		a := NewTestAgent(t, TestACLConfig())
@@ -250,6 +274,10 @@ func TestACL_Legacy_Get(t *testing.T) {
 }
 
 func TestACL_Legacy_List(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := NewTestAgent(t, TestACLConfig())
 	defer a.Shutdown()
@@ -278,6 +306,10 @@ func TestACL_Legacy_List(t *testing.T) {
 }
 
 func TestACLReplicationStatus(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := NewTestAgent(t, TestACLConfig())
 	defer a.Shutdown()
