@@ -27,6 +27,10 @@ import (
 //       functionality as that will be done with other tests.
 
 func TestACL_Disabled_Response(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := NewTestAgent(t, "")
 	defer a.Shutdown()
@@ -84,6 +88,10 @@ func jsonBody(v interface{}) io.Reader {
 }
 
 func TestACL_Bootstrap(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := NewTestAgent(t, TestACLConfig()+`
       acl_master_token = ""
@@ -132,6 +140,10 @@ func TestACL_Bootstrap(t *testing.T) {
 }
 
 func TestACL_HTTP(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := NewTestAgent(t, TestACLConfig())
 	defer a.Shutdown()
@@ -847,6 +859,7 @@ func TestACL_HTTP(t *testing.T) {
 				found := false
 				for _, actual := range tokens {
 					if actual.AccessorID == tokenID {
+						require.Equal(t, expected.SecretID, actual.SecretID)
 						require.Equal(t, expected.Description, actual.Description)
 						require.Equal(t, expected.Policies, actual.Policies)
 						require.Equal(t, expected.Local, actual.Local)
@@ -1124,6 +1137,10 @@ func TestACL_HTTP(t *testing.T) {
 }
 
 func TestACL_LoginProcedure_HTTP(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	// This tests AuthMethods, BindingRules, Login, and Logout.
 	t.Parallel()
 	a := NewTestAgent(t, TestACLConfig())
@@ -1185,6 +1202,8 @@ func TestACL_LoginProcedure_HTTP(t *testing.T) {
 				Config: map[string]interface{}{
 					"SessionID": testSessionID,
 				},
+				TokenLocality: "global",
+				MaxTokenTTL:   500_000_000_000,
 			}
 
 			req, _ := http.NewRequest("PUT", "/v1/acl/auth-method?token=root", jsonBody(methodInput))
@@ -1288,6 +1307,7 @@ func TestACL_LoginProcedure_HTTP(t *testing.T) {
 			resp := httptest.NewRecorder()
 			raw, err := a.srv.ACLAuthMethodList(resp, req)
 			require.NoError(t, err)
+
 			methods, ok := raw.(structs.ACLAuthMethodListStubs)
 			require.True(t, ok)
 
@@ -1301,6 +1321,8 @@ func TestACL_LoginProcedure_HTTP(t *testing.T) {
 						require.Equal(t, expected.Name, actual.Name)
 						require.Equal(t, expected.Type, actual.Type)
 						require.Equal(t, expected.Description, actual.Description)
+						require.Equal(t, expected.MaxTokenTTL, actual.MaxTokenTTL)
+						require.Equal(t, expected.TokenLocality, actual.TokenLocality)
 						require.Equal(t, expected.CreateIndex, actual.CreateIndex)
 						require.Equal(t, expected.ModifyIndex, actual.ModifyIndex)
 						found = true
@@ -1660,6 +1682,10 @@ func TestACL_LoginProcedure_HTTP(t *testing.T) {
 }
 
 func TestACLEndpoint_LoginLogout_jwt(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := NewTestAgent(t, TestACLConfigWithParams(nil))
@@ -1822,6 +1848,10 @@ func TestACLEndpoint_LoginLogout_jwt(t *testing.T) {
 }
 
 func TestACL_Authorize(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a1 := NewTestAgent(t, TestACLConfigWithParams(nil))
 	defer a1.Shutdown()
