@@ -11,6 +11,12 @@ import (
 	"github.com/hashicorp/go-memdb"
 	"github.com/mitchellh/hashstructure"
 
+	"github.com/hashicorp/go-bexpr"
+	"github.com/hashicorp/serf/coordinate"
+	"github.com/hashicorp/serf/serf"
+	"github.com/prometheus/client_golang/prometheus"
+	"github.com/prometheus/client_golang/prometheus/promhttp"
+
 	"github.com/hashicorp/consul/acl"
 	cachetype "github.com/hashicorp/consul/agent/cache-types"
 	"github.com/hashicorp/consul/agent/debug"
@@ -23,11 +29,6 @@ import (
 	"github.com/hashicorp/consul/logging"
 	"github.com/hashicorp/consul/logging/monitor"
 	"github.com/hashicorp/consul/types"
-	"github.com/hashicorp/go-bexpr"
-	"github.com/hashicorp/serf/coordinate"
-	"github.com/hashicorp/serf/serf"
-	"github.com/prometheus/client_golang/prometheus"
-	"github.com/prometheus/client_golang/prometheus/promhttp"
 )
 
 type Self struct {
@@ -74,19 +75,21 @@ func (s *HTTPHandlers) AgentSelf(resp http.ResponseWriter, req *http.Request) (i
 	}
 
 	config := struct {
-		Datacenter string
-		NodeName   string
-		NodeID     string
-		Revision   string
-		Server     bool
-		Version    string
+		Datacenter        string
+		PrimaryDatacenter string
+		NodeName          string
+		NodeID            string
+		Revision          string
+		Server            bool
+		Version           string
 	}{
-		Datacenter: s.agent.config.Datacenter,
-		NodeName:   s.agent.config.NodeName,
-		NodeID:     string(s.agent.config.NodeID),
-		Revision:   s.agent.config.Revision,
-		Server:     s.agent.config.ServerMode,
-		Version:    s.agent.config.Version,
+		Datacenter:        s.agent.config.Datacenter,
+		PrimaryDatacenter: s.agent.config.PrimaryDatacenter,
+		NodeName:          s.agent.config.NodeName,
+		NodeID:            string(s.agent.config.NodeID),
+		Revision:          s.agent.config.Revision,
+		Server:            s.agent.config.ServerMode,
+		Version:           s.agent.config.Version,
 	}
 	return Self{
 		Config:      config,
@@ -196,6 +199,7 @@ func buildAgentService(s *structs.NodeService, dc string) api.AgentService {
 		Meta:              s.Meta,
 		Port:              s.Port,
 		Address:           s.Address,
+		SocketPath:        s.SocketPath,
 		TaggedAddresses:   taggedAddrs,
 		EnableTagOverride: s.EnableTagOverride,
 		CreateIndex:       s.CreateIndex,
@@ -277,10 +281,7 @@ func (s *HTTPHandlers) AgentServices(resp http.ResponseWriter, req *http.Request
 // blocking watch using hash-based blocking.
 func (s *HTTPHandlers) AgentService(resp http.ResponseWriter, req *http.Request) (interface{}, error) {
 	// Get the proxy ID. Note that this is the ID of a proxy's service instance.
-	id, err := getPathSuffixUnescaped(req.URL.Path, "/v1/agent/service/")
-	if err != nil {
-		return nil, err
-	}
+	id := strings.TrimPrefix(req.URL.Path, "/v1/agent/service/")
 
 	// Maybe block
 	var queryOpts structs.QueryOptions
@@ -299,7 +300,7 @@ func (s *HTTPHandlers) AgentService(resp http.ResponseWriter, req *http.Request)
 	}
 
 	// need to resolve to default the meta
-	_, err = s.agent.delegate.ResolveTokenAndDefaultMeta(token, &entMeta, nil)
+	_, err := s.agent.delegate.ResolveTokenAndDefaultMeta(token, &entMeta, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -466,10 +467,7 @@ func (s *HTTPHandlers) AgentJoin(resp http.ResponseWriter, req *http.Request) (i
 	}
 
 	// Get the address
-	addr, err := getPathSuffixUnescaped(req.URL.Path, "/v1/agent/join/")
-	if err != nil {
-		return nil, err
-	}
+	addr := strings.TrimPrefix(req.URL.Path, "/v1/agent/join/")
 
 	if wan {
 		if s.agent.config.ConnectMeshGatewayWANFederationEnabled {
@@ -515,10 +513,7 @@ func (s *HTTPHandlers) AgentForceLeave(resp http.ResponseWriter, req *http.Reque
 	//Check the value of the prune query
 	_, prune := req.URL.Query()["prune"]
 
-	addr, err := getPathSuffixUnescaped(req.URL.Path, "/v1/agent/force-leave/")
-	if err != nil {
-		return nil, err
-	}
+	addr := strings.TrimPrefix(req.URL.Path, "/v1/agent/force-leave/")
 	return nil, s.agent.ForceLeave(addr, prune)
 }
 
@@ -601,11 +596,7 @@ func (s *HTTPHandlers) AgentRegisterCheck(resp http.ResponseWriter, req *http.Re
 }
 
 func (s *HTTPHandlers) AgentDeregisterCheck(resp http.ResponseWriter, req *http.Request) (interface{}, error) {
-	ID, err := getPathSuffixUnescaped(req.URL.Path, "/v1/agent/check/deregister/")
-	if err != nil {
-		return nil, err
-	}
-	checkID := structs.NewCheckID(types.CheckID(ID), nil)
+	checkID := structs.NewCheckID(types.CheckID(strings.TrimPrefix(req.URL.Path, "/v1/agent/check/deregister/")), nil)
 
 	// Get the provided token, if any, and vet against any ACL policies.
 	var token string
@@ -634,21 +625,13 @@ func (s *HTTPHandlers) AgentDeregisterCheck(resp http.ResponseWriter, req *http.
 }
 
 func (s *HTTPHandlers) AgentCheckPass(resp http.ResponseWriter, req *http.Request) (interface{}, error) {
-	ID, err := getPathSuffixUnescaped(req.URL.Path, "/v1/agent/check/pass/")
-	if err != nil {
-		return nil, err
-	}
-	checkID := types.CheckID(ID)
+	checkID := types.CheckID(strings.TrimPrefix(req.URL.Path, "/v1/agent/check/pass/"))
 	note := req.URL.Query().Get("note")
 	return s.agentCheckUpdate(resp, req, checkID, api.HealthPassing, note)
 }
 
 func (s *HTTPHandlers) AgentCheckWarn(resp http.ResponseWriter, req *http.Request) (interface{}, error) {
-	ID, err := getPathSuffixUnescaped(req.URL.Path, "/v1/agent/check/warn/")
-	if err != nil {
-		return nil, err
-	}
-	checkID := types.CheckID(ID)
+	checkID := types.CheckID(strings.TrimPrefix(req.URL.Path, "/v1/agent/check/warn/"))
 	note := req.URL.Query().Get("note")
 
 	return s.agentCheckUpdate(resp, req, checkID, api.HealthWarning, note)
@@ -656,11 +639,7 @@ func (s *HTTPHandlers) AgentCheckWarn(resp http.ResponseWriter, req *http.Reques
 }
 
 func (s *HTTPHandlers) AgentCheckFail(resp http.ResponseWriter, req *http.Request) (interface{}, error) {
-	ID, err := getPathSuffixUnescaped(req.URL.Path, "/v1/agent/check/fail/")
-	if err != nil {
-		return nil, err
-	}
-	checkID := types.CheckID(ID)
+	checkID := types.CheckID(strings.TrimPrefix(req.URL.Path, "/v1/agent/check/fail/"))
 	note := req.URL.Query().Get("note")
 
 	return s.agentCheckUpdate(resp, req, checkID, api.HealthCritical, note)
@@ -699,12 +678,7 @@ func (s *HTTPHandlers) AgentCheckUpdate(resp http.ResponseWriter, req *http.Requ
 		return nil, nil
 	}
 
-	ID, err := getPathSuffixUnescaped(req.URL.Path, "/v1/agent/check/update/")
-	if err != nil {
-		return nil, err
-	}
-
-	checkID := types.CheckID(ID)
+	checkID := types.CheckID(strings.TrimPrefix(req.URL.Path, "/v1/agent/check/update/"))
 
 	return s.agentCheckUpdate(resp, req, checkID, update.Status, update.Output)
 }
@@ -782,10 +756,7 @@ func returnTextPlain(req *http.Request) bool {
 // AgentHealthServiceByID return the local Service Health given its ID
 func (s *HTTPHandlers) AgentHealthServiceByID(resp http.ResponseWriter, req *http.Request) (interface{}, error) {
 	// Pull out the service id (service id since there may be several instance of the same service on this host)
-	serviceID, err := getPathSuffixUnescaped(req.URL.Path, "/v1/agent/health/service/id/")
-	if err != nil {
-		return nil, err
-	}
+	serviceID := strings.TrimPrefix(req.URL.Path, "/v1/agent/health/service/id/")
 	if serviceID == "" {
 		return nil, &BadRequestError{Reason: "Missing serviceID"}
 	}
@@ -839,11 +810,7 @@ func (s *HTTPHandlers) AgentHealthServiceByID(resp http.ResponseWriter, req *htt
 // AgentHealthServiceByName return the worse status of all the services with given name on an agent
 func (s *HTTPHandlers) AgentHealthServiceByName(resp http.ResponseWriter, req *http.Request) (interface{}, error) {
 	// Pull out the service name
-	serviceName, err := getPathSuffixUnescaped(req.URL.Path, "/v1/agent/health/service/name/")
-	if err != nil {
-		return nil, err
-	}
-
+	serviceName := strings.TrimPrefix(req.URL.Path, "/v1/agent/health/service/name/")
 	if serviceName == "" {
 		return nil, &BadRequestError{Reason: "Missing service Name"}
 	}
@@ -1029,25 +996,29 @@ func (s *HTTPHandlers) AgentRegisterService(resp http.ResponseWriter, req *http.
 		replaceExistingChecks = true
 	}
 
-	if replaceExistingChecks {
-		if err := s.agent.AddServiceAndReplaceChecks(ns, chkTypes, true, token, ConfigSourceRemote); err != nil {
-			return nil, err
-		}
-	} else {
-		if err := s.agent.AddService(ns, chkTypes, true, token, ConfigSourceRemote); err != nil {
-			return nil, err
-		}
+	addReq := AddServiceRequest{
+		Service:               ns,
+		chkTypes:              chkTypes,
+		persist:               true,
+		token:                 token,
+		Source:                ConfigSourceRemote,
+		replaceExistingChecks: replaceExistingChecks,
 	}
-	// Add sidecar.
+	if err := s.agent.AddService(addReq); err != nil {
+		return nil, err
+	}
+
 	if sidecar != nil {
-		if replaceExistingChecks {
-			if err := s.agent.AddServiceAndReplaceChecks(sidecar, sidecarChecks, true, sidecarToken, ConfigSourceRemote); err != nil {
-				return nil, err
-			}
-		} else {
-			if err := s.agent.AddService(sidecar, sidecarChecks, true, sidecarToken, ConfigSourceRemote); err != nil {
-				return nil, err
-			}
+		addReq := AddServiceRequest{
+			Service:               sidecar,
+			chkTypes:              sidecarChecks,
+			persist:               true,
+			token:                 sidecarToken,
+			Source:                ConfigSourceRemote,
+			replaceExistingChecks: replaceExistingChecks,
+		}
+		if err := s.agent.AddService(addReq); err != nil {
+			return nil, err
 		}
 	}
 	s.syncChanges()
@@ -1055,12 +1026,7 @@ func (s *HTTPHandlers) AgentRegisterService(resp http.ResponseWriter, req *http.
 }
 
 func (s *HTTPHandlers) AgentDeregisterService(resp http.ResponseWriter, req *http.Request) (interface{}, error) {
-	serviceID, err := getPathSuffixUnescaped(req.URL.Path, "/v1/agent/service/deregister/")
-	if err != nil {
-		return nil, err
-	}
-
-	sid := structs.NewServiceID(serviceID, nil)
+	sid := structs.NewServiceID(strings.TrimPrefix(req.URL.Path, "/v1/agent/service/deregister/"), nil)
 
 	// Get the provided token, if any, and vet against any ACL policies.
 	var token string
@@ -1091,12 +1057,7 @@ func (s *HTTPHandlers) AgentDeregisterService(resp http.ResponseWriter, req *htt
 
 func (s *HTTPHandlers) AgentServiceMaintenance(resp http.ResponseWriter, req *http.Request) (interface{}, error) {
 	// Ensure we have a service ID
-	serviceID, err := getPathSuffixUnescaped(req.URL.Path, "/v1/agent/service/maintenance/")
-	if err != nil {
-		return nil, err
-	}
-
-	sid := structs.NewServiceID(serviceID, nil)
+	sid := structs.NewServiceID(strings.TrimPrefix(req.URL.Path, "/v1/agent/service/maintenance/"), nil)
 
 	if sid.ID == "" {
 		resp.WriteHeader(http.StatusBadRequest)
@@ -1295,10 +1256,7 @@ func (s *HTTPHandlers) AgentToken(resp http.ResponseWriter, req *http.Request) (
 	}
 
 	// Figure out the target token.
-	target, err := getPathSuffixUnescaped(req.URL.Path, "/v1/agent/token/")
-	if err != nil {
-		return nil, err
-	}
+	target := strings.TrimPrefix(req.URL.Path, "/v1/agent/token/")
 
 	err = s.agent.tokens.WithPersistenceLock(func() error {
 		triggerAntiEntropySync := false
@@ -1371,10 +1329,7 @@ func (s *HTTPHandlers) AgentConnectCARoots(resp http.ResponseWriter, req *http.R
 func (s *HTTPHandlers) AgentConnectCALeafCert(resp http.ResponseWriter, req *http.Request) (interface{}, error) {
 	// Get the service name. Note that this is the name of the service,
 	// not the ID of the service instance.
-	serviceName, err := getPathSuffixUnescaped(req.URL.Path, "/v1/agent/connect/ca/leaf/")
-	if err != nil {
-		return nil, err
-	}
+	serviceName := strings.TrimPrefix(req.URL.Path, "/v1/agent/connect/ca/leaf/")
 
 	args := cachetype.ConnectCALeafRequest{
 		Service: serviceName, // Need name not ID
