@@ -92,6 +92,10 @@ func New(config Config) (*AutoConfig, error) {
 		}
 	}
 
+	if err := config.EnterpriseConfig.validateAndFinalize(); err != nil {
+		return nil, err
+	}
+
 	return &AutoConfig{
 		acConfig: config,
 		logger:   logger,
@@ -103,17 +107,17 @@ func New(config Config) (*AutoConfig, error) {
 func (ac *AutoConfig) ReadConfig() (*config.RuntimeConfig, error) {
 	ac.Lock()
 	defer ac.Unlock()
-	cfg, warnings, err := ac.acConfig.Loader(ac.autoConfigSource)
+	result, err := ac.acConfig.Loader(ac.autoConfigSource)
 	if err != nil {
-		return cfg, err
+		return result.RuntimeConfig, err
 	}
 
-	for _, w := range warnings {
+	for _, w := range result.Warnings {
 		ac.logger.Warn(w)
 	}
 
-	ac.config = cfg
-	return cfg, nil
+	ac.config = result.RuntimeConfig
+	return ac.config, nil
 }
 
 // InitialConfiguration will perform a one-time RPC request to the configured servers
@@ -126,13 +130,8 @@ func (ac *AutoConfig) ReadConfig() (*config.RuntimeConfig, error) {
 // The context passed in can be used to cancel the retrieval of the initial configuration
 // like when receiving a signal during startup.
 func (ac *AutoConfig) InitialConfiguration(ctx context.Context) (*config.RuntimeConfig, error) {
-	if ac.config == nil {
-		config, err := ac.ReadConfig()
-		if err != nil {
-			return nil, err
-		}
-
-		ac.config = config
+	if err := ac.maybeLoadConfig(); err != nil {
+		return nil, err
 	}
 
 	switch {
@@ -180,6 +179,23 @@ func (ac *AutoConfig) InitialConfiguration(ctx context.Context) (*config.Runtime
 	}
 }
 
+// maybeLoadConfig will read the Consul configuration using the
+// provided config loader if and only if the config field of
+// the struct is nil. When it does this it will fill in that
+// field. If the config field already is non-nil then this
+// is a noop.
+func (ac *AutoConfig) maybeLoadConfig() error {
+	if ac.config == nil {
+		config, err := ac.ReadConfig()
+		if err != nil {
+			return err
+		}
+
+		ac.config = config
+	}
+	return nil
+}
+
 // introToken is responsible for determining the correct intro token to use
 // when making the initial AutoConfig.InitialConfiguration RPC request.
 func (ac *AutoConfig) introToken() (string, error) {
