@@ -182,7 +182,7 @@ func TestAutoEncrypt_hosts(t *testing.T) {
 				},
 			}
 
-			hosts, err := ac.autoEncryptHosts()
+			hosts, err := ac.joinHosts()
 			if tcase.err != "" {
 				testutil.RequireErrorContains(t, err, tcase.err)
 			} else {
@@ -287,7 +287,7 @@ func TestAutoEncrypt_InitialConfiguration(t *testing.T) {
 			tls = true
 		}
 	`)
-	loader.opts.Config.NodeName = &nodeName
+	loader.opts.FlagValues.NodeName = &nodeName
 	mcfg.Config.Loader = loader.Load
 
 	indexedRoots, cert, extraCerts := mcfg.setupInitialTLS(t, nodeName, datacenter, token)
