@@ -10,6 +10,7 @@ import (
 	"testing"
 	"time"
 
+	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
 
@@ -17,25 +18,28 @@ func TestLoad(t *testing.T) {
 	// Basically just testing that injection of the extra
 	// source works.
 	devMode := true
-	builderOpts := BuilderOpts{
+	builderOpts := LoadOpts{
 		// putting this in dev mode so that the config validates
 		// without having to specify a data directory
 		DevMode: &devMode,
+		DefaultConfig: FileSource{
+			Name:   "test",
+			Format: "hcl",
+			Data:   `node_name = "hobbiton"`,
+		},
+		Overrides: []Source{
+			FileSource{
+				Name:   "overrides",
+				Format: "json",
+				Data:   `{"check_reap_interval": "1ms"}`,
+			},
+		},
 	}
 
-	cfg, warnings, err := Load(builderOpts, FileSource{
-		Name:   "test",
-		Format: "hcl",
-		Data:   `node_name = "hobbiton"`,
-	},
-		FileSource{
-			Name:   "overrides",
-			Format: "json",
-			Data:   `{"check_reap_interval": "1ms"}`,
-		})
-
+	result, err := Load(builderOpts)
 	require.NoError(t, err)
-	require.Empty(t, warnings)
+	require.Empty(t, result.Warnings)
+	cfg := result.RuntimeConfig
 	require.NotNil(t, cfg)
 	require.Equal(t, "hobbiton", cfg.NodeName)
 	require.Equal(t, 1*time.Millisecond, cfg.CheckReapInterval)
@@ -65,7 +69,7 @@ func TestShouldParseFile(t *testing.T) {
 func TestNewBuilder_PopulatesSourcesFromConfigFiles(t *testing.T) {
 	paths := setupConfigFiles(t)
 
-	b, err := NewBuilder(BuilderOpts{ConfigFiles: paths})
+	b, err := newBuilder(LoadOpts{ConfigFiles: paths})
 	require.NoError(t, err)
 
 	expected := []Source{
@@ -81,7 +85,7 @@ func TestNewBuilder_PopulatesSourcesFromConfigFiles(t *testing.T) {
 func TestNewBuilder_PopulatesSourcesFromConfigFiles_WithConfigFormat(t *testing.T) {
 	paths := setupConfigFiles(t)
 
-	b, err := NewBuilder(BuilderOpts{ConfigFiles: paths, ConfigFormat: "hcl"})
+	b, err := newBuilder(LoadOpts{ConfigFiles: paths, ConfigFormat: "hcl"})
 	require.NoError(t, err)
 
 	expected := []Source{
@@ -132,13 +136,13 @@ func TestBuilder_BuildAndValidate_NodeName(t *testing.T) {
 	}
 
 	fn := func(t *testing.T, tc testCase) {
-		b, err := NewBuilder(BuilderOpts{
-			Config: Config{
+		b, err := newBuilder(LoadOpts{
+			FlagValues: Config{
 				NodeName: pString(tc.nodeName),
 				DataDir:  pString("dir"),
 			},
 		})
-		patchBuilderShims(b)
+		patchLoadOptsShims(&b.opts)
 		require.NoError(t, err)
 		_, err = b.BuildAndValidate()
 		require.NoError(t, err)
@@ -171,20 +175,82 @@ func TestBuilder_BuildAndValidate_NodeName(t *testing.T) {
 	}
 }
 
-func patchBuilderShims(b *Builder) {
-	b.opts.hostname = func() (string, error) {
-		return "thehostname", nil
+func TestBuilder_unixPermissionsVal(t *testing.T) {
+
+	b, _ := newBuilder(LoadOpts{
+		FlagValues: Config{
+			NodeName: pString("foo"),
+			DataDir:  pString("dir"),
+		},
+	})
+
+	goodmode := "666"
+	badmode := "9666"
+
+	patchLoadOptsShims(&b.opts)
+	require.NoError(t, b.err)
+	_ = b.unixPermissionsVal("local_bind_socket_mode", &goodmode)
+	require.NoError(t, b.err)
+	require.Len(t, b.Warnings, 0)
+
+	_ = b.unixPermissionsVal("local_bind_socket_mode", &badmode)
+	require.NotNil(t, b.err)
+	require.Contains(t, b.err.Error(), "local_bind_socket_mode: invalid mode")
+	require.Len(t, b.Warnings, 0)
+}
+
+func patchLoadOptsShims(opts *LoadOpts) {
+	if opts.hostname == nil {
+		opts.hostname = func() (string, error) {
+			return "thehostname", nil
+		}
 	}
-	b.opts.getPrivateIPv4 = func() ([]*net.IPAddr, error) {
-		return []*net.IPAddr{ipAddr("10.0.0.1")}, nil
+	if opts.getPrivateIPv4 == nil {
+		opts.getPrivateIPv4 = func() ([]*net.IPAddr, error) {
+			return []*net.IPAddr{ipAddr("10.0.0.1")}, nil
+		}
 	}
-	b.opts.getPublicIPv6 = func() ([]*net.IPAddr, error) {
-		return []*net.IPAddr{ipAddr("dead:beef::1")}, nil
+	if opts.getPublicIPv6 == nil {
+		opts.getPublicIPv6 = func() ([]*net.IPAddr, error) {
+			return []*net.IPAddr{ipAddr("dead:beef::1")}, nil
+		}
 	}
 }
 
+func TestLoad_HTTPMaxConnsPerClientExceedsRLimit(t *testing.T) {
+	hcl := `
+		limits{
+			# We put a very high value to be sure to fail
+			# This value is more than max on Windows as well
+			http_max_conns_per_client = 16777217
+		}`
+
+	opts := LoadOpts{
+		DefaultConfig: FileSource{
+			Name:   "test",
+			Format: "hcl",
+			Data: `
+		    ae_interval = "1m"
+		    data_dir="/tmp/00000000001979"
+			bind_addr = "127.0.0.1"
+			advertise_addr = "127.0.0.1"
+			datacenter = "dc1"
+			bootstrap = true
+			server = true
+			node_id = "00000000001979"
+			node_name = "Node-00000000001979"
+		`,
+		},
+		HCL: []string{hcl},
+	}
+
+	_, err := Load(opts)
+	require.Error(t, err)
+	assert.Contains(t, err.Error(), "but limits.http_max_conns_per_client: 16777217 needs at least 16777237")
+}
+
 func TestBuilder_DurationVal_InvalidDuration(t *testing.T) {
-	b := Builder{}
+	b := builder{}
 	badDuration1 := "not-a-duration"
 	badDuration2 := "also-not"
 	b.durationVal("field1", &badDuration1)
@@ -197,20 +263,22 @@ func TestBuilder_DurationVal_InvalidDuration(t *testing.T) {
 }
 
 func TestBuilder_ServiceVal_MultiError(t *testing.T) {
-	b := Builder{}
+	b := builder{}
 	b.serviceVal(&ServiceDefinition{
-		Meta: map[string]string{"": "empty-key"},
-		Port: intPtr(12345),
+		Meta:       map[string]string{"": "empty-key"},
+		Port:       intPtr(12345),
+		SocketPath: strPtr("/var/run/socket.sock"),
 		Checks: []CheckDefinition{
 			{Interval: strPtr("bad-interval")},
 		},
 		Weights: &ServiceWeights{Passing: intPtr(-1)},
 	})
 	require.Error(t, b.err)
-	require.Contains(t, b.err.Error(), "3 errors")
+	require.Contains(t, b.err.Error(), "4 errors")
 	require.Contains(t, b.err.Error(), "bad-interval")
 	require.Contains(t, b.err.Error(), "Key cannot be blank")
 	require.Contains(t, b.err.Error(), "Invalid weight")
+	require.Contains(t, b.err.Error(), "cannot have both socket path")
 }
 
 func intPtr(v int) *int {
