@@ -471,6 +471,7 @@ type RuntimeConfig struct {
 	//     method = string
 	//     disable_redirects = (true|false)
 	//     tcp = string
+	//     h2ping = string
 	//     interval = string
 	//     docker_container_id = string
 	//     shell = string
@@ -922,8 +923,8 @@ type RuntimeConfig struct {
 
 	// RPCRateLimit and RPCMaxBurst control how frequently RPC calls are allowed
 	// to happen. In any large enough time interval, rate limiter limits the
-	// rate to RPCRate tokens per second, with a maximum burst size of
-	// RPCMaxBurst events. As a special case, if RPCRate == Inf (the infinite
+	// rate to RPCRateLimit tokens per second, with a maximum burst size of
+	// RPCMaxBurst events. As a special case, if RPCRateLimit == Inf (the infinite
 	// rate), RPCMaxBurst is ignored.
 	//
 	// See https://en.wikipedia.org/wiki/Token_bucket for more about token
@@ -1661,7 +1662,6 @@ func (c *RuntimeConfig) ConnectCAConfiguration() (*structs.CAConfiguration, erro
 	ca := &structs.CAConfiguration{
 		Provider: "consul",
 		Config: map[string]interface{}{
-			"RotationPeriod":      structs.DefaultCARotationPeriod,
 			"LeafCertTTL":         structs.DefaultLeafCertTTL,
 			"IntermediateCertTTL": structs.DefaultIntermediateCertTTL,
 		},
@@ -1921,3 +1921,16 @@ func isFloat(t reflect.Type) bool { return t.Kind() == reflect.Float32 || t.Kind
 func isComplex(t reflect.Type) bool {
 	return t.Kind() == reflect.Complex64 || t.Kind() == reflect.Complex128
 }
+
+// ApplyDefaultQueryOptions returns a function which will set default values on
+// the options based on the configuration. The RuntimeConfig must not be nil.
+func ApplyDefaultQueryOptions(config *RuntimeConfig) func(options *structs.QueryOptions) {
+	return func(options *structs.QueryOptions) {
+		switch {
+		case options.MaxQueryTime > config.MaxQueryTime:
+			options.MaxQueryTime = config.MaxQueryTime
+		case options.MaxQueryTime == 0:
+			options.MaxQueryTime = config.DefaultQueryTime
+		}
+	}
+}
