@@ -11,8 +11,6 @@ var enterpriseReadReplicaWarnings = []string{enterpriseConfigKeyError{key: "read
 var enterpriseConfigKeyWarnings = []string{
 	enterpriseConfigKeyError{key: "license_path"}.Error(),
 	enterpriseConfigKeyError{key: "read_replica (or the deprecated non_voting_server)"}.Error(),
-	enterpriseConfigKeyError{key: "segment"}.Error(),
-	enterpriseConfigKeyError{key: "segments"}.Error(),
 	enterpriseConfigKeyError{key: "autopilot.redundancy_zone_tag"}.Error(),
 	enterpriseConfigKeyError{key: "autopilot.upgrade_version_tag"}.Error(),
 	enterpriseConfigKeyError{key: "autopilot.disable_upgrade_migration"}.Error(),
