@@ -119,6 +119,10 @@ func TestConsulCAProvider_Bootstrap_WithCert(t *testing.T) {
 }
 
 func TestConsulCAProvider_SignLeaf(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	for _, tc := range KeyTestCases {
@@ -228,6 +232,10 @@ func TestConsulCAProvider_SignLeaf(t *testing.T) {
 }
 
 func TestConsulCAProvider_CrossSignCA(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	tests := CASigningKeyTypeCases()
@@ -349,6 +357,10 @@ func testCrossSignProviders(t *testing.T, provider1, provider2 Provider) {
 }
 
 func TestConsulProvider_SignIntermediate(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	tests := CASigningKeyTypeCases()
