@@ -3,8 +3,6 @@ package connect
 import (
 	"fmt"
 	"net/url"
-
-	"github.com/hashicorp/consul/agent/structs"
 )
 
 // SpiffeIDService is the structure to represent the SPIFFE ID for an agent.
@@ -23,11 +21,6 @@ func (id *SpiffeIDAgent) URI() *url.URL {
 	return &result
 }
 
-// CertURI impl.
-func (id *SpiffeIDAgent) Authorize(_ *structs.Intention) (bool, bool) {
-	return false, false
-}
-
 func (id *SpiffeIDAgent) CommonName() string {
 	return AgentCN(id.Agent, id.Host)
 }
