@@ -292,6 +292,10 @@ func TestACLReplication_diffACLTokens(t *testing.T) {
 }
 
 func TestACLReplication_Tokens(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
 		c.ACLDatacenter = "dc1"
@@ -505,6 +509,10 @@ func TestACLReplication_Tokens(t *testing.T) {
 }
 
 func TestACLReplication_Policies(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
 		c.ACLDatacenter = "dc1"
@@ -626,6 +634,10 @@ func TestACLReplication_Policies(t *testing.T) {
 }
 
 func TestACLReplication_TokensRedacted(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
 		c.ACLDatacenter = "dc1"
@@ -772,6 +784,10 @@ func TestACLReplication_TokensRedacted(t *testing.T) {
 }
 
 func TestACLReplication_AllTypes(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
 		c.ACLDatacenter = "dc1"
