@@ -5,13 +5,15 @@ import (
 	"path/filepath"
 	"strings"
 
+	"github.com/hashicorp/go-hclog"
+	"github.com/hashicorp/serf/serf"
+
 	"github.com/hashicorp/consul/agent/metadata"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/lib"
+	libserf "github.com/hashicorp/consul/lib/serf"
 	"github.com/hashicorp/consul/logging"
 	"github.com/hashicorp/consul/types"
-	"github.com/hashicorp/go-hclog"
-	"github.com/hashicorp/serf/serf"
 )
 
 // setupSerf is used to setup and initialize a Serf
@@ -27,6 +29,9 @@ func (c *Client) setupSerf(conf *serf.Config, ch chan serf.Event, path string) (
 	conf.Tags["vsn_min"] = fmt.Sprintf("%d", ProtocolVersionMin)
 	conf.Tags["vsn_max"] = fmt.Sprintf("%d", ProtocolVersionMax)
 	conf.Tags["build"] = c.config.Build
+	if c.config.AdvertiseReconnectTimeout != 0 {
+		conf.Tags[libserf.ReconnectTimeoutTag] = c.config.AdvertiseReconnectTimeout.String()
+	}
 	if c.acls.ACLsEnabled() {
 		// we start in legacy mode and then transition to normal
 		// mode once we know the cluster can handle it.
@@ -63,7 +68,9 @@ func (c *Client) setupSerf(conf *serf.Config, ch chan serf.Event, path string) (
 		return nil, err
 	}
 
-	c.addEnterpriseSerfTags(conf.Tags)
+	addEnterpriseSerfTags(conf.Tags)
+
+	conf.ReconnectTimeoutOverride = libserf.NewReconnectOverride(c.logger)
 
 	return serf.Create(conf)
 }
