@@ -100,12 +100,7 @@ func (s *Server) reconcileLocalConfig(ctx context.Context, configs []structs.Con
 			Entry:      entry,
 		}
 
-		resp, err := s.raftApply(structs.ConfigEntryRequestType, &req)
-		if err == nil {
-			if respErr, ok := resp.(error); ok {
-				err = respErr
-			}
-		}
+		_, err := s.raftApply(structs.ConfigEntryRequestType, &req)
 		if err != nil {
 			merr = multierror.Append(merr, fmt.Errorf("Failed to apply config entry %s: %w", op, err))
 		}
