@@ -9,15 +9,16 @@ import (
 	"testing"
 	"time"
 
+	msgpackrpc "github.com/hashicorp/net-rpc-msgpackrpc"
+	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
+
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/connect"
 	ca "github.com/hashicorp/consul/agent/connect/ca"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/sdk/testutil/retry"
 	"github.com/hashicorp/consul/testrpc"
-	msgpackrpc "github.com/hashicorp/net-rpc-msgpackrpc"
-	"github.com/stretchr/testify/assert"
-	"github.com/stretchr/testify/require"
 )
 
 func testParseCert(t *testing.T, pemValue string) *x509.Certificate {
@@ -30,6 +31,10 @@ func testParseCert(t *testing.T, pemValue string) *x509.Certificate {
 
 // Test listing root CAs.
 func TestConnectCARoots(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	assert := assert.New(t)
@@ -74,6 +79,10 @@ func TestConnectCARoots(t *testing.T) {
 }
 
 func TestConnectCAConfig_GetSet(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	assert := assert.New(t)
@@ -107,9 +116,8 @@ func TestConnectCAConfig_GetSet(t *testing.T) {
 	newConfig := &structs.CAConfiguration{
 		Provider: "consul",
 		Config: map[string]interface{}{
-			"PrivateKey":     "",
-			"RootCert":       "",
-			"RotationPeriod": 180 * 24 * time.Hour,
+			"PrivateKey": "",
+			"RootCert":   "",
 			// This verifies the state persistence for providers although Consul
 			// provider doesn't actually use that mechanism outside of tests.
 			"test_state": testState,
@@ -145,6 +153,10 @@ func TestConnectCAConfig_GetSet(t *testing.T) {
 }
 
 func TestConnectCAConfig_GetSet_ACLDeny(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
@@ -235,6 +247,10 @@ pY0heYeK9A6iOLrzqxSerkXXQyj5e9bE4VgUnxgPU6g=
 // signing works when requested (and is denied when not requested). This occurs
 // if the current CA is not able to cross sign external CA certificates.
 func TestConnectCAConfig_GetSetForceNoCrossSigning(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	require := require.New(t)
@@ -338,6 +354,10 @@ func TestConnectCAConfig_GetSetForceNoCrossSigning(t *testing.T) {
 }
 
 func TestConnectCAConfig_TriggerRotation(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	assert := assert.New(t)
@@ -366,9 +386,8 @@ func TestConnectCAConfig_TriggerRotation(t *testing.T) {
 	newConfig := &structs.CAConfiguration{
 		Provider: "consul",
 		Config: map[string]interface{}{
-			"PrivateKey":     newKey,
-			"RootCert":       "",
-			"RotationPeriod": 90 * 24 * time.Hour,
+			"PrivateKey": newKey,
+			"RootCert":   "",
 		},
 	}
 	{
@@ -493,6 +512,10 @@ func TestConnectCAConfig_TriggerRotation(t *testing.T) {
 }
 
 func TestConnectCAConfig_UpdateSecondary(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	assert := assert.New(t)
@@ -544,9 +567,8 @@ func TestConnectCAConfig_UpdateSecondary(t *testing.T) {
 	newConfig := &structs.CAConfiguration{
 		Provider: "consul",
 		Config: map[string]interface{}{
-			"PrivateKey":     newKey,
-			"RootCert":       "",
-			"RotationPeriod": 90 * 24 * time.Hour,
+			"PrivateKey": newKey,
+			"RootCert":   "",
 		},
 	}
 	{
@@ -630,9 +652,8 @@ func TestConnectCAConfig_UpdateSecondary(t *testing.T) {
 		newConfig := &structs.CAConfiguration{
 			Provider: "consul",
 			Config: map[string]interface{}{
-				"PrivateKey":     newKey,
-				"RootCert":       "",
-				"RotationPeriod": 180 * 24 * time.Hour,
+				"PrivateKey": newKey,
+				"RootCert":   "",
 			},
 		}
 		{
@@ -649,6 +670,10 @@ func TestConnectCAConfig_UpdateSecondary(t *testing.T) {
 
 // Test CA signing
 func TestConnectCASign(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	tests := []struct {
@@ -751,6 +776,10 @@ func BenchmarkConnectCASign(b *testing.B) {
 }
 
 func TestConnectCASign_rateLimit(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	require := require.New(t)
@@ -812,6 +841,10 @@ func TestConnectCASign_rateLimit(t *testing.T) {
 }
 
 func TestConnectCASign_concurrencyLimit(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	require := require.New(t)
@@ -916,6 +949,10 @@ func TestConnectCASign_concurrencyLimit(t *testing.T) {
 }
 
 func TestConnectCASignValidation(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
