@@ -4,13 +4,15 @@ import (
 	"testing"
 	"time"
 
+	"github.com/stretchr/testify/require"
+
+	"github.com/hashicorp/consul/agent/configentry"
 	"github.com/hashicorp/consul/agent/connect"
 	"github.com/hashicorp/consul/agent/structs"
-	"github.com/stretchr/testify/require"
 )
 
 type compileTestCase struct {
-	entries *structs.DiscoveryChainConfigEntries
+	entries *configentry.DiscoveryChainSet
 	setup   func(req *CompileRequest)
 	expect  *structs.CompiledDiscoveryChain
 	// expectIsDefault tests behavior of CompiledDiscoveryChain.IsDefault()
@@ -51,6 +53,8 @@ func TestCompile(t *testing.T) {
 		"default resolver with external sni":               testcase_DefaultResolver_ExternalSNI(),
 		"resolver with no entries and inferring defaults":  testcase_DefaultResolver(),
 		"default resolver with proxy defaults":             testcase_DefaultResolver_WithProxyDefaults(),
+		"loadbalancer splitter and resolver":               testcase_LBSplitterAndResolver(),
+		"loadbalancer resolver":                            testcase_LBResolver(),
 		"service redirect to service with default resolver is not a default chain": testcase_RedirectToDefaultResolverIsNotDefaultChain(),
 
 		"all the bells and whistles": testcase_AllBellsAndWhistles(),
@@ -157,7 +161,7 @@ func testcase_JustRouterWithDefaults() compileTestCase {
 		Protocol:  "http",
 		StartNode: "router:main.default",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"router:main.default": &structs.DiscoveryGraphNode{
+			"router:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeRouter,
 				Name: "main.default",
 				Routes: []*structs.DiscoveryRoute{
@@ -167,7 +171,7 @@ func testcase_JustRouterWithDefaults() compileTestCase {
 					},
 				},
 			},
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -209,7 +213,7 @@ func testcase_JustRouterWithNoDestination() compileTestCase {
 		Protocol:  "http",
 		StartNode: "router:main.default",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"router:main.default": &structs.DiscoveryGraphNode{
+			"router:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeRouter,
 				Name: "main.default",
 				Routes: []*structs.DiscoveryRoute{
@@ -229,7 +233,7 @@ func testcase_JustRouterWithNoDestination() compileTestCase {
 					},
 				},
 			},
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -269,7 +273,7 @@ func testcase_RouterWithDefaults_NoSplit_WithResolver() compileTestCase {
 		Protocol:  "http",
 		StartNode: "router:main.default",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"router:main.default": &structs.DiscoveryGraphNode{
+			"router:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeRouter,
 				Name: "main.default",
 				Routes: []*structs.DiscoveryRoute{
@@ -279,7 +283,7 @@ func testcase_RouterWithDefaults_NoSplit_WithResolver() compileTestCase {
 					},
 				},
 			},
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -289,7 +293,10 @@ func testcase_RouterWithDefaults_NoSplit_WithResolver() compileTestCase {
 			},
 		},
 		Targets: map[string]*structs.DiscoveryTarget{
-			"main.default.dc1": newTarget("main", "", "default", "dc1", nil),
+			"main.default.dc1": targetWithConnectTimeout(
+				newTarget("main", "", "default", "dc1", nil),
+				33*time.Second,
+			),
 		},
 	}
 
@@ -320,7 +327,7 @@ func testcase_RouterWithDefaults_WithNoopSplit_DefaultResolver() compileTestCase
 		Protocol:  "http",
 		StartNode: "router:main.default",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"router:main.default": &structs.DiscoveryGraphNode{
+			"router:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeRouter,
 				Name: "main.default",
 				Routes: []*structs.DiscoveryRoute{
@@ -330,7 +337,7 @@ func testcase_RouterWithDefaults_WithNoopSplit_DefaultResolver() compileTestCase
 					},
 				},
 			},
-			"splitter:main.default": &structs.DiscoveryGraphNode{
+			"splitter:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeSplitter,
 				Name: "main.default",
 				Splits: []*structs.DiscoverySplit{
@@ -340,7 +347,7 @@ func testcase_RouterWithDefaults_WithNoopSplit_DefaultResolver() compileTestCase
 					},
 				},
 			},
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -382,7 +389,7 @@ func testcase_NoopSplit_DefaultResolver_ProtocolFromProxyDefaults() compileTestC
 		Protocol:  "http",
 		StartNode: "router:main.default",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"router:main.default": &structs.DiscoveryGraphNode{
+			"router:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeRouter,
 				Name: "main.default",
 				Routes: []*structs.DiscoveryRoute{
@@ -392,7 +399,7 @@ func testcase_NoopSplit_DefaultResolver_ProtocolFromProxyDefaults() compileTestC
 					},
 				},
 			},
-			"splitter:main.default": &structs.DiscoveryGraphNode{
+			"splitter:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeSplitter,
 				Name: "main.default",
 				Splits: []*structs.DiscoverySplit{
@@ -402,7 +409,7 @@ func testcase_NoopSplit_DefaultResolver_ProtocolFromProxyDefaults() compileTestC
 					},
 				},
 			},
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -451,7 +458,7 @@ func testcase_RouterWithDefaults_WithNoopSplit_WithResolver() compileTestCase {
 		Protocol:  "http",
 		StartNode: "router:main.default",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"router:main.default": &structs.DiscoveryGraphNode{
+			"router:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeRouter,
 				Name: "main.default",
 				Routes: []*structs.DiscoveryRoute{
@@ -461,7 +468,7 @@ func testcase_RouterWithDefaults_WithNoopSplit_WithResolver() compileTestCase {
 					},
 				},
 			},
-			"splitter:main.default": &structs.DiscoveryGraphNode{
+			"splitter:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeSplitter,
 				Name: "main.default",
 				Splits: []*structs.DiscoverySplit{
@@ -471,7 +478,7 @@ func testcase_RouterWithDefaults_WithNoopSplit_WithResolver() compileTestCase {
 					},
 				},
 			},
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -481,7 +488,10 @@ func testcase_RouterWithDefaults_WithNoopSplit_WithResolver() compileTestCase {
 			},
 		},
 		Targets: map[string]*structs.DiscoveryTarget{
-			"main.default.dc1": newTarget("main", "", "default", "dc1", nil),
+			"main.default.dc1": targetWithConnectTimeout(
+				newTarget("main", "", "default", "dc1", nil),
+				33*time.Second,
+			),
 		},
 	}
 
@@ -532,7 +542,7 @@ func testcase_RouteBypassesSplit() compileTestCase {
 		Protocol:  "http",
 		StartNode: "router:main.default",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"router:main.default": &structs.DiscoveryGraphNode{
+			"router:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeRouter,
 				Name: "main.default",
 				Routes: []*structs.DiscoveryRoute{
@@ -546,7 +556,7 @@ func testcase_RouteBypassesSplit() compileTestCase {
 					},
 				},
 			},
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -555,7 +565,7 @@ func testcase_RouteBypassesSplit() compileTestCase {
 					Target:         "main.default.dc1",
 				},
 			},
-			"resolver:bypass.other.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:bypass.other.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "bypass.other.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -595,7 +605,7 @@ func testcase_NoopSplit_DefaultResolver() compileTestCase {
 		Protocol:  "http",
 		StartNode: "splitter:main.default",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"splitter:main.default": &structs.DiscoveryGraphNode{
+			"splitter:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeSplitter,
 				Name: "main.default",
 				Splits: []*structs.DiscoverySplit{
@@ -605,7 +615,7 @@ func testcase_NoopSplit_DefaultResolver() compileTestCase {
 					},
 				},
 			},
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -648,7 +658,7 @@ func testcase_NoopSplit_WithResolver() compileTestCase {
 		Protocol:  "http",
 		StartNode: "splitter:main.default",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"splitter:main.default": &structs.DiscoveryGraphNode{
+			"splitter:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeSplitter,
 				Name: "main.default",
 				Splits: []*structs.DiscoverySplit{
@@ -658,7 +668,7 @@ func testcase_NoopSplit_WithResolver() compileTestCase {
 					},
 				},
 			},
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -668,7 +678,10 @@ func testcase_NoopSplit_WithResolver() compileTestCase {
 			},
 		},
 		Targets: map[string]*structs.DiscoveryTarget{
-			"main.default.dc1": newTarget("main", "", "default", "dc1", nil),
+			"main.default.dc1": targetWithConnectTimeout(
+				newTarget("main", "", "default", "dc1", nil),
+				33*time.Second,
+			),
 		},
 	}
 
@@ -708,7 +721,7 @@ func testcase_SubsetSplit() compileTestCase {
 		Protocol:  "http",
 		StartNode: "splitter:main.default",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"splitter:main.default": &structs.DiscoveryGraphNode{
+			"splitter:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeSplitter,
 				Name: "main.default",
 				Splits: []*structs.DiscoverySplit{
@@ -722,7 +735,7 @@ func testcase_SubsetSplit() compileTestCase {
 					},
 				},
 			},
-			"resolver:v2.main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:v2.main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "v2.main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -730,7 +743,7 @@ func testcase_SubsetSplit() compileTestCase {
 					Target:         "v2.main.default.dc1",
 				},
 			},
-			"resolver:v1.main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:v1.main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "v1.main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -777,7 +790,7 @@ func testcase_ServiceSplit() compileTestCase {
 		Protocol:  "http",
 		StartNode: "splitter:main.default",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"splitter:main.default": &structs.DiscoveryGraphNode{
+			"splitter:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeSplitter,
 				Name: "main.default",
 				Splits: []*structs.DiscoverySplit{
@@ -791,7 +804,7 @@ func testcase_ServiceSplit() compileTestCase {
 					},
 				},
 			},
-			"resolver:foo.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:foo.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "foo.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -800,7 +813,7 @@ func testcase_ServiceSplit() compileTestCase {
 					Target:         "foo.default.dc1",
 				},
 			},
-			"resolver:bar.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:bar.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "bar.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -866,7 +879,7 @@ func testcase_SplitBypassesSplit() compileTestCase {
 		Protocol:  "http",
 		StartNode: "splitter:main.default",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"splitter:main.default": &structs.DiscoveryGraphNode{
+			"splitter:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeSplitter,
 				Name: "main.default",
 				Splits: []*structs.DiscoverySplit{
@@ -876,7 +889,7 @@ func testcase_SplitBypassesSplit() compileTestCase {
 					},
 				},
 			},
-			"resolver:bypassed.next.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:bypassed.next.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "bypassed.next.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -913,7 +926,7 @@ func testcase_ServiceRedirect() compileTestCase {
 		Protocol:  "tcp",
 		StartNode: "resolver:other.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:other.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:other.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "other.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -960,7 +973,7 @@ func testcase_ServiceAndSubsetRedirect() compileTestCase {
 		Protocol:  "tcp",
 		StartNode: "resolver:v2.other.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:v2.other.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:v2.other.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "v2.other.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -996,7 +1009,7 @@ func testcase_DatacenterRedirect() compileTestCase {
 		Protocol:  "tcp",
 		StartNode: "resolver:main.default.dc9",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:main.default.dc9": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc9": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc9",
 				Resolver: &structs.DiscoveryResolver{
@@ -1035,7 +1048,7 @@ func testcase_DatacenterRedirect_WithMeshGateways() compileTestCase {
 		Protocol:  "tcp",
 		StartNode: "resolver:main.default.dc9",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:main.default.dc9": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc9": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc9",
 				Resolver: &structs.DiscoveryResolver{
@@ -1071,7 +1084,7 @@ func testcase_ServiceFailover() compileTestCase {
 		Protocol:  "tcp",
 		StartNode: "resolver:main.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -1114,7 +1127,7 @@ func testcase_ServiceFailoverThroughRedirect() compileTestCase {
 		Protocol:  "tcp",
 		StartNode: "resolver:main.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -1157,7 +1170,7 @@ func testcase_Resolver_CircularFailover() compileTestCase {
 		Protocol:  "tcp",
 		StartNode: "resolver:main.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -1198,7 +1211,7 @@ func testcase_ServiceAndSubsetFailover() compileTestCase {
 		Protocol:  "tcp",
 		StartNode: "resolver:main.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -1238,7 +1251,7 @@ func testcase_DatacenterFailover() compileTestCase {
 		Protocol:  "tcp",
 		StartNode: "resolver:main.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -1282,7 +1295,7 @@ func testcase_DatacenterFailover_WithMeshGateways() compileTestCase {
 		Protocol:  "tcp",
 		StartNode: "resolver:main.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -1343,7 +1356,7 @@ func testcase_NoopSplit_WithDefaultSubset() compileTestCase {
 		Protocol:  "http",
 		StartNode: "splitter:main.default",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"splitter:main.default": &structs.DiscoveryGraphNode{
+			"splitter:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeSplitter,
 				Name: "main.default",
 				Splits: []*structs.DiscoverySplit{
@@ -1353,7 +1366,7 @@ func testcase_NoopSplit_WithDefaultSubset() compileTestCase {
 					},
 				},
 			},
-			"resolver:v2.main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:v2.main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "v2.main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -1380,7 +1393,7 @@ func testcase_DefaultResolver() compileTestCase {
 		Protocol:  "tcp",
 		StartNode: "resolver:main.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -1415,7 +1428,7 @@ func testcase_DefaultResolver_WithProxyDefaults() compileTestCase {
 		Protocol:  "grpc",
 		StartNode: "resolver:main.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -1448,7 +1461,7 @@ func testcase_RedirectToDefaultResolverIsNotDefaultChain() compileTestCase {
 		Protocol:  "tcp",
 		StartNode: "resolver:other.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:other.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:other.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "other.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -1484,7 +1497,7 @@ func testcase_Resolve_WithDefaultSubset() compileTestCase {
 		Protocol:  "tcp",
 		StartNode: "resolver:v2.main.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:v2.main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:v2.main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "v2.main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -1516,7 +1529,7 @@ func testcase_DefaultResolver_ExternalSNI() compileTestCase {
 		Protocol:  "tcp",
 		StartNode: "resolver:main.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -1651,7 +1664,7 @@ func testcase_MultiDatacenterCanary() compileTestCase {
 		Protocol:  "http",
 		StartNode: "splitter:main.default",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"splitter:main.default": &structs.DiscoveryGraphNode{
+			"splitter:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeSplitter,
 				Name: "main.default",
 				Splits: []*structs.DiscoverySplit{
@@ -1665,7 +1678,7 @@ func testcase_MultiDatacenterCanary() compileTestCase {
 					},
 				},
 			},
-			"resolver:main.default.dc2": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc2": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc2",
 				Resolver: &structs.DiscoveryResolver{
@@ -1673,7 +1686,7 @@ func testcase_MultiDatacenterCanary() compileTestCase {
 					Target:         "main.default.dc2",
 				},
 			},
-			"resolver:main.default.dc3": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc3": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc3",
 				Resolver: &structs.DiscoveryResolver{
@@ -1683,8 +1696,14 @@ func testcase_MultiDatacenterCanary() compileTestCase {
 			},
 		},
 		Targets: map[string]*structs.DiscoveryTarget{
-			"main.default.dc2": newTarget("main", "", "default", "dc2", nil),
-			"main.default.dc3": newTarget("main", "", "default", "dc3", nil),
+			"main.default.dc2": targetWithConnectTimeout(
+				newTarget("main", "", "default", "dc2", nil),
+				33*time.Second,
+			),
+			"main.default.dc3": targetWithConnectTimeout(
+				newTarget("main", "", "default", "dc3", nil),
+				33*time.Second,
+			),
 		},
 	}
 	return compileTestCase{entries: entries, expect: expect}
@@ -1760,6 +1779,17 @@ func testcase_AllBellsAndWhistles() compileTestCase {
 				"prod": {Filter: "ServiceMeta.env == prod"},
 				"qa":   {Filter: "ServiceMeta.env == qa"},
 			},
+			LoadBalancer: &structs.LoadBalancer{
+				Policy: "ring_hash",
+				RingHashConfig: &structs.RingHashConfig{
+					MaximumRingSize: 100,
+				},
+				HashPolicies: []structs.HashPolicy{
+					{
+						SourceIP: true,
+					},
+				},
+			},
 		},
 		&structs.ServiceResolverConfigEntry{
 			Kind:          "service-resolver",
@@ -1782,7 +1812,7 @@ func testcase_AllBellsAndWhistles() compileTestCase {
 		Protocol:  "http",
 		StartNode: "router:main.default",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"router:main.default": &structs.DiscoveryGraphNode{
+			"router:main.default": {
 				Type: structs.DiscoveryGraphNodeTypeRouter,
 				Name: "main.default",
 				Routes: []*structs.DiscoveryRoute{
@@ -1800,7 +1830,7 @@ func testcase_AllBellsAndWhistles() compileTestCase {
 					},
 				},
 			},
-			"splitter:svc-split.default": &structs.DiscoveryGraphNode{
+			"splitter:svc-split.default": {
 				Type: structs.DiscoveryGraphNodeTypeSplitter,
 				Name: "svc-split.default",
 				Splits: []*structs.DiscoverySplit{
@@ -1821,16 +1851,38 @@ func testcase_AllBellsAndWhistles() compileTestCase {
 						NextNode: "resolver:v3.main.default.dc1",
 					},
 				},
+				LoadBalancer: &structs.LoadBalancer{
+					Policy: "ring_hash",
+					RingHashConfig: &structs.RingHashConfig{
+						MaximumRingSize: 100,
+					},
+					HashPolicies: []structs.HashPolicy{
+						{
+							SourceIP: true,
+						},
+					},
+				},
 			},
-			"resolver:prod.redirected.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:prod.redirected.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "prod.redirected.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
 					ConnectTimeout: 5 * time.Second,
 					Target:         "prod.redirected.default.dc1",
 				},
+				LoadBalancer: &structs.LoadBalancer{
+					Policy: "ring_hash",
+					RingHashConfig: &structs.RingHashConfig{
+						MaximumRingSize: 100,
+					},
+					HashPolicies: []structs.HashPolicy{
+						{
+							SourceIP: true,
+						},
+					},
+				},
 			},
-			"resolver:v1.main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:v1.main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "v1.main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -1838,7 +1890,7 @@ func testcase_AllBellsAndWhistles() compileTestCase {
 					Target:         "v1.main.default.dc1",
 				},
 			},
-			"resolver:v2.main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:v2.main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "v2.main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -1846,7 +1898,7 @@ func testcase_AllBellsAndWhistles() compileTestCase {
 					Target:         "v2.main.default.dc1",
 				},
 			},
-			"resolver:v3.main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:v3.main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "v3.main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -1854,7 +1906,7 @@ func testcase_AllBellsAndWhistles() compileTestCase {
 					Target:         "v3.main.default.dc1",
 				},
 			},
-			"resolver:default-subset.main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:default-subset.main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "default-subset.main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -2006,7 +2058,7 @@ func testcase_FailoverCrossesProtocols() compileTestCase {
 			Kind: structs.ServiceResolver,
 			Name: "main",
 			Failover: map[string]structs.ServiceResolverFailover{
-				"*": structs.ServiceResolverFailover{
+				"*": {
 					Service: "other",
 				},
 			},
@@ -2075,7 +2127,7 @@ func testcase_ResolverProtocolOverride() compileTestCase {
 		Protocol:  "http2",
 		StartNode: "resolver:main.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -2108,7 +2160,7 @@ func testcase_ResolverProtocolOverrideIgnored() compileTestCase {
 		Protocol:  "http2",
 		StartNode: "resolver:main.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -2145,7 +2197,7 @@ func testcase_RouterIgnored_ResolverProtocolOverride() compileTestCase {
 		Protocol:  "tcp",
 		StartNode: "resolver:main.default.dc1",
 		Nodes: map[string]*structs.DiscoveryGraphNode{
-			"resolver:main.default.dc1": &structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
 				Type: structs.DiscoveryGraphNodeTypeResolver,
 				Name: "main.default.dc1",
 				Resolver: &structs.DiscoveryResolver{
@@ -2219,6 +2271,231 @@ func testcase_CircularSplit() compileTestCase {
 	}
 }
 
+func testcase_LBSplitterAndResolver() compileTestCase {
+	entries := newEntries()
+	setServiceProtocol(entries, "foo", "http")
+	setServiceProtocol(entries, "bar", "http")
+	setServiceProtocol(entries, "baz", "http")
+
+	entries.AddSplitters(
+		&structs.ServiceSplitterConfigEntry{
+			Kind: "service-splitter",
+			Name: "main",
+			Splits: []structs.ServiceSplit{
+				{Weight: 60, Service: "foo"},
+				{Weight: 20, Service: "bar"},
+				{Weight: 20, Service: "baz"},
+			},
+		},
+	)
+
+	entries.AddResolvers(
+		&structs.ServiceResolverConfigEntry{
+			Kind: "service-resolver",
+			Name: "foo",
+			LoadBalancer: &structs.LoadBalancer{
+				Policy: "least_request",
+				LeastRequestConfig: &structs.LeastRequestConfig{
+					ChoiceCount: 3,
+				},
+			},
+		},
+		&structs.ServiceResolverConfigEntry{
+			Kind: "service-resolver",
+			Name: "bar",
+			LoadBalancer: &structs.LoadBalancer{
+				Policy: "ring_hash",
+				RingHashConfig: &structs.RingHashConfig{
+					MaximumRingSize: 101,
+				},
+				HashPolicies: []structs.HashPolicy{
+					{
+						SourceIP: true,
+					},
+				},
+			},
+		},
+		&structs.ServiceResolverConfigEntry{
+			Kind: "service-resolver",
+			Name: "baz",
+			LoadBalancer: &structs.LoadBalancer{
+				Policy: "maglev",
+				HashPolicies: []structs.HashPolicy{
+					{
+						Field:      "cookie",
+						FieldValue: "chocolate-chip",
+						CookieConfig: &structs.CookieConfig{
+							TTL:  2 * time.Minute,
+							Path: "/bowl",
+						},
+						Terminal: true,
+					},
+				},
+			},
+		},
+	)
+
+	expect := &structs.CompiledDiscoveryChain{
+		Protocol:  "http",
+		StartNode: "splitter:main.default",
+		Nodes: map[string]*structs.DiscoveryGraphNode{
+			"splitter:main.default": {
+				Type: structs.DiscoveryGraphNodeTypeSplitter,
+				Name: "main.default",
+				Splits: []*structs.DiscoverySplit{
+					{
+						Weight:   60,
+						NextNode: "resolver:foo.default.dc1",
+					},
+					{
+						Weight:   20,
+						NextNode: "resolver:bar.default.dc1",
+					},
+					{
+						Weight:   20,
+						NextNode: "resolver:baz.default.dc1",
+					},
+				},
+				// The LB config from bar is attached because splitters only care about hash-based policies,
+				// and it's the config from bar not baz because we pick the first one we encounter in the Splits.
+				LoadBalancer: &structs.LoadBalancer{
+					Policy: "ring_hash",
+					RingHashConfig: &structs.RingHashConfig{
+						MaximumRingSize: 101,
+					},
+					HashPolicies: []structs.HashPolicy{
+						{
+							SourceIP: true,
+						},
+					},
+				},
+			},
+			// Each service's LB config is passed down from the service-resolver to the resolver node
+			"resolver:foo.default.dc1": {
+				Type: structs.DiscoveryGraphNodeTypeResolver,
+				Name: "foo.default.dc1",
+				Resolver: &structs.DiscoveryResolver{
+					Default:        false,
+					ConnectTimeout: 5 * time.Second,
+					Target:         "foo.default.dc1",
+				},
+				LoadBalancer: &structs.LoadBalancer{
+					Policy: "least_request",
+					LeastRequestConfig: &structs.LeastRequestConfig{
+						ChoiceCount: 3,
+					},
+				},
+			},
+			"resolver:bar.default.dc1": {
+				Type: structs.DiscoveryGraphNodeTypeResolver,
+				Name: "bar.default.dc1",
+				Resolver: &structs.DiscoveryResolver{
+					Default:        false,
+					ConnectTimeout: 5 * time.Second,
+					Target:         "bar.default.dc1",
+				},
+				LoadBalancer: &structs.LoadBalancer{
+					Policy: "ring_hash",
+					RingHashConfig: &structs.RingHashConfig{
+						MaximumRingSize: 101,
+					},
+					HashPolicies: []structs.HashPolicy{
+						{
+							SourceIP: true,
+						},
+					},
+				},
+			},
+			"resolver:baz.default.dc1": {
+				Type: structs.DiscoveryGraphNodeTypeResolver,
+				Name: "baz.default.dc1",
+				Resolver: &structs.DiscoveryResolver{
+					Default:        false,
+					ConnectTimeout: 5 * time.Second,
+					Target:         "baz.default.dc1",
+				},
+				LoadBalancer: &structs.LoadBalancer{
+					Policy: "maglev",
+					HashPolicies: []structs.HashPolicy{
+						{
+							Field:      "cookie",
+							FieldValue: "chocolate-chip",
+							CookieConfig: &structs.CookieConfig{
+								TTL:  2 * time.Minute,
+								Path: "/bowl",
+							},
+							Terminal: true,
+						},
+					},
+				},
+			},
+		},
+		Targets: map[string]*structs.DiscoveryTarget{
+			"foo.default.dc1": newTarget("foo", "", "default", "dc1", nil),
+			"bar.default.dc1": newTarget("bar", "", "default", "dc1", nil),
+			"baz.default.dc1": newTarget("baz", "", "default", "dc1", nil),
+		},
+	}
+
+	return compileTestCase{entries: entries, expect: expect}
+}
+
+// ensure chain with LB cfg in resolver isn't a default chain (!IsDefault)
+func testcase_LBResolver() compileTestCase {
+	entries := newEntries()
+	setServiceProtocol(entries, "main", "http")
+
+	entries.AddResolvers(
+		&structs.ServiceResolverConfigEntry{
+			Kind: "service-resolver",
+			Name: "main",
+			LoadBalancer: &structs.LoadBalancer{
+				Policy: "ring_hash",
+				RingHashConfig: &structs.RingHashConfig{
+					MaximumRingSize: 101,
+				},
+				HashPolicies: []structs.HashPolicy{
+					{
+						SourceIP: true,
+					},
+				},
+			},
+		},
+	)
+
+	expect := &structs.CompiledDiscoveryChain{
+		Protocol:  "http",
+		StartNode: "resolver:main.default.dc1",
+		Nodes: map[string]*structs.DiscoveryGraphNode{
+			"resolver:main.default.dc1": {
+				Type: structs.DiscoveryGraphNodeTypeResolver,
+				Name: "main.default.dc1",
+				Resolver: &structs.DiscoveryResolver{
+					Default:        false,
+					ConnectTimeout: 5 * time.Second,
+					Target:         "main.default.dc1",
+				},
+				LoadBalancer: &structs.LoadBalancer{
+					Policy: "ring_hash",
+					RingHashConfig: &structs.RingHashConfig{
+						MaximumRingSize: 101,
+					},
+					HashPolicies: []structs.HashPolicy{
+						{
+							SourceIP: true,
+						},
+					},
+				},
+			},
+		},
+		Targets: map[string]*structs.DiscoveryTarget{
+			"main.default.dc1": newTarget("main", "", "default", "dc1", nil),
+		},
+	}
+
+	return compileTestCase{entries: entries, expect: expect}
+}
+
 func newSimpleRoute(name string, muts ...func(*structs.ServiceRoute)) structs.ServiceRoute {
 	r := structs.ServiceRoute{
 		Match: &structs.ServiceRouteMatch{
@@ -2234,7 +2511,7 @@ func newSimpleRoute(name string, muts ...func(*structs.ServiceRoute)) structs.Se
 	return r
 }
 
-func setGlobalProxyProtocol(entries *structs.DiscoveryChainConfigEntries, protocol string) {
+func setGlobalProxyProtocol(entries *configentry.DiscoveryChainSet, protocol string) {
 	entries.GlobalProxy = &structs.ProxyConfigEntry{
 		Kind: structs.ProxyDefaults,
 		Name: structs.ProxyConfigGlobal,
@@ -2244,7 +2521,7 @@ func setGlobalProxyProtocol(entries *structs.DiscoveryChainConfigEntries, protoc
 	}
 }
 
-func setServiceProtocol(entries *structs.DiscoveryChainConfigEntries, name, protocol string) {
+func setServiceProtocol(entries *configentry.DiscoveryChainSet, name, protocol string) {
 	entries.AddServices(&structs.ServiceConfigEntry{
 		Kind:     structs.ServiceDefaults,
 		Name:     name,
@@ -2252,8 +2529,8 @@ func setServiceProtocol(entries *structs.DiscoveryChainConfigEntries, name, prot
 	})
 }
 
-func newEntries() *structs.DiscoveryChainConfigEntries {
-	return &structs.DiscoveryChainConfigEntries{
+func newEntries() *configentry.DiscoveryChainSet {
+	return &configentry.DiscoveryChainSet{
 		Routers:   make(map[structs.ServiceID]*structs.ServiceRouterConfigEntry),
 		Splitters: make(map[structs.ServiceID]*structs.ServiceSplitterConfigEntry),
 		Resolvers: make(map[structs.ServiceID]*structs.ServiceResolverConfigEntry),
@@ -2264,8 +2541,14 @@ func newTarget(service, serviceSubset, namespace, datacenter string, modFn func(
 	t := structs.NewDiscoveryTarget(service, serviceSubset, namespace, datacenter)
 	t.SNI = connect.TargetSNI(t, "trustdomain.consul")
 	t.Name = t.SNI
+	t.ConnectTimeout = 5 * time.Second // default
 	if modFn != nil {
 		modFn(t)
 	}
 	return t
 }
+
+func targetWithConnectTimeout(t *structs.DiscoveryTarget, connectTimeout time.Duration) *structs.DiscoveryTarget {
+	t.ConnectTimeout = connectTimeout
+	return t
+}
