@@ -3,6 +3,7 @@
 package consul
 
 import (
+	"context"
 	"errors"
 	"net"
 	"strings"
@@ -22,7 +23,7 @@ var (
 
 type EnterpriseServer struct{}
 
-func (s *Server) initEnterprise() error {
+func (s *Server) initEnterprise(_ Deps) error {
 	return nil
 }
 
@@ -46,11 +47,7 @@ func (s *Server) handleEnterpriseLeave() {
 	return
 }
 
-func (s *Server) enterpriseStats() map[string]map[string]string {
-	return nil
-}
-
-func (s *Server) establishEnterpriseLeadership() error {
+func (s *Server) establishEnterpriseLeadership(_ context.Context) error {
 	return nil
 }
 
