@@ -18,6 +18,10 @@ import (
 )
 
 func TestFederationState_Apply_Upsert(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
@@ -100,6 +104,10 @@ func TestFederationState_Apply_Upsert(t *testing.T) {
 }
 
 func TestFederationState_Apply_Upsert_ACLDeny(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
@@ -173,6 +181,10 @@ func TestFederationState_Apply_Upsert_ACLDeny(t *testing.T) {
 }
 
 func TestFederationState_Get(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
@@ -212,6 +224,10 @@ func TestFederationState_Get(t *testing.T) {
 }
 
 func TestFederationState_Get_ACLDeny(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
@@ -281,6 +297,10 @@ func TestFederationState_Get_ACLDeny(t *testing.T) {
 }
 
 func TestFederationState_List(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
@@ -373,6 +393,10 @@ func TestFederationState_List(t *testing.T) {
 }
 
 func TestFederationState_List_ACLDeny(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
@@ -563,6 +587,10 @@ func TestFederationState_List_ACLDeny(t *testing.T) {
 }
 
 func TestFederationState_Apply_Delete(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
@@ -642,6 +670,10 @@ func TestFederationState_Apply_Delete(t *testing.T) {
 }
 
 func TestFederationState_Apply_Delete_ACLDeny(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
