@@ -6,15 +6,20 @@ import (
 	"testing"
 	"time"
 
+	msgpackrpc "github.com/hashicorp/net-rpc-msgpackrpc"
+	"github.com/stretchr/testify/require"
+
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/sdk/testutil"
-	msgpackrpc "github.com/hashicorp/net-rpc-msgpackrpc"
-	"github.com/stretchr/testify/require"
 )
 
 // Test basic creation
 func TestIntentionApply_new(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServer(t)
@@ -98,6 +103,10 @@ func TestIntentionApply_new(t *testing.T) {
 
 // Test the source type defaults
 func TestIntentionApply_defaultSourceType(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	require := require.New(t)
@@ -144,6 +153,10 @@ func TestIntentionApply_defaultSourceType(t *testing.T) {
 
 // Shouldn't be able to create with an ID set
 func TestIntentionApply_createWithID(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	require := require.New(t)
@@ -174,6 +187,10 @@ func TestIntentionApply_createWithID(t *testing.T) {
 
 // Test basic updating
 func TestIntentionApply_updateGood(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServer(t)
@@ -254,6 +271,10 @@ func TestIntentionApply_updateGood(t *testing.T) {
 
 // Shouldn't be able to update a non-existent intention
 func TestIntentionApply_updateNonExist(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	require := require.New(t)
@@ -284,6 +305,10 @@ func TestIntentionApply_updateNonExist(t *testing.T) {
 
 // Test basic deleting
 func TestIntentionApply_deleteGood(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	require := require.New(t)
@@ -339,6 +364,10 @@ func TestIntentionApply_deleteGood(t *testing.T) {
 }
 
 func TestIntentionApply_WithoutIDs(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServer(t)
@@ -823,6 +852,10 @@ func TestIntentionApply_WithoutIDs(t *testing.T) {
 
 // Test apply with a deny ACL
 func TestIntentionApply_aclDeny(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	require := require.New(t)
@@ -903,6 +936,10 @@ service "foo" {
 }
 
 func TestIntention_WildcardACLEnforcement(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	_, srv, codec := testACLServerWithConfig(t, nil, false)
@@ -1220,6 +1257,10 @@ func TestIntention_WildcardACLEnforcement(t *testing.T) {
 
 // Test apply with delete and a default deny ACL
 func TestIntentionApply_aclDelete(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	require := require.New(t)
@@ -1297,6 +1338,10 @@ service "foo" {
 
 // Test apply with update and a default deny ACL
 func TestIntentionApply_aclUpdate(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	require := require.New(t)
@@ -1362,6 +1407,10 @@ service "foo" {
 
 // Test apply with a management token
 func TestIntentionApply_aclManagement(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	require := require.New(t)
@@ -1403,6 +1452,10 @@ func TestIntentionApply_aclManagement(t *testing.T) {
 
 // Test update changing the name where an ACL won't allow it
 func TestIntentionApply_aclUpdateChange(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	require := require.New(t)
@@ -1465,6 +1518,10 @@ service "foo" {
 
 // Test reading with ACLs
 func TestIntentionGet_acl(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
@@ -1562,6 +1619,10 @@ func TestIntentionGet_acl(t *testing.T) {
 }
 
 func TestIntentionList(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	require := require.New(t)
@@ -1587,6 +1648,10 @@ func TestIntentionList(t *testing.T) {
 
 // Test listing with ACLs
 func TestIntentionList_acl(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, testServerACLConfig)
@@ -1668,6 +1733,10 @@ func TestIntentionList_acl(t *testing.T) {
 // Test basic matching. We don't need to exhaustively test inputs since this
 // is tested in the agent/consul/state package.
 func TestIntentionMatch_good(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServer(t)
@@ -1735,8 +1804,104 @@ func TestIntentionMatch_good(t *testing.T) {
 	require.Equal(t, expected, actual)
 }
 
+func TestIntentionMatch_BlockOnNoChange(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
+	t.Parallel()
+
+	_, s1 := testServerWithConfig(t, func(c *Config) {
+		c.DevMode = true // keep it in ram to make it 10x faster on macos
+	})
+
+	codec := rpcClient(t, s1)
+
+	waitForLeaderEstablishment(t, s1)
+
+	run := func(t *testing.T, dataPrefix string, expectMatches int) {
+		rpcBlockingQueryTestHarness(t,
+			func(minQueryIndex uint64) (*structs.QueryMeta, <-chan error) {
+				args := &structs.IntentionQueryRequest{
+					Datacenter: "dc1",
+					Match: &structs.IntentionQueryMatch{
+						Type: structs.IntentionMatchDestination,
+						Entries: []structs.IntentionMatchEntry{
+							{Name: "bar"},
+						},
+					},
+				}
+				args.QueryOptions.MinQueryIndex = minQueryIndex
+
+				var out structs.IndexedIntentionMatches
+				errCh := channelCallRPC(s1, "Intention.Match", args, &out, func() error {
+					if len(out.Matches) != 1 {
+						return fmt.Errorf("expected 1 match got %d", len(out.Matches))
+					}
+					if len(out.Matches[0]) != expectMatches {
+						return fmt.Errorf("expected %d inner matches got %d", expectMatches, len(out.Matches[0]))
+					}
+					return nil
+				})
+				return &out.QueryMeta, errCh
+			},
+			func(i int) <-chan error {
+				var out string
+				return channelCallRPC(s1, "Intention.Apply", &structs.IntentionRequest{
+					Datacenter: "dc1",
+					Op:         structs.IntentionOpCreate,
+					Intention: &structs.Intention{
+						// {"default", "*", "default", "baz"}, // shouldn't match
+						SourceNS:        "default",
+						SourceName:      "*",
+						DestinationNS:   "default",
+						DestinationName: fmt.Sprintf(dataPrefix+"%d", i),
+						Action:          structs.IntentionActionAllow,
+					},
+				}, &out, nil)
+			},
+		)
+	}
+
+	runStep(t, "test the errNotFound path", func(t *testing.T) {
+		run(t, "other", 0)
+	})
+
+	// Create some records
+	{
+		insert := [][]string{
+			{"default", "*", "default", "*"},
+			{"default", "*", "default", "bar"},
+			{"default", "*", "default", "baz"}, // shouldn't match
+		}
+
+		for _, v := range insert {
+			var out string
+			require.NoError(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &structs.IntentionRequest{
+				Datacenter: "dc1",
+				Op:         structs.IntentionOpCreate,
+				Intention: &structs.Intention{
+					SourceNS:        v[0],
+					SourceName:      v[1],
+					DestinationNS:   v[2],
+					DestinationName: v[3],
+					Action:          structs.IntentionActionAllow,
+				},
+			}, &out))
+		}
+	}
+
+	runStep(t, "test the errNotChanged path", func(t *testing.T) {
+		run(t, "completely-different-other", 2)
+	})
+}
+
 // Test matching with ACLs
 func TestIntentionMatch_acl(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	_, srv, codec := testACLServerWithConfig(t, nil, false)
@@ -1819,6 +1984,10 @@ func TestIntentionMatch_acl(t *testing.T) {
 
 // Test the Check method defaults to allow with no ACL set.
 func TestIntentionCheck_defaultNoACL(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServer(t)
@@ -1845,6 +2014,10 @@ func TestIntentionCheck_defaultNoACL(t *testing.T) {
 
 // Test the Check method defaults to deny with allowlist ACLs.
 func TestIntentionCheck_defaultACLDeny(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
@@ -1877,6 +2050,10 @@ func TestIntentionCheck_defaultACLDeny(t *testing.T) {
 
 // Test the Check method defaults to deny with denylist ACLs.
 func TestIntentionCheck_defaultACLAllow(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
@@ -1909,6 +2086,10 @@ func TestIntentionCheck_defaultACLAllow(t *testing.T) {
 
 // Test the Check method requires service:read permission.
 func TestIntentionCheck_aclDeny(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
@@ -1962,6 +2143,10 @@ service "bar" {
 
 // Test the Check method returns allow/deny properly.
 func TestIntentionCheck_match(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	_, srv, codec := testACLServerWithConfig(t, nil, false)
