@@ -2,6 +2,7 @@ package consul
 
 import (
 	"encoding/base64"
+	"fmt"
 	"os"
 	"strings"
 	"testing"
@@ -20,6 +21,10 @@ import (
 )
 
 func TestInternal_NodeInfo(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
@@ -74,6 +79,10 @@ func TestInternal_NodeInfo(t *testing.T) {
 }
 
 func TestInternal_NodeDump(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
@@ -166,6 +175,10 @@ func TestInternal_NodeDump(t *testing.T) {
 }
 
 func TestInternal_NodeDump_Filter(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
@@ -224,6 +237,10 @@ func TestInternal_NodeDump_Filter(t *testing.T) {
 }
 
 func TestInternal_KeyringOperation(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	key1 := "H1dfkSZOVnP/JUnaBfTzXg=="
 	keyBytes1, err := base64.StdEncoding.DecodeString(key1)
@@ -307,6 +324,10 @@ func TestInternal_KeyringOperation(t *testing.T) {
 }
 
 func TestInternal_KeyringOperationList_LocalOnly(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	key1 := "H1dfkSZOVnP/JUnaBfTzXg=="
 	keyBytes1, err := base64.StdEncoding.DecodeString(key1)
@@ -388,6 +409,10 @@ func TestInternal_KeyringOperationList_LocalOnly(t *testing.T) {
 }
 
 func TestInternal_KeyringOperationWrite_LocalOnly(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	key1 := "H1dfkSZOVnP/JUnaBfTzXg=="
 	keyBytes1, err := base64.StdEncoding.DecodeString(key1)
@@ -421,6 +446,10 @@ func TestInternal_KeyringOperationWrite_LocalOnly(t *testing.T) {
 }
 
 func TestInternal_NodeInfo_FilterACL(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir, token, srv, codec := testACLFilterServer(t)
 	defer os.RemoveAll(dir)
@@ -472,6 +501,10 @@ func TestInternal_NodeInfo_FilterACL(t *testing.T) {
 }
 
 func TestInternal_NodeDump_FilterACL(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir, token, srv, codec := testACLFilterServer(t)
 	defer os.RemoveAll(dir)
@@ -522,6 +555,10 @@ func TestInternal_NodeDump_FilterACL(t *testing.T) {
 }
 
 func TestInternal_EventFire_Token(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir, srv := testServerWithConfig(t, func(c *Config) {
 		c.ACLDatacenter = "dc1"
@@ -558,6 +595,10 @@ func TestInternal_EventFire_Token(t *testing.T) {
 }
 
 func TestInternal_ServiceDump(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
@@ -651,6 +692,10 @@ func TestInternal_ServiceDump(t *testing.T) {
 }
 
 func TestInternal_ServiceDump_Kind(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
@@ -707,6 +752,10 @@ func TestInternal_ServiceDump_Kind(t *testing.T) {
 }
 
 func TestInternal_GatewayServiceDump_Terminating(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
@@ -905,6 +954,10 @@ func TestInternal_GatewayServiceDump_Terminating(t *testing.T) {
 }
 
 func TestInternal_GatewayServiceDump_Terminating_ACL(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
 		c.ACLDatacenter = "dc1"
@@ -1031,6 +1084,10 @@ func TestInternal_GatewayServiceDump_Terminating_ACL(t *testing.T) {
 }
 
 func TestInternal_GatewayServiceDump_Ingress(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
@@ -1240,6 +1297,10 @@ func TestInternal_GatewayServiceDump_Ingress(t *testing.T) {
 }
 
 func TestInternal_GatewayServiceDump_Ingress_ACL(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
 		c.ACLDatacenter = "dc1"
@@ -1381,6 +1442,10 @@ func TestInternal_GatewayServiceDump_Ingress_ACL(t *testing.T) {
 }
 
 func TestInternal_GatewayIntentions(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
@@ -1613,6 +1678,10 @@ service_prefix "terminating-gateway" { policy = "read" }
 }
 
 func TestInternal_ServiceTopology(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
@@ -1623,20 +1692,67 @@ func TestInternal_ServiceTopology(t *testing.T) {
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 
-	// api and api-proxy on node foo - upstream: web
-	// web and web-proxy on node bar - upstream: redis
-	// web and web-proxy on node baz - upstream: redis
-	// redis and redis-proxy on node zip
 	// wildcard deny intention
-	// web -> redis exact intentino
+	// ingress-gateway on node edge - upstream: api
+	// ingress -> api gateway config entry (but no intention)
+
+	// api and api-proxy on node foo - transparent proxy
+	// api -> web exact intention
+
+	// web and web-proxy on node bar - upstream: redis
+	// web and web-proxy on node baz - transparent proxy
+	// web -> redis exact intention
+
+	// redis and redis-proxy on node zip
+
 	registerTestTopologyEntries(t, codec, "")
 
 	var (
-		api   = structs.NewServiceName("api", structs.DefaultEnterpriseMeta())
-		web   = structs.NewServiceName("web", structs.DefaultEnterpriseMeta())
-		redis = structs.NewServiceName("redis", structs.DefaultEnterpriseMeta())
+		ingress = structs.NewServiceName("ingress", structs.DefaultEnterpriseMeta())
+		api     = structs.NewServiceName("api", structs.DefaultEnterpriseMeta())
+		web     = structs.NewServiceName("web", structs.DefaultEnterpriseMeta())
+		redis   = structs.NewServiceName("redis", structs.DefaultEnterpriseMeta())
 	)
 
+	t.Run("ingress", func(t *testing.T) {
+		retry.Run(t, func(r *retry.R) {
+			args := structs.ServiceSpecificRequest{
+				Datacenter:  "dc1",
+				ServiceName: "ingress",
+			}
+			var out structs.IndexedServiceTopology
+			require.NoError(r, msgpackrpc.CallWithCodec(codec, "Internal.ServiceTopology", &args, &out))
+			require.False(r, out.FilteredByACLs)
+			require.Equal(r, "http", out.ServiceTopology.MetricsProtocol)
+
+			// foo/api, foo/api-proxy
+			require.Len(r, out.ServiceTopology.Upstreams, 2)
+			require.Len(r, out.ServiceTopology.Downstreams, 0)
+
+			expectUp := map[string]structs.IntentionDecisionSummary{
+				api.String(): {
+					DefaultAllow:   true,
+					Allowed:        false,
+					HasPermissions: false,
+					ExternalSource: "nomad",
+
+					// From wildcard deny
+					HasExact: false,
+				},
+			}
+			require.Equal(r, expectUp, out.ServiceTopology.UpstreamDecisions)
+
+			expectUpstreamSources := map[string]string{
+				api.String(): structs.TopologySourceRegistration,
+			}
+			require.Equal(r, expectUpstreamSources, out.ServiceTopology.UpstreamSources)
+			require.Empty(r, out.ServiceTopology.DownstreamSources)
+
+			// The ingress gateway has an explicit upstream
+			require.False(r, out.ServiceTopology.TransparentProxy)
+		})
+	})
+
 	t.Run("api", func(t *testing.T) {
 		retry.Run(t, func(r *retry.R) {
 			args := structs.ServiceSpecificRequest{
@@ -1648,12 +1764,12 @@ func TestInternal_ServiceTopology(t *testing.T) {
 			require.False(r, out.FilteredByACLs)
 			require.Equal(r, "http", out.ServiceTopology.MetricsProtocol)
 
-			// bar/web, bar/web-proxy, baz/web, baz/web-proxy
-			require.Len(r, out.ServiceTopology.Upstreams, 4)
-			require.Len(r, out.ServiceTopology.Downstreams, 0)
+			// edge/ingress
+			require.Len(r, out.ServiceTopology.Downstreams, 1)
 
-			expectUp := map[string]structs.IntentionDecisionSummary{
-				web.String(): {
+			expectDown := map[string]structs.IntentionDecisionSummary{
+				ingress.String(): {
+					DefaultAllow:   true,
 					Allowed:        false,
 					HasPermissions: false,
 					ExternalSource: "nomad",
@@ -1662,7 +1778,33 @@ func TestInternal_ServiceTopology(t *testing.T) {
 					HasExact: false,
 				},
 			}
+			require.Equal(r, expectDown, out.ServiceTopology.DownstreamDecisions)
+
+			expectDownstreamSources := map[string]string{
+				ingress.String(): structs.TopologySourceRegistration,
+			}
+			require.Equal(r, expectDownstreamSources, out.ServiceTopology.DownstreamSources)
+
+			// bar/web, bar/web-proxy, baz/web, baz/web-proxy
+			require.Len(r, out.ServiceTopology.Upstreams, 4)
+
+			expectUp := map[string]structs.IntentionDecisionSummary{
+				web.String(): {
+					DefaultAllow:   true,
+					Allowed:        true,
+					HasPermissions: false,
+					HasExact:       true,
+				},
+			}
 			require.Equal(r, expectUp, out.ServiceTopology.UpstreamDecisions)
+
+			expectUpstreamSources := map[string]string{
+				web.String(): structs.TopologySourceSpecificIntention,
+			}
+			require.Equal(r, expectUpstreamSources, out.ServiceTopology.UpstreamSources)
+
+			// The only instance of api's proxy is in transparent mode
+			require.True(r, out.ServiceTopology.TransparentProxy)
 		})
 	})
 
@@ -1682,27 +1824,40 @@ func TestInternal_ServiceTopology(t *testing.T) {
 
 			expectDown := map[string]structs.IntentionDecisionSummary{
 				api.String(): {
-					Allowed:        false,
+					DefaultAllow:   true,
+					Allowed:        true,
 					HasPermissions: false,
-					ExternalSource: "nomad",
-
-					// From wildcard deny
-					HasExact: false,
+					HasExact:       true,
 				},
 			}
 			require.Equal(r, expectDown, out.ServiceTopology.DownstreamDecisions)
 
+			expectDownstreamSources := map[string]string{
+				api.String(): structs.TopologySourceSpecificIntention,
+			}
+			require.Equal(r, expectDownstreamSources, out.ServiceTopology.DownstreamSources)
+
 			// zip/redis, zip/redis-proxy
 			require.Len(r, out.ServiceTopology.Upstreams, 2)
 
 			expectUp := map[string]structs.IntentionDecisionSummary{
 				redis.String(): {
+					DefaultAllow:   true,
 					Allowed:        false,
 					HasPermissions: true,
 					HasExact:       true,
 				},
 			}
 			require.Equal(r, expectUp, out.ServiceTopology.UpstreamDecisions)
+
+			expectUpstreamSources := map[string]string{
+				// We prefer from-registration over intention source when there is a mix
+				redis.String(): structs.TopologySourceRegistration,
+			}
+			require.Equal(r, expectUpstreamSources, out.ServiceTopology.UpstreamSources)
+
+			// Not all instances of web are in transparent mode
+			require.False(r, out.ServiceTopology.TransparentProxy)
 		})
 	})
 
@@ -1724,17 +1879,85 @@ func TestInternal_ServiceTopology(t *testing.T) {
 
 			expectDown := map[string]structs.IntentionDecisionSummary{
 				web.String(): {
+					DefaultAllow:   true,
 					Allowed:        false,
 					HasPermissions: true,
 					HasExact:       true,
 				},
 			}
 			require.Equal(r, expectDown, out.ServiceTopology.DownstreamDecisions)
+
+			expectDownstreamSources := map[string]string{
+				web.String(): structs.TopologySourceRegistration,
+			}
+			require.Equal(r, expectDownstreamSources, out.ServiceTopology.DownstreamSources)
+			require.Empty(r, out.ServiceTopology.UpstreamSources)
+
+			// No proxies are in transparent mode
+			require.False(r, out.ServiceTopology.TransparentProxy)
+		})
+	})
+}
+
+func TestInternal_ServiceTopology_RoutingConfig(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
+	t.Parallel()
+	dir1, s1 := testServer(t)
+	defer os.RemoveAll(dir1)
+	defer s1.Shutdown()
+
+	testrpc.WaitForLeader(t, s1.RPC, "dc1")
+
+	codec := rpcClient(t, s1)
+	defer codec.Close()
+
+	// dashboard -> routing-config -> { counting, counting-v2 }
+	registerTestRoutingConfigTopologyEntries(t, codec)
+
+	t.Run("dashboard", func(t *testing.T) {
+		retry.Run(t, func(r *retry.R) {
+			args := structs.ServiceSpecificRequest{
+				Datacenter:  "dc1",
+				ServiceName: "dashboard",
+			}
+			var out structs.IndexedServiceTopology
+			require.NoError(r, msgpackrpc.CallWithCodec(codec, "Internal.ServiceTopology", &args, &out))
+			require.False(r, out.FilteredByACLs)
+			require.Equal(r, "http", out.ServiceTopology.MetricsProtocol)
+
+			require.Empty(r, out.ServiceTopology.Downstreams)
+			require.Empty(r, out.ServiceTopology.DownstreamDecisions)
+			require.Empty(r, out.ServiceTopology.DownstreamSources)
+
+			// routing-config will not appear as an Upstream service
+			// but will be present in UpstreamSources as a k-v pair.
+			require.Empty(r, out.ServiceTopology.Upstreams)
+
+			sn := structs.NewServiceName("routing-config", structs.DefaultEnterpriseMeta()).String()
+
+			expectUp := map[string]structs.IntentionDecisionSummary{
+				sn: {DefaultAllow: true, Allowed: true},
+			}
+			require.Equal(r, expectUp, out.ServiceTopology.UpstreamDecisions)
+
+			expectUpstreamSources := map[string]string{
+				sn: structs.TopologySourceRoutingConfig,
+			}
+			require.Equal(r, expectUpstreamSources, out.ServiceTopology.UpstreamSources)
+
+			require.False(r, out.ServiceTopology.TransparentProxy)
 		})
 	})
 }
 
 func TestInternal_ServiceTopology_ACL(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
 		c.ACLDatacenter = "dc1"
@@ -1750,9 +1973,17 @@ func TestInternal_ServiceTopology_ACL(t *testing.T) {
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 
-	// api and api-proxy on node foo - upstream: web
+	// wildcard deny intention
+	// ingress-gateway on node edge - upstream: api
+	// ingress -> api gateway config entry (but no intention)
+
+	// api and api-proxy on node foo - transparent proxy
+	// api -> web exact intention
+
 	// web and web-proxy on node bar - upstream: redis
-	// web and web-proxy on node baz - upstream: redis
+	// web and web-proxy on node baz - transparent proxy
+	// web -> redis exact intention
+
 	// redis and redis-proxy on node zip
 	registerTestTopologyEntries(t, codec, TestDefaultMasterToken)
 
@@ -1814,3 +2045,207 @@ service "web" { policy = "read" }
 		require.True(t, acl.IsErrPermissionDenied(err))
 	})
 }
+
+func TestInternal_IntentionUpstreams(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
+	t.Parallel()
+	dir1, s1 := testServer(t)
+	defer os.RemoveAll(dir1)
+	defer s1.Shutdown()
+
+	testrpc.WaitForLeader(t, s1.RPC, "dc1")
+
+	codec := rpcClient(t, s1)
+	defer codec.Close()
+
+	// Services:
+	// api and api-proxy on node foo
+	// web and web-proxy on node foo
+	//
+	// Intentions
+	// * -> * (deny) intention
+	// web -> api (allow)
+	registerIntentionUpstreamEntries(t, codec, "")
+
+	t.Run("web", func(t *testing.T) {
+		retry.Run(t, func(r *retry.R) {
+			args := structs.ServiceSpecificRequest{
+				Datacenter:  "dc1",
+				ServiceName: "web",
+			}
+			var out structs.IndexedServiceList
+			require.NoError(r, msgpackrpc.CallWithCodec(codec, "Internal.IntentionUpstreams", &args, &out))
+
+			// foo/api
+			require.Len(r, out.Services, 1)
+
+			expectUp := structs.ServiceList{
+				structs.NewServiceName("api", structs.DefaultEnterpriseMeta()),
+			}
+			require.Equal(r, expectUp, out.Services)
+		})
+	})
+}
+
+func TestInternal_IntentionUpstreams_BlockOnNoChange(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
+	t.Parallel()
+
+	_, s1 := testServerWithConfig(t, func(c *Config) {
+		c.DevMode = true // keep it in ram to make it 10x faster on macos
+	})
+
+	codec := rpcClient(t, s1)
+
+	waitForLeaderEstablishment(t, s1)
+
+	{ // ensure it's default deny to start
+		var out bool
+		require.NoError(t, msgpackrpc.CallWithCodec(codec, "ConfigEntry.Apply", &structs.ConfigEntryRequest{
+			Entry: &structs.ServiceIntentionsConfigEntry{
+				Kind: structs.ServiceIntentions,
+				Name: "*",
+				Sources: []*structs.SourceIntention{
+					{
+						Name:   "*",
+						Action: structs.IntentionActionDeny,
+					},
+				},
+			},
+		}, &out))
+		require.True(t, out)
+	}
+
+	run := func(t *testing.T, dataPrefix string, expectServices int) {
+		rpcBlockingQueryTestHarness(t,
+			func(minQueryIndex uint64) (*structs.QueryMeta, <-chan error) {
+				args := &structs.ServiceSpecificRequest{
+					Datacenter:  "dc1",
+					ServiceName: "web",
+				}
+				args.QueryOptions.MinQueryIndex = minQueryIndex
+
+				var out structs.IndexedServiceList
+				errCh := channelCallRPC(s1, "Internal.IntentionUpstreams", args, &out, func() error {
+					if len(out.Services) != expectServices {
+						return fmt.Errorf("expected %d services got %d", expectServices, len(out.Services))
+					}
+					return nil
+				})
+				return &out.QueryMeta, errCh
+			},
+			func(i int) <-chan error {
+				var out string
+				return channelCallRPC(s1, "Intention.Apply", &structs.IntentionRequest{
+					Datacenter: "dc1",
+					Op:         structs.IntentionOpCreate,
+					Intention: &structs.Intention{
+						SourceName:      fmt.Sprintf(dataPrefix+"-src-%d", i),
+						DestinationName: fmt.Sprintf(dataPrefix+"-dst-%d", i),
+						Action:          structs.IntentionActionAllow,
+					},
+				}, &out, nil)
+			},
+		)
+	}
+
+	runStep(t, "test the errNotFound path", func(t *testing.T) {
+		run(t, "other", 0)
+	})
+
+	// Services:
+	// api and api-proxy on node foo
+	// web and web-proxy on node foo
+	//
+	// Intentions
+	// * -> * (deny) intention
+	// web -> api (allow)
+	registerIntentionUpstreamEntries(t, codec, "")
+
+	runStep(t, "test the errNotChanged path", func(t *testing.T) {
+		run(t, "completely-different-other", 1)
+	})
+}
+
+func TestInternal_IntentionUpstreams_ACL(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
+	t.Parallel()
+	dir1, s1 := testServerWithConfig(t, func(c *Config) {
+		c.ACLDatacenter = "dc1"
+		c.ACLsEnabled = true
+		c.ACLMasterToken = TestDefaultMasterToken
+		c.ACLDefaultPolicy = "deny"
+	})
+	defer os.RemoveAll(dir1)
+	defer s1.Shutdown()
+
+	testrpc.WaitForLeader(t, s1.RPC, "dc1")
+
+	codec := rpcClient(t, s1)
+	defer codec.Close()
+
+	// Services:
+	// api and api-proxy on node foo
+	// web and web-proxy on node foo
+	//
+	// Intentions
+	// * -> * (deny) intention
+	// web -> api (allow)
+	registerIntentionUpstreamEntries(t, codec, TestDefaultMasterToken)
+
+	t.Run("valid token", func(t *testing.T) {
+		// Token grants read to read api service
+		userToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `
+service_prefix "api" { policy = "read" }
+`)
+		require.NoError(t, err)
+
+		retry.Run(t, func(r *retry.R) {
+			args := structs.ServiceSpecificRequest{
+				Datacenter:   "dc1",
+				ServiceName:  "web",
+				QueryOptions: structs.QueryOptions{Token: userToken.SecretID},
+			}
+			var out structs.IndexedServiceList
+			require.NoError(r, msgpackrpc.CallWithCodec(codec, "Internal.IntentionUpstreams", &args, &out))
+
+			// foo/api
+			require.Len(r, out.Services, 1)
+
+			expectUp := structs.ServiceList{
+				structs.NewServiceName("api", structs.DefaultEnterpriseMeta()),
+			}
+			require.Equal(r, expectUp, out.Services)
+		})
+	})
+
+	t.Run("invalid token filters results", func(t *testing.T) {
+		// Token grants read to read an unrelated service, mongo
+		userToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `
+service_prefix "mongo" { policy = "read" }
+`)
+		require.NoError(t, err)
+
+		retry.Run(t, func(r *retry.R) {
+			args := structs.ServiceSpecificRequest{
+				Datacenter:   "dc1",
+				ServiceName:  "web",
+				QueryOptions: structs.QueryOptions{Token: userToken.SecretID},
+			}
+			var out structs.IndexedServiceList
+			require.NoError(r, msgpackrpc.CallWithCodec(codec, "Internal.IntentionUpstreams", &args, &out))
+
+			// Token can't read api service
+			require.Empty(r, out.Services)
+		})
+	})
+}
