@@ -1,4 +1,3 @@
-//go:build !consulent
 // +build !consulent
 
 package usagemetrics
@@ -153,6 +152,14 @@ func TestUsageReporter_emitNodeUsage_OSS(t *testing.T) {
 						{Name: "kind", Value: "service-splitter"},
 					},
 				},
+				"consul.usage.test.consul.state.config_entries;datacenter=dc1;kind=mesh": {
+					Name:  "consul.usage.test.consul.state.config_entries",
+					Value: 0,
+					Labels: []metrics.Label{
+						{Name: "datacenter", Value: "dc1"},
+						{Name: "kind", Value: "mesh"},
+					},
+				},
 				"consul.usage.test.consul.state.config_entries;datacenter=dc1;kind=proxy-defaults": {
 					Name:  "consul.usage.test.consul.state.config_entries",
 					Value: 0,
@@ -327,6 +334,14 @@ func TestUsageReporter_emitNodeUsage_OSS(t *testing.T) {
 						{Name: "kind", Value: "service-splitter"},
 					},
 				},
+				"consul.usage.test.consul.state.config_entries;datacenter=dc1;kind=mesh": {
+					Name:  "consul.usage.test.consul.state.config_entries",
+					Value: 0,
+					Labels: []metrics.Label{
+						{Name: "datacenter", Value: "dc1"},
+						{Name: "kind", Value: "mesh"},
+					},
+				},
 				"consul.usage.test.consul.state.config_entries;datacenter=dc1;kind=proxy-defaults": {
 					Name:  "consul.usage.test.consul.state.config_entries",
 					Value: 0,
@@ -521,6 +536,14 @@ func TestUsageReporter_emitServiceUsage_OSS(t *testing.T) {
 						{Name: "kind", Value: "service-splitter"},
 					},
 				},
+				"consul.usage.test.consul.state.config_entries;datacenter=dc1;kind=mesh": {
+					Name:  "consul.usage.test.consul.state.config_entries",
+					Value: 0,
+					Labels: []metrics.Label{
+						{Name: "datacenter", Value: "dc1"},
+						{Name: "kind", Value: "mesh"},
+					},
+				},
 				"consul.usage.test.consul.state.config_entries;datacenter=dc1;kind=proxy-defaults": {
 					Name:  "consul.usage.test.consul.state.config_entries",
 					Value: 0,
@@ -552,31 +575,28 @@ func TestUsageReporter_emitServiceUsage_OSS(t *testing.T) {
 
 				tgw := structs.TestNodeServiceTerminatingGateway(t, "1.1.1.1")
 				tgw.ID = "terminating-gateway"
-
-				igw := structs.TestNodeServiceIngressGateway(t, "1.1.1.1")
-				igw.ID = "ingress-gateway"
 				// Typical services and some consul services spread across two nodes
 				require.NoError(t, s.EnsureService(5, "foo", &structs.NodeService{ID: "db", Service: "db", Tags: nil, Address: "", Port: 5000}))
 				require.NoError(t, s.EnsureService(6, "bar", &structs.NodeService{ID: "api", Service: "api", Tags: nil, Address: "", Port: 5000}))
 				require.NoError(t, s.EnsureService(7, "foo", &structs.NodeService{ID: "consul", Service: "consul", Tags: nil}))
 				require.NoError(t, s.EnsureService(8, "bar", &structs.NodeService{ID: "consul", Service: "consul", Tags: nil}))
 				require.NoError(t, s.EnsureService(9, "foo", &structs.NodeService{ID: "db-connect-proxy", Service: "db-connect-proxy", Tags: nil, Address: "", Port: 5000, Kind: structs.ServiceKindConnectProxy}))
-				require.NoError(t, s.EnsureService(10, "foo", igw))
+				require.NoError(t, s.EnsureRegistration(10, structs.TestRegisterIngressGateway(t)))
 				require.NoError(t, s.EnsureService(11, "foo", mgw))
 				require.NoError(t, s.EnsureService(12, "foo", tgw))
 				require.NoError(t, s.EnsureService(13, "bar", &structs.NodeService{ID: "db-native", Service: "db", Tags: nil, Address: "", Port: 5000, Connect: structs.ServiceConnect{Native: true}}))
 				require.NoError(t, s.EnsureConfigEntry(14, &structs.IngressGatewayConfigEntry{
 					Kind: structs.IngressGateway,
 					Name: "foo",
-				}, nil))
+				}))
 				require.NoError(t, s.EnsureConfigEntry(15, &structs.IngressGatewayConfigEntry{
 					Kind: structs.IngressGateway,
 					Name: "bar",
-				}, nil))
+				}))
 				require.NoError(t, s.EnsureConfigEntry(16, &structs.IngressGatewayConfigEntry{
 					Kind: structs.IngressGateway,
 					Name: "baz",
-				}, nil))
+				}))
 			},
 			getMembersFunc: func() []serf.Member {
 				return []serf.Member{
@@ -749,6 +769,14 @@ func TestUsageReporter_emitServiceUsage_OSS(t *testing.T) {
 						{Name: "kind", Value: "service-splitter"},
 					},
 				},
+				"consul.usage.test.consul.state.config_entries;datacenter=dc1;kind=mesh": {
+					Name:  "consul.usage.test.consul.state.config_entries",
+					Value: 0,
+					Labels: []metrics.Label{
+						{Name: "datacenter", Value: "dc1"},
+						{Name: "kind", Value: "mesh"},
+					},
+				},
 				"consul.usage.test.consul.state.config_entries;datacenter=dc1;kind=proxy-defaults": {
 					Name:  "consul.usage.test.consul.state.config_entries",
 					Value: 0,
@@ -937,6 +965,14 @@ func TestUsageReporter_emitKVUsage_OSS(t *testing.T) {
 						{Name: "kind", Value: "service-splitter"},
 					},
 				},
+				"consul.usage.test.consul.state.config_entries;datacenter=dc1;kind=mesh": {
+					Name:  "consul.usage.test.consul.state.config_entries",
+					Value: 0,
+					Labels: []metrics.Label{
+						{Name: "datacenter", Value: "dc1"},
+						{Name: "kind", Value: "mesh"},
+					},
+				},
 				"consul.usage.test.consul.state.config_entries;datacenter=dc1;kind=proxy-defaults": {
 					Name:  "consul.usage.test.consul.state.config_entries",
 					Value: 0,
@@ -1120,6 +1156,14 @@ func TestUsageReporter_emitKVUsage_OSS(t *testing.T) {
 						{Name: "kind", Value: "service-splitter"},
 					},
 				},
+				"consul.usage.test.consul.state.config_entries;datacenter=dc1;kind=mesh": {
+					Name:  "consul.usage.test.consul.state.config_entries",
+					Value: 0,
+					Labels: []metrics.Label{
+						{Name: "datacenter", Value: "dc1"},
+						{Name: "kind", Value: "mesh"},
+					},
+				},
 				"consul.usage.test.consul.state.config_entries;datacenter=dc1;kind=proxy-defaults": {
 					Name:  "consul.usage.test.consul.state.config_entries",
 					Value: 0,
