@@ -1203,12 +1203,16 @@ func (d *DNSServer) trimDNSResponse(cfg *dnsConfig, network string, req, resp *d
 
 // lookupServiceNodes returns nodes with a given service.
 func (d *DNSServer) lookupServiceNodes(cfg *dnsConfig, lookup serviceLookup) (structs.IndexedCheckServiceNodes, error) {
+	serviceTags := []string{}
+	if lookup.Tag != "" {
+		serviceTags = []string{lookup.Tag}
+	}
 	args := structs.ServiceSpecificRequest{
 		Connect:     lookup.Connect,
 		Ingress:     lookup.Ingress,
 		Datacenter:  lookup.Datacenter,
 		ServiceName: lookup.Service,
-		ServiceTags: []string{lookup.Tag},
+		ServiceTags: serviceTags,
 		TagFilter:   lookup.Tag != "",
 		QueryOptions: structs.QueryOptions{
 			Token:            d.agent.tokens.UserToken(),
@@ -1432,18 +1436,16 @@ func (d *DNSServer) serviceNodeRecords(cfg *dnsConfig, dc string, nodes structs.
 		}
 		handled[records[0].String()] = struct{}{}
 
-		if records != nil {
-			switch records[0].(type) {
-			case *dns.CNAME:
-				// keep track of the first CNAME + associated RRs but don't add to the resp.Answer yet
-				// this will only be added if no non-CNAME RRs are found
-				if len(answerCNAME) == 0 {
-					answerCNAME = records
-				}
-			default:
-				resp.Answer = append(resp.Answer, records...)
-				had_answer = true
+		switch records[0].(type) {
+		case *dns.CNAME:
+			// keep track of the first CNAME + associated RRs but don't add to the resp.Answer yet
+			// this will only be added if no non-CNAME RRs are found
+			if len(answerCNAME) == 0 {
+				answerCNAME = records
 			}
+		default:
+			resp.Answer = append(resp.Answer, records...)
+			had_answer = true
 		}
 
 		if had_answer {
