@@ -18,6 +18,10 @@ import (
 	"github.com/NYTimes/gziphandler"
 	"github.com/armon/go-metrics"
 	"github.com/armon/go-metrics/prometheus"
+	"github.com/hashicorp/go-cleanhttp"
+	"github.com/mitchellh/mapstructure"
+	"github.com/pkg/errors"
+
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/cache"
 	"github.com/hashicorp/consul/agent/config"
@@ -27,9 +31,6 @@ import (
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/lib"
 	"github.com/hashicorp/consul/logging"
-	"github.com/hashicorp/go-cleanhttp"
-	"github.com/mitchellh/mapstructure"
-	"github.com/pkg/errors"
 )
 
 var HTTPSummaries = []prometheus.SummaryDefinition{
@@ -430,12 +431,20 @@ func (s *HTTPHandlers) wrap(handler endpoint, methods []string) http.HandlerFunc
 		}
 
 		handleErr := func(err error) {
-			httpLogger.Error("Request error",
-				"method", req.Method,
-				"url", logURL,
-				"from", req.RemoteAddr,
-				"error", err,
-			)
+			if req.Context().Err() != nil {
+				httpLogger.Info("Request cancelled",
+					"method", req.Method,
+					"url", logURL,
+					"from", req.RemoteAddr,
+					"error", err)
+			} else {
+				httpLogger.Error("Request error",
+					"method", req.Method,
+					"url", logURL,
+					"from", req.RemoteAddr,
+					"error", err)
+			}
+
 			switch {
 			case isForbidden(err):
 				resp.WriteHeader(http.StatusForbidden)
@@ -722,6 +731,13 @@ func setMeta(resp http.ResponseWriter, m structs.QueryMetaCompat) {
 	setLastContact(resp, m.GetLastContact())
 	setKnownLeader(resp, m.GetKnownLeader())
 	setConsistency(resp, m.GetConsistencyLevel())
+	setQueryBackend(resp, m.GetBackend())
+}
+
+func setQueryBackend(resp http.ResponseWriter, backend structs.QueryBackend) {
+	if b := backend.String(); b != "" {
+		resp.Header().Set("X-Consul-Query-Backend", b)
+	}
 }
 
 // setCacheMeta sets http response headers to indicate cache status.
@@ -853,7 +869,7 @@ func (s *HTTPHandlers) parseConsistency(resp http.ResponseWriter, req *http.Requ
 	if _, ok := query["leader"]; ok {
 		defaults = false
 	}
-	if _, ok := query["cached"]; ok {
+	if _, ok := query["cached"]; ok && s.agent.config.HTTPUseCache {
 		b.SetUseCache(true)
 		defaults = false
 	}
@@ -1032,9 +1048,9 @@ func parseMetaPair(raw string) (string, string) {
 	return pair[0], ""
 }
 
-// parseInternal is a convenience method for endpoints that need
-// to use both parseWait and parseDC.
-func (s *HTTPHandlers) parseInternal(resp http.ResponseWriter, req *http.Request, dc *string, b structs.QueryOptionsCompat) bool {
+// parse is a convenience method for endpoints that need to use both parseWait
+// and parseDC.
+func (s *HTTPHandlers) parse(resp http.ResponseWriter, req *http.Request, dc *string, b structs.QueryOptionsCompat) bool {
 	s.parseDC(req, dc)
 	var token string
 	s.parseTokenWithDefault(req, &token)
@@ -1051,12 +1067,6 @@ func (s *HTTPHandlers) parseInternal(resp http.ResponseWriter, req *http.Request
 	return parseWait(resp, req, b)
 }
 
-// parse is a convenience method for endpoints that need
-// to use both parseWait and parseDC.
-func (s *HTTPHandlers) parse(resp http.ResponseWriter, req *http.Request, dc *string, b structs.QueryOptionsCompat) bool {
-	return s.parseInternal(resp, req, dc, b)
-}
-
 func (s *HTTPHandlers) checkWriteAccess(req *http.Request) error {
 	if req.Method == http.MethodGet || req.Method == http.MethodHead || req.Method == http.MethodOptions {
 		return nil
@@ -1088,15 +1098,3 @@ func (s *HTTPHandlers) parseFilter(req *http.Request, filter *string) {
 		*filter = other
 	}
 }
-
-func getPathSuffixUnescaped(path string, prefixToTrim string) (string, error) {
-	// The suffix may be URL-encoded, so attempt to decode
-	suffixRaw := strings.TrimPrefix(path, prefixToTrim)
-	suffixUnescaped, err := url.PathUnescape(suffixRaw)
-
-	if err != nil {
-		return suffixRaw, fmt.Errorf("failure in unescaping path param %q: %v", suffixRaw, err)
-	}
-
-	return suffixUnescaped, nil
-}
