@@ -10,7 +10,7 @@ import (
 	"github.com/hashicorp/consul/agent/structs"
 )
 
-func (s *HTTPServer) parseEntMeta(req *http.Request, entMeta *structs.EnterpriseMeta) error {
+func (s *HTTPHandlers) parseEntMeta(req *http.Request, entMeta *structs.EnterpriseMeta) error {
 	if headerNS := req.Header.Get("X-Consul-Namespace"); headerNS != "" {
 		return BadRequestError{Reason: "Invalid header: \"X-Consul-Namespace\" - Namespaces are a Consul Enterprise feature"}
 	}
@@ -20,11 +20,23 @@ func (s *HTTPServer) parseEntMeta(req *http.Request, entMeta *structs.Enterprise
 	return nil
 }
 
-func (s *HTTPServer) parseEntMetaNoWildcard(req *http.Request, _ *structs.EnterpriseMeta) error {
+func (s *HTTPHandlers) validateEnterpriseIntentionNamespace(logName, ns string, _ bool) error {
+	if ns == "" {
+		return nil
+	} else if strings.ToLower(ns) == structs.IntentionDefaultNamespace {
+		return nil
+	}
+
+	// No special handling for wildcard namespaces as they are pointless in OSS.
+
+	return BadRequestError{Reason: "Invalid " + logName + "(" + ns + ")" + ": Namespaces is a Consul Enterprise feature"}
+}
+
+func (s *HTTPHandlers) parseEntMetaNoWildcard(req *http.Request, _ *structs.EnterpriseMeta) error {
 	return s.parseEntMeta(req, nil)
 }
 
-func (s *HTTPServer) rewordUnknownEnterpriseFieldError(err error) error {
+func (s *HTTPHandlers) rewordUnknownEnterpriseFieldError(err error) error {
 	if err == nil {
 		return nil
 	}
@@ -43,8 +55,6 @@ func (s *HTTPServer) rewordUnknownEnterpriseFieldError(err error) error {
 	return err
 }
 
-func (s *HTTPServer) addEnterpriseHTMLTemplateVars(vars map[string]interface{}) {}
-
 func parseACLAuthMethodEnterpriseMeta(req *http.Request, _ *structs.ACLAuthMethodEnterpriseMeta) error {
 	if methodNS := req.URL.Query().Get("authmethod-ns"); methodNS != "" {
 		return BadRequestError{Reason: "Invalid query parameter: \"authmethod-ns\" - Namespaces are a Consul Enterprise feature"}
@@ -54,6 +64,12 @@ func parseACLAuthMethodEnterpriseMeta(req *http.Request, _ *structs.ACLAuthMetho
 }
 
 // enterpriseHandler is a noop for the enterprise implementation. we pass the original back
-func (s *HTTPServer) enterpriseHandler(next http.Handler) http.Handler {
+func (s *HTTPHandlers) enterpriseHandler(next http.Handler) http.Handler {
 	return next
 }
+
+// uiTemplateDataTransform returns an optional uiserver.UIDataTransform to allow
+// altering UI data in enterprise.
+func (s *HTTPHandlers) uiTemplateDataTransform(data map[string]interface{}) error {
+	return nil
+}
