@@ -14,6 +14,10 @@ import (
 )
 
 func TestIntentionList(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := NewTestAgent(t, "")
@@ -85,6 +89,10 @@ func TestIntentionList(t *testing.T) {
 }
 
 func TestIntentionMatch(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := NewTestAgent(t, "")
@@ -221,6 +229,10 @@ func TestIntentionMatch(t *testing.T) {
 }
 
 func TestIntentionCheck(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := NewTestAgent(t, "")
@@ -300,7 +312,56 @@ func TestIntentionCheck(t *testing.T) {
 	})
 }
 
+func TestIntentionGetExact(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
+	t.Parallel()
+
+	hcl := `
+	bootstrap = false
+	bootstrap_expect = 2
+	server = true
+	`
+
+	a1 := NewTestAgent(t, hcl)
+	a2 := NewTestAgent(t, hcl)
+
+	_, err := a1.JoinLAN([]string{
+		fmt.Sprintf("127.0.0.1:%d", a2.Config.SerfPortLAN),
+	})
+	require.NoError(t, err)
+
+	testrpc.WaitForTestAgent(t, a1.RPC, "dc1")
+	testrpc.WaitForTestAgent(t, a2.RPC, "dc1")
+	testrpc.WaitForLeader(t, a1.RPC, "dc1")
+	testrpc.WaitForLeader(t, a2.RPC, "dc1")
+
+	run := func(t *testing.T, a *TestAgent) {
+		req, err := http.NewRequest("GET", "/v1/connect/intentions/exact?source=foo&destination=bar", nil)
+		require.NoError(t, err)
+
+		resp := httptest.NewRecorder()
+		obj, _ := a.srv.IntentionExact(resp, req)
+		require.Equal(t, http.StatusNotFound, resp.Code)
+		require.Nil(t, obj)
+	}
+
+	// One of these will be the leader and the other will be a follower so we
+	// test direct RPC handling and RPC forwarding of errors at the same time.
+	for i, a := range []*TestAgent{a1, a2} {
+		t.Run(fmt.Sprintf("test agent %d of 2", i+1), func(t *testing.T) {
+			run(t, a)
+		})
+	}
+}
+
 func TestIntentionPutExact(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := NewTestAgent(t, "")
@@ -368,6 +429,10 @@ func TestIntentionPutExact(t *testing.T) {
 }
 
 func TestIntentionCreate(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := NewTestAgent(t, "")
@@ -410,6 +475,10 @@ func TestIntentionCreate(t *testing.T) {
 }
 
 func TestIntentionSpecificGet(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := NewTestAgent(t, "")
@@ -460,6 +529,10 @@ func TestIntentionSpecificGet(t *testing.T) {
 }
 
 func TestIntentionSpecificUpdate(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := NewTestAgent(t, "")
@@ -506,6 +579,10 @@ func TestIntentionSpecificUpdate(t *testing.T) {
 }
 
 func TestIntentionDeleteExact(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := NewTestAgent(t, "")
@@ -597,6 +674,10 @@ func TestIntentionDeleteExact(t *testing.T) {
 }
 
 func TestIntentionSpecificDelete(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := NewTestAgent(t, "")
