@@ -1,6 +1,7 @@
 package proxycfg
 
 import (
+	"context"
 	"path"
 	"testing"
 	"time"
@@ -14,6 +15,7 @@ import (
 	"github.com/hashicorp/consul/agent/connect"
 	"github.com/hashicorp/consul/agent/consul/discoverychain"
 	"github.com/hashicorp/consul/agent/local"
+	"github.com/hashicorp/consul/agent/rpcclient/health"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/agent/token"
 	"github.com/hashicorp/consul/sdk/testutil"
@@ -49,6 +51,10 @@ func assertLastReqArgs(t *testing.T, types *TestCacheTypes, token string, source
 }
 
 func TestManager_BasicLifecycle(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	// Create a bunch of common data for the various test cases.
 	roots, leaf := TestCerts(t)
 
@@ -101,6 +107,11 @@ func TestManager_BasicLifecycle(t *testing.T) {
 			},
 		)
 	}
+
+	upstreams := structs.TestUpstreams(t)
+	for i := range upstreams {
+		upstreams[i].DestinationNamespace = structs.IntentionDefaultNamespace
+	}
 	webProxy := &structs.NodeService{
 		Kind:    structs.ServiceKindConnectProxy,
 		ID:      "web-sidecar-proxy",
@@ -115,7 +126,7 @@ func TestManager_BasicLifecycle(t *testing.T) {
 			Config: map[string]interface{}{
 				"foo": "bar",
 			},
-			Upstreams: structs.TestUpstreams(t),
+			Upstreams: upstreams,
 		},
 	}
 
@@ -179,6 +190,8 @@ func TestManager_BasicLifecycle(t *testing.T) {
 		EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
 	})
 
+	db := structs.NewServiceName("db", nil)
+
 	// Create test cases using some of the common data above.
 	tests := []*testcase_BasicLifecycle{
 		{
@@ -186,7 +199,7 @@ func TestManager_BasicLifecycle(t *testing.T) {
 			setup: func(t *testing.T, types *TestCacheTypes) {
 				// Note that we deliberately leave the 'geo-cache' prepared query to time out
 				types.health.Set(dbHealthCacheKey, &structs.IndexedCheckServiceNodes{
-					Nodes: TestUpstreamNodes(t),
+					Nodes: TestUpstreamNodes(t, db.Name),
 				})
 				types.compiledChain.Set(dbChainCacheKey, &structs.DiscoveryChainResponse{
 					Chain: dbDefaultChain(),
@@ -206,18 +219,26 @@ func TestManager_BasicLifecycle(t *testing.T) {
 					ConfigSnapshotUpstreams: ConfigSnapshotUpstreams{
 						Leaf: leaf,
 						DiscoveryChain: map[string]*structs.CompiledDiscoveryChain{
-							"db": dbDefaultChain(),
+							db.String(): dbDefaultChain(),
 						},
-						WatchedUpstreams: nil, // Clone() clears this out
+						WatchedDiscoveryChains: map[string]context.CancelFunc{},
+						WatchedUpstreams:       nil, // Clone() clears this out
 						WatchedUpstreamEndpoints: map[string]map[string]structs.CheckServiceNodes{
-							"db": {
-								"db.default.dc1": TestUpstreamNodes(t),
+							db.String(): {
+								"db.default.dc1": TestUpstreamNodes(t, db.Name),
 							},
 						},
 						WatchedGateways: nil, // Clone() clears this out
 						WatchedGatewayEndpoints: map[string]map[string]structs.CheckServiceNodes{
-							"db": {},
+							db.String(): {},
 						},
+						UpstreamConfig: map[string]*structs.Upstream{
+							upstreams[0].Identifier(): &upstreams[0],
+							upstreams[1].Identifier(): &upstreams[1],
+							upstreams[2].Identifier(): &upstreams[2],
+						},
+						PassthroughUpstreams: map[string]map[string]map[string]struct{}{},
+						PassthroughIndices:   map[string]indexedTarget{},
 					},
 					PreparedQueryEndpoints: map[string]structs.CheckServiceNodes{},
 					WatchedServiceChecks:   map[structs.ServiceID][]structs.CheckType{},
@@ -232,7 +253,7 @@ func TestManager_BasicLifecycle(t *testing.T) {
 			setup: func(t *testing.T, types *TestCacheTypes) {
 				// Note that we deliberately leave the 'geo-cache' prepared query to time out
 				types.health.Set(db_v1_HealthCacheKey, &structs.IndexedCheckServiceNodes{
-					Nodes: TestUpstreamNodes(t),
+					Nodes: TestUpstreamNodes(t, db.Name),
 				})
 				types.health.Set(db_v2_HealthCacheKey, &structs.IndexedCheckServiceNodes{
 					Nodes: TestUpstreamNodesAlternate(t),
@@ -255,19 +276,27 @@ func TestManager_BasicLifecycle(t *testing.T) {
 					ConfigSnapshotUpstreams: ConfigSnapshotUpstreams{
 						Leaf: leaf,
 						DiscoveryChain: map[string]*structs.CompiledDiscoveryChain{
-							"db": dbSplitChain(),
+							db.String(): dbSplitChain(),
 						},
-						WatchedUpstreams: nil, // Clone() clears this out
+						WatchedDiscoveryChains: map[string]context.CancelFunc{},
+						WatchedUpstreams:       nil, // Clone() clears this out
 						WatchedUpstreamEndpoints: map[string]map[string]structs.CheckServiceNodes{
-							"db": {
-								"v1.db.default.dc1": TestUpstreamNodes(t),
+							db.String(): {
+								"v1.db.default.dc1": TestUpstreamNodes(t, db.Name),
 								"v2.db.default.dc1": TestUpstreamNodesAlternate(t),
 							},
 						},
 						WatchedGateways: nil, // Clone() clears this out
 						WatchedGatewayEndpoints: map[string]map[string]structs.CheckServiceNodes{
-							"db": {},
+							db.String(): {},
 						},
+						UpstreamConfig: map[string]*structs.Upstream{
+							upstreams[0].Identifier(): &upstreams[0],
+							upstreams[1].Identifier(): &upstreams[1],
+							upstreams[2].Identifier(): &upstreams[2],
+						},
+						PassthroughUpstreams: map[string]map[string]map[string]struct{}{},
+						PassthroughIndices:   map[string]indexedTarget{},
 					},
 					PreparedQueryEndpoints: map[string]structs.CheckServiceNodes{},
 					WatchedServiceChecks:   map[structs.ServiceID][]structs.CheckType{},
@@ -329,16 +358,19 @@ func testManager_BasicLifecycle(
 	require := require.New(t)
 	logger := testutil.Logger(t)
 	state := local.NewState(local.Config{}, logger, &token.Store{})
-	source := &structs.QuerySource{
-		Node:       "node1",
-		Datacenter: "dc1",
-	}
+	source := &structs.QuerySource{Datacenter: "dc1"}
 
 	// Stub state syncing
 	state.TriggerSyncChanges = func() {}
 
 	// Create manager
-	m, err := NewManager(ManagerConfig{Cache: c, State: state, Source: source, Logger: logger})
+	m, err := NewManager(ManagerConfig{
+		Cache:  c,
+		Health: &health.Client{Cache: c, CacheName: cachetype.HealthServicesName},
+		State:  state,
+		Source: source,
+		Logger: logger,
+	})
 	require.NoError(err)
 
 	// And run it
@@ -540,6 +572,7 @@ func TestManager_SyncState_DefaultToken(t *testing.T) {
 
 	m, err := NewManager(ManagerConfig{
 		Cache:  c,
+		Health: &health.Client{Cache: c, CacheName: cachetype.HealthServicesName},
 		State:  state,
 		Tokens: tokens,
 		Source: &structs.QuerySource{Datacenter: "dc1"},
@@ -584,6 +617,7 @@ func TestManager_SyncState_No_Notify(t *testing.T) {
 
 	m, err := NewManager(ManagerConfig{
 		Cache:  c,
+		Health: &health.Client{Cache: c, CacheName: cachetype.HealthServicesName},
 		State:  state,
 		Tokens: tokens,
 		Source: &structs.QuerySource{Datacenter: "dc1"},
