@@ -9,13 +9,14 @@ import (
 	"strings"
 	"time"
 
+	"github.com/hashicorp/go-hclog"
+	"github.com/mitchellh/copystructure"
+	"github.com/mitchellh/mapstructure"
+
 	"github.com/hashicorp/consul/agent/cache"
 	cachetype "github.com/hashicorp/consul/agent/cache-types"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/logging"
-	"github.com/hashicorp/go-hclog"
-	"github.com/mitchellh/copystructure"
-	"github.com/mitchellh/mapstructure"
 )
 
 type CacheNotifier interface {
@@ -23,6 +24,10 @@ type CacheNotifier interface {
 		correlationID string, ch chan<- cache.UpdateEvent) error
 }
 
+type Health interface {
+	Notify(ctx context.Context, req structs.ServiceSpecificRequest, correlationID string, ch chan<- cache.UpdateEvent) error
+}
+
 const (
 	coalesceTimeout                    = 200 * time.Millisecond
 	rootsWatchID                       = "roots"
@@ -40,6 +45,8 @@ const (
 	serviceConfigIDPrefix              = "service-config:"
 	serviceResolverIDPrefix            = "service-resolver:"
 	serviceIntentionsIDPrefix          = "service-intentions:"
+	intentionUpstreamsID               = "intention-upstreams"
+	meshConfigEntryID                  = "mesh"
 	svcChecksWatchIDPrefix             = cachetype.ServiceHTTPChecksName + ":"
 	serviceIDPrefix                    = string(structs.UpstreamDestTypeService) + ":"
 	preparedQueryIDPrefix              = string(structs.UpstreamDestTypePreparedQuery) + ":"
@@ -54,6 +61,7 @@ type state struct {
 	logger                hclog.Logger
 	source                *structs.QuerySource
 	cache                 CacheNotifier
+	health                Health
 	dnsConfig             DNSConfig
 	serverSNIFn           ServerSNIFunc
 	intentionDefaultAllow bool
@@ -155,6 +163,7 @@ func newState(ns *structs.NodeService, token string) (*state, error) {
 		taggedAddresses: taggedAddresses,
 		proxyCfg:        proxyCfg,
 		token:           token,
+
 		// 10 is fairly arbitrary here but allow for the 3 mandatory and a
 		// reasonable number of upstream watches to all deliver their initial
 		// messages in parallel without blocking the cache.Notify loops. It's not a
@@ -175,13 +184,14 @@ func newState(ns *structs.NodeService, token string) (*state, error) {
 func (s *state) Watch() (<-chan ConfigSnapshot, error) {
 	s.ctx, s.cancel = context.WithCancel(context.Background())
 
-	err := s.initWatches()
+	snap := s.initialConfigSnapshot()
+	err := s.initWatches(&snap)
 	if err != nil {
 		s.cancel()
 		return nil, err
 	}
 
-	go s.run()
+	go s.run(&snap)
 
 	return s.snapCh, nil
 }
@@ -195,10 +205,10 @@ func (s *state) Close() error {
 }
 
 // initWatches sets up the watches needed for the particular service
-func (s *state) initWatches() error {
+func (s *state) initWatches(snap *ConfigSnapshot) error {
 	switch s.kind {
 	case structs.ServiceKindConnectProxy:
-		return s.initWatchesConnectProxy()
+		return s.initWatchesConnectProxy(snap)
 	case structs.ServiceKindTerminatingGateway:
 		return s.initWatchesTerminatingGateway()
 	case structs.ServiceKindMeshGateway:
@@ -221,29 +231,9 @@ func (s *state) watchMeshGateway(ctx context.Context, dc string, upstreamID stri
 	}, "mesh-gateway:"+dc+":"+upstreamID, s.ch)
 }
 
-func (s *state) watchConnectProxyService(ctx context.Context, correlationId string, service string, dc string, filter string, entMeta *structs.EnterpriseMeta) error {
-	var finalMeta structs.EnterpriseMeta
-	finalMeta.Merge(entMeta)
-
-	return s.cache.Notify(ctx, cachetype.HealthServicesName, &structs.ServiceSpecificRequest{
-		Datacenter: dc,
-		QueryOptions: structs.QueryOptions{
-			Token:  s.token,
-			Filter: filter,
-		},
-		ServiceName: service,
-		Connect:     true,
-		// Note that Identifier doesn't type-prefix for service any more as it's
-		// the default and makes metrics and other things much cleaner. It's
-		// simpler for us if we have the type to make things unambiguous.
-		Source:         *s.source,
-		EnterpriseMeta: finalMeta,
-	}, correlationId, s.ch)
-}
-
 // initWatchesConnectProxy sets up the watches needed based on current proxy registration
 // state.
-func (s *state) initWatchesConnectProxy() error {
+func (s *state) initWatchesConnectProxy(snap *ConfigSnapshot) error {
 	// Watch for root changes
 	err := s.cache.Notify(s.ctx, cachetype.ConnectCARootName, &structs.DCSpecificRequest{
 		Datacenter:   s.source.Datacenter,
@@ -295,12 +285,55 @@ func (s *state) initWatchesConnectProxy() error {
 	// default the namespace to the namespace of this proxy service
 	currentNamespace := s.proxyID.NamespaceOrDefault()
 
+	if s.proxyCfg.Mode == structs.ProxyModeTransparent {
+		// When in transparent proxy we will infer upstreams from intentions with this source
+		err := s.cache.Notify(s.ctx, cachetype.IntentionUpstreamsName, &structs.ServiceSpecificRequest{
+			Datacenter:     s.source.Datacenter,
+			QueryOptions:   structs.QueryOptions{Token: s.token},
+			ServiceName:    s.proxyCfg.DestinationServiceName,
+			EnterpriseMeta: structs.NewEnterpriseMeta(s.proxyID.NamespaceOrEmpty()),
+		}, intentionUpstreamsID, s.ch)
+		if err != nil {
+			return err
+		}
+
+		err = s.cache.Notify(s.ctx, cachetype.ConfigEntryName, &structs.ConfigEntryQuery{
+			Kind:           structs.MeshConfig,
+			Name:           structs.MeshConfigMesh,
+			Datacenter:     s.source.Datacenter,
+			QueryOptions:   structs.QueryOptions{Token: s.token},
+			EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
+		}, meshConfigEntryID, s.ch)
+		if err != nil {
+			return err
+		}
+	}
+
 	// Watch for updates to service endpoints for all upstreams
-	for _, u := range s.proxyCfg.Upstreams {
+	for i := range s.proxyCfg.Upstreams {
+		u := s.proxyCfg.Upstreams[i]
+
+		// Store defaults keyed under wildcard so they can be applied to centrally configured upstreams
+		if u.DestinationName == structs.WildcardSpecifier {
+			snap.ConnectProxy.UpstreamConfig[u.DestinationID().String()] = &u
+			continue
+		}
+
+		snap.ConnectProxy.UpstreamConfig[u.Identifier()] = &u
+		// This can be true if the upstream is a synthetic entry populated from centralized upstream config.
+		// Watches should not be created for them.
+		if u.CentrallyConfigured {
+			continue
+		}
+
 		dc := s.source.Datacenter
 		if u.Datacenter != "" {
 			dc = u.Datacenter
 		}
+		if s.proxyCfg.Mode == structs.ProxyModeTransparent && (dc == "" || dc == s.source.Datacenter) {
+			// In transparent proxy mode, watches for upstreams in the local DC are handled by the IntentionUpstreams watch.
+			continue
+		}
 
 		ns := currentNamespace
 		if u.DestinationNamespace != "" {
@@ -346,7 +379,7 @@ func (s *state) initWatchesConnectProxy() error {
 				OverrideConnectTimeout: cfg.ConnectTimeout(),
 			}, "discovery-chain:"+u.Identifier(), s.ch)
 			if err != nil {
-				return err
+				return fmt.Errorf("failed to watch discovery chain for %s: %v", u.Identifier(), err)
 			}
 
 		default:
@@ -443,7 +476,7 @@ func (s *state) initWatchesMeshGateway() error {
 			return err
 		}
 
-		err = s.cache.Notify(s.ctx, cachetype.HealthServicesName, &structs.ServiceSpecificRequest{
+		err = s.health.Notify(s.ctx, structs.ServiceSpecificRequest{
 			Datacenter:   s.source.Datacenter,
 			QueryOptions: structs.QueryOptions{Token: s.token},
 			ServiceName:  structs.ConsulServiceName,
@@ -541,12 +574,16 @@ func (s *state) initialConfigSnapshot() ConfigSnapshot {
 	switch s.kind {
 	case structs.ServiceKindConnectProxy:
 		snap.ConnectProxy.DiscoveryChain = make(map[string]*structs.CompiledDiscoveryChain)
+		snap.ConnectProxy.WatchedDiscoveryChains = make(map[string]context.CancelFunc)
 		snap.ConnectProxy.WatchedUpstreams = make(map[string]map[string]context.CancelFunc)
 		snap.ConnectProxy.WatchedUpstreamEndpoints = make(map[string]map[string]structs.CheckServiceNodes)
 		snap.ConnectProxy.WatchedGateways = make(map[string]map[string]context.CancelFunc)
 		snap.ConnectProxy.WatchedGatewayEndpoints = make(map[string]map[string]structs.CheckServiceNodes)
 		snap.ConnectProxy.WatchedServiceChecks = make(map[structs.ServiceID][]structs.CheckType)
 		snap.ConnectProxy.PreparedQueryEndpoints = make(map[string]structs.CheckServiceNodes)
+		snap.ConnectProxy.UpstreamConfig = make(map[string]*structs.Upstream)
+		snap.ConnectProxy.PassthroughUpstreams = make(map[string]map[string]map[string]struct{})
+		snap.ConnectProxy.PassthroughIndices = make(map[string]indexedTarget)
 	case structs.ServiceKindTerminatingGateway:
 		snap.TerminatingGateway.WatchedServices = make(map[structs.ServiceName]context.CancelFunc)
 		snap.TerminatingGateway.WatchedIntentions = make(map[structs.ServiceName]context.CancelFunc)
@@ -582,15 +619,13 @@ func (s *state) initialConfigSnapshot() ConfigSnapshot {
 	return snap
 }
 
-func (s *state) run() {
+func (s *state) run(snap *ConfigSnapshot) {
 	// Close the channel we return from Watch when we stop so consumers can stop
 	// watching and clean up their goroutines. It's important we do this here and
 	// not in Close since this routine sends on this chan and so might panic if it
 	// gets closed from another goroutine.
 	defer close(s.snapCh)
 
-	snap := s.initialConfigSnapshot()
-
 	// This turns out to be really fiddly/painful by just using time.Timer.C
 	// directly in the code below since you can't detect when a timer is stopped
 	// vs waiting in order to know to reset it. So just use a chan to send
@@ -603,9 +638,9 @@ func (s *state) run() {
 		case <-s.ctx.Done():
 			return
 		case u := <-s.ch:
-			s.logger.Trace("A blocking query returned; handling snapshot update")
+			s.logger.Trace("A blocking query returned; handling snapshot update", "correlationID", u.CorrelationID)
 
-			if err := s.handleUpdate(u, &snap); err != nil {
+			if err := s.handleUpdate(u, snap); err != nil {
 				s.logger.Error("Failed to handle update from watch",
 					"id", u.CorrelationID, "error", err,
 				)
@@ -731,6 +766,116 @@ func (s *state) handleUpdateConnectProxy(u cache.UpdateEvent, snap *ConfigSnapsh
 		}
 		snap.ConnectProxy.IntentionsSet = true
 
+	case u.CorrelationID == intentionUpstreamsID:
+		resp, ok := u.Result.(*structs.IndexedServiceList)
+		if !ok {
+			return fmt.Errorf("invalid type for response %T", u.Result)
+		}
+
+		seenServices := make(map[string]struct{})
+		for _, svc := range resp.Services {
+			seenServices[svc.String()] = struct{}{}
+
+			cfgMap := make(map[string]interface{})
+			u, ok := snap.ConnectProxy.UpstreamConfig[svc.String()]
+			if ok {
+				cfgMap = u.Config
+			} else {
+				// Use the centralized upstream defaults if they exist and there isn't specific configuration for this upstream
+				// This is only relevant to upstreams from intentions because for explicit upstreams the defaulting is handled
+				// by the ResolveServiceConfig endpoint.
+				wildcardSID := structs.NewServiceID(structs.WildcardSpecifier, structs.WildcardEnterpriseMeta())
+				defaults, ok := snap.ConnectProxy.UpstreamConfig[wildcardSID.String()]
+				if ok {
+					u = defaults
+					cfgMap = defaults.Config
+					snap.ConnectProxy.UpstreamConfig[svc.String()] = defaults
+				}
+			}
+
+			cfg, err := parseReducedUpstreamConfig(cfgMap)
+			if err != nil {
+				// Don't hard fail on a config typo, just warn. We'll fall back on
+				// the plain discovery chain if there is an error so it's safe to
+				// continue.
+				s.logger.Warn("failed to parse upstream config",
+					"upstream", u.Identifier(),
+					"error", err,
+				)
+			}
+
+			meshGateway := s.proxyCfg.MeshGateway
+			if u != nil {
+				meshGateway = meshGateway.OverlayWith(u.MeshGateway)
+			}
+			watchOpts := discoveryChainWatchOpts{
+				id:          svc.String(),
+				name:        svc.Name,
+				namespace:   svc.NamespaceOrDefault(),
+				datacenter:  s.source.Datacenter,
+				cfg:         cfg,
+				meshGateway: meshGateway,
+			}
+			err = s.watchDiscoveryChain(snap, watchOpts)
+			if err != nil {
+				return fmt.Errorf("failed to watch discovery chain for %s: %v", svc.String(), err)
+			}
+		}
+
+		// Clean up data from services that were not in the update
+		for sn := range snap.ConnectProxy.WatchedUpstreams {
+			if upstream, ok := snap.ConnectProxy.UpstreamConfig[sn]; ok && upstream.Datacenter != "" && upstream.Datacenter != s.source.Datacenter {
+				continue
+			}
+			if _, ok := seenServices[sn]; !ok {
+				delete(snap.ConnectProxy.WatchedUpstreams, sn)
+			}
+		}
+		for sn := range snap.ConnectProxy.WatchedUpstreamEndpoints {
+			if upstream, ok := snap.ConnectProxy.UpstreamConfig[sn]; ok && upstream.Datacenter != "" && upstream.Datacenter != s.source.Datacenter {
+				continue
+			}
+			if _, ok := seenServices[sn]; !ok {
+				delete(snap.ConnectProxy.WatchedUpstreamEndpoints, sn)
+			}
+		}
+		for sn := range snap.ConnectProxy.WatchedGateways {
+			if upstream, ok := snap.ConnectProxy.UpstreamConfig[sn]; ok && upstream.Datacenter != "" && upstream.Datacenter != s.source.Datacenter {
+				continue
+			}
+			if _, ok := seenServices[sn]; !ok {
+				delete(snap.ConnectProxy.WatchedGateways, sn)
+			}
+		}
+		for sn := range snap.ConnectProxy.WatchedGatewayEndpoints {
+			if upstream, ok := snap.ConnectProxy.UpstreamConfig[sn]; ok && upstream.Datacenter != "" && upstream.Datacenter != s.source.Datacenter {
+				continue
+			}
+			if _, ok := seenServices[sn]; !ok {
+				delete(snap.ConnectProxy.WatchedGatewayEndpoints, sn)
+			}
+		}
+		for sn, cancelFn := range snap.ConnectProxy.WatchedDiscoveryChains {
+			if upstream, ok := snap.ConnectProxy.UpstreamConfig[sn]; ok && upstream.Datacenter != "" && upstream.Datacenter != s.source.Datacenter {
+				continue
+			}
+			if _, ok := seenServices[sn]; !ok {
+				cancelFn()
+				delete(snap.ConnectProxy.WatchedDiscoveryChains, sn)
+				delete(snap.ConnectProxy.DiscoveryChain, sn)
+			}
+		}
+		for uid := range snap.ConnectProxy.PassthroughUpstreams {
+			if _, ok := seenServices[uid]; !ok {
+				delete(snap.ConnectProxy.PassthroughUpstreams, uid)
+			}
+		}
+		for addr, indexed := range snap.ConnectProxy.PassthroughIndices {
+			if _, ok := seenServices[indexed.serviceName]; !ok {
+				delete(snap.ConnectProxy.PassthroughIndices, addr)
+			}
+		}
+
 	case strings.HasPrefix(u.CorrelationID, "upstream:"+preparedQueryIDPrefix):
 		resp, ok := u.Result.(*structs.PreparedQueryExecuteResponse)
 		if !ok {
@@ -746,24 +891,47 @@ func (s *state) handleUpdateConnectProxy(u cache.UpdateEvent, snap *ConfigSnapsh
 		}
 		svcID := structs.ServiceIDFromString(strings.TrimPrefix(u.CorrelationID, svcChecksWatchIDPrefix))
 		snap.ConnectProxy.WatchedServiceChecks[svcID] = resp
+
+	case u.CorrelationID == meshConfigEntryID:
+		resp, ok := u.Result.(*structs.ConfigEntryResponse)
+		if !ok {
+			return fmt.Errorf("invalid type for response: %T", u.Result)
+		}
+
+		if resp.Entry != nil {
+			meshConf, ok := resp.Entry.(*structs.MeshConfigEntry)
+			if !ok {
+				return fmt.Errorf("invalid type for config entry: %T", resp.Entry)
+			}
+			snap.ConnectProxy.MeshConfig = meshConf
+		} else {
+			snap.ConnectProxy.MeshConfig = nil
+		}
+		snap.ConnectProxy.MeshConfigSet = true
+
 	default:
-		return s.handleUpdateUpstreams(u, &snap.ConnectProxy.ConfigSnapshotUpstreams)
+		return s.handleUpdateUpstreams(u, snap)
 	}
 	return nil
 }
 
-func (s *state) handleUpdateUpstreams(u cache.UpdateEvent, snap *ConfigSnapshotUpstreams) error {
+func (s *state) handleUpdateUpstreams(u cache.UpdateEvent, snap *ConfigSnapshot) error {
 	if u.Err != nil {
 		return fmt.Errorf("error filling agent cache: %v", u.Err)
 	}
 
+	upstreamsSnapshot := &snap.ConnectProxy.ConfigSnapshotUpstreams
+	if snap.Kind == structs.ServiceKindIngressGateway {
+		upstreamsSnapshot = &snap.IngressGateway.ConfigSnapshotUpstreams
+	}
+
 	switch {
 	case u.CorrelationID == leafWatchID:
 		leaf, ok := u.Result.(*structs.IssuedCert)
 		if !ok {
 			return fmt.Errorf("invalid type for response: %T", u.Result)
 		}
-		snap.Leaf = leaf
+		upstreamsSnapshot.Leaf = leaf
 
 	case strings.HasPrefix(u.CorrelationID, "discovery-chain:"):
 		resp, ok := u.Result.(*structs.DiscoveryChainResponse)
@@ -771,9 +939,9 @@ func (s *state) handleUpdateUpstreams(u cache.UpdateEvent, snap *ConfigSnapshotU
 			return fmt.Errorf("invalid type for response: %T", u.Result)
 		}
 		svc := strings.TrimPrefix(u.CorrelationID, "discovery-chain:")
-		snap.DiscoveryChain[svc] = resp.Chain
+		upstreamsSnapshot.DiscoveryChain[svc] = resp.Chain
 
-		if err := s.resetWatchesFromChain(svc, resp.Chain, snap); err != nil {
+		if err := s.resetWatchesFromChain(svc, resp.Chain, upstreamsSnapshot); err != nil {
 			return err
 		}
 
@@ -788,10 +956,59 @@ func (s *state) handleUpdateUpstreams(u cache.UpdateEvent, snap *ConfigSnapshotU
 			return fmt.Errorf("invalid correlation id %q", u.CorrelationID)
 		}
 
-		if _, ok := snap.WatchedUpstreamEndpoints[svc]; !ok {
-			snap.WatchedUpstreamEndpoints[svc] = make(map[string]structs.CheckServiceNodes)
+		if _, ok := upstreamsSnapshot.WatchedUpstreamEndpoints[svc]; !ok {
+			upstreamsSnapshot.WatchedUpstreamEndpoints[svc] = make(map[string]structs.CheckServiceNodes)
+		}
+		upstreamsSnapshot.WatchedUpstreamEndpoints[svc][targetID] = resp.Nodes
+
+		if s.kind != structs.ServiceKindConnectProxy || s.proxyCfg.Mode != structs.ProxyModeTransparent {
+			return nil
+		}
+
+		// Clear out this target's existing passthrough upstreams and indices so that they can be repopulated below.
+		if _, ok := upstreamsSnapshot.PassthroughUpstreams[svc]; ok {
+			for addr := range upstreamsSnapshot.PassthroughUpstreams[svc][targetID] {
+				if indexed := upstreamsSnapshot.PassthroughIndices[addr]; indexed.targetID == targetID && indexed.serviceName == svc {
+					delete(upstreamsSnapshot.PassthroughIndices, addr)
+				}
+			}
+			upstreamsSnapshot.PassthroughUpstreams[svc][targetID] = make(map[string]struct{})
+		}
+
+		passthroughs := make(map[string]struct{})
+
+		for _, node := range resp.Nodes {
+			if !node.Service.Proxy.TransparentProxy.DialedDirectly {
+				continue
+			}
+
+			// Make sure to use an external address when crossing partition or DC boundaries.
+			csnIdx, addr, _ := node.BestAddress(node.Node.Datacenter != snap.Datacenter)
+
+			existing := upstreamsSnapshot.PassthroughIndices[addr]
+			if existing.idx > csnIdx {
+				// The last known instance with this address had a higher index so it takes precedence.
+				continue
+			}
+
+			// The current instance has a higher Raft index so we ensure the passthrough address is only
+			// associated with this upstream target. Older associations are cleaned up as needed.
+			delete(upstreamsSnapshot.PassthroughUpstreams[existing.serviceName][existing.targetID], addr)
+			if len(upstreamsSnapshot.PassthroughUpstreams[existing.serviceName][existing.targetID]) == 0 {
+				delete(upstreamsSnapshot.PassthroughUpstreams[existing.serviceName], existing.targetID)
+			}
+			if len(upstreamsSnapshot.PassthroughUpstreams[existing.serviceName]) == 0 {
+				delete(upstreamsSnapshot.PassthroughUpstreams, existing.serviceName)
+			}
+
+			upstreamsSnapshot.PassthroughIndices[addr] = indexedTarget{idx: csnIdx, serviceName: svc, targetID: targetID}
+			passthroughs[addr] = struct{}{}
+		}
+		if len(passthroughs) > 0 {
+			upstreamsSnapshot.PassthroughUpstreams[svc] = map[string]map[string]struct{}{
+				targetID: passthroughs,
+			}
 		}
-		snap.WatchedUpstreamEndpoints[svc][targetID] = resp.Nodes
 
 	case strings.HasPrefix(u.CorrelationID, "mesh-gateway:"):
 		resp, ok := u.Result.(*structs.IndexedNodesWithGateways)
@@ -803,10 +1020,10 @@ func (s *state) handleUpdateUpstreams(u cache.UpdateEvent, snap *ConfigSnapshotU
 		if !ok {
 			return fmt.Errorf("invalid correlation id %q", u.CorrelationID)
 		}
-		if _, ok = snap.WatchedGatewayEndpoints[svc]; !ok {
-			snap.WatchedGatewayEndpoints[svc] = make(map[string]structs.CheckServiceNodes)
+		if _, ok = upstreamsSnapshot.WatchedGatewayEndpoints[svc]; !ok {
+			upstreamsSnapshot.WatchedGatewayEndpoints[svc] = make(map[string]structs.CheckServiceNodes)
 		}
-		snap.WatchedGatewayEndpoints[svc][dc] = resp.Nodes
+		upstreamsSnapshot.WatchedGatewayEndpoints[svc][dc] = resp.Nodes
 	default:
 		return fmt.Errorf("unknown correlation ID: %s", u.CorrelationID)
 	}
@@ -861,14 +1078,29 @@ func (s *state) resetWatchesFromChain(
 		cancelFn()
 	}
 
-	needGateways := make(map[string]struct{})
+	var (
+		watchedChainEndpoints bool
+		needGateways          = make(map[string]struct{})
+	)
+
+	chainID := chain.ID()
 	for _, target := range chain.Targets {
-		s.logger.Trace("initializing watch of target",
-			"upstream", id,
-			"chain", chain.ServiceName,
-			"target", target.ID,
-			"mesh-gateway-mode", target.MeshGateway.Mode,
-		)
+		if target.ID == chainID {
+			watchedChainEndpoints = true
+		}
+
+		opts := targetWatchOpts{
+			upstreamID: id,
+			chainID:    target.ID,
+			service:    target.Service,
+			filter:     target.Subset.Filter,
+			datacenter: target.Datacenter,
+			entMeta:    target.GetEnterpriseMetadata(),
+		}
+		err := s.watchUpstreamTarget(snap, opts)
+		if err != nil {
+			return fmt.Errorf("failed to watch target %q for upstream %q", target.ID, id)
+		}
 
 		// We'll get endpoints from the gateway query, but the health still has
 		// to come from the backing service query.
@@ -878,22 +1110,31 @@ func (s *state) resetWatchesFromChain(
 		case structs.MeshGatewayModeLocal:
 			needGateways[s.source.Datacenter] = struct{}{}
 		}
+	}
 
-		ctx, cancel := context.WithCancel(s.ctx)
-		err := s.watchConnectProxyService(
-			ctx,
-			"upstream-target:"+target.ID+":"+id,
-			target.Service,
-			target.Datacenter,
-			target.Subset.Filter,
-			target.GetEnterpriseMetadata(),
-		)
+	// If the discovery chain's targets do not lead to watching all endpoints
+	// for the upstream, then create a separate watch for those too.
+	// This is needed in transparent mode because if there is some service A that
+	// redirects to service B, the dialing proxy needs to associate A's virtual IP
+	// with A's discovery chain.
+	//
+	// Outside of transparent mode we only watch the chain target, B,
+	// since A is a virtual service and traffic will not be sent to it.
+	if !watchedChainEndpoints && s.proxyCfg.Mode == structs.ProxyModeTransparent {
+		chainEntMeta := structs.NewEnterpriseMeta(chain.Namespace)
+
+		opts := targetWatchOpts{
+			upstreamID: id,
+			chainID:    chainID,
+			service:    chain.ServiceName,
+			filter:     "",
+			datacenter: chain.Datacenter,
+			entMeta:    &chainEntMeta,
+		}
+		err := s.watchUpstreamTarget(snap, opts)
 		if err != nil {
-			cancel()
-			return err
+			return fmt.Errorf("failed to watch target %q for upstream %q", chainID, id)
 		}
-
-		snap.WatchedUpstreams[id][target.ID] = cancel
 	}
 
 	for dc := range needGateways {
@@ -934,6 +1175,52 @@ func (s *state) resetWatchesFromChain(
 	return nil
 }
 
+type targetWatchOpts struct {
+	upstreamID string
+	chainID    string
+	service    string
+	filter     string
+	datacenter string
+	entMeta    *structs.EnterpriseMeta
+}
+
+func (s *state) watchUpstreamTarget(snap *ConfigSnapshotUpstreams, opts targetWatchOpts) error {
+	s.logger.Trace("initializing watch of target",
+		"upstream", opts.upstreamID,
+		"chain", opts.service,
+		"target", opts.chainID,
+	)
+
+	var finalMeta structs.EnterpriseMeta
+	finalMeta.Merge(opts.entMeta)
+
+	correlationID := "upstream-target:" + opts.chainID + ":" + opts.upstreamID
+
+	ctx, cancel := context.WithCancel(s.ctx)
+	err := s.health.Notify(ctx, structs.ServiceSpecificRequest{
+		Datacenter: opts.datacenter,
+		QueryOptions: structs.QueryOptions{
+			Token:  s.token,
+			Filter: opts.filter,
+		},
+		ServiceName: opts.service,
+		Connect:     true,
+		// Note that Identifier doesn't type-prefix for service any more as it's
+		// the default and makes metrics and other things much cleaner. It's
+		// simpler for us if we have the type to make things unambiguous.
+		Source:         *s.source,
+		EnterpriseMeta: finalMeta,
+	}, correlationID, s.ch)
+
+	if err != nil {
+		cancel()
+		return err
+	}
+	snap.WatchedUpstreams[opts.upstreamID][opts.chainID] = cancel
+
+	return nil
+}
+
 func (s *state) handleUpdateTerminatingGateway(u cache.UpdateEvent, snap *ConfigSnapshot) error {
 	if u.Err != nil {
 		return fmt.Errorf("error filling agent cache: %v", u.Err)
@@ -966,7 +1253,7 @@ func (s *state) handleUpdateTerminatingGateway(u cache.UpdateEvent, snap *Config
 			// Watch the health endpoint to discover endpoints for the service
 			if _, ok := snap.TerminatingGateway.WatchedServices[svc.Service]; !ok {
 				ctx, cancel := context.WithCancel(s.ctx)
-				err := s.cache.Notify(ctx, cachetype.HealthServicesName, &structs.ServiceSpecificRequest{
+				err := s.health.Notify(ctx, structs.ServiceSpecificRequest{
 					Datacenter:     s.source.Datacenter,
 					QueryOptions:   structs.QueryOptions{Token: s.token},
 					ServiceName:    svc.Service.Name,
@@ -1264,7 +1551,7 @@ func (s *state) handleUpdateMeshGateway(u cache.UpdateEvent, snap *ConfigSnapsho
 
 			if _, ok := snap.MeshGateway.WatchedServices[svc]; !ok {
 				ctx, cancel := context.WithCancel(s.ctx)
-				err := s.cache.Notify(ctx, cachetype.HealthServicesName, &structs.ServiceSpecificRequest{
+				err := s.health.Notify(ctx, structs.ServiceSpecificRequest{
 					Datacenter:     s.source.Datacenter,
 					QueryOptions:   structs.QueryOptions{Token: s.token},
 					ServiceName:    svc.Name,
@@ -1462,9 +1749,15 @@ func (s *state) handleUpdateIngressGateway(u cache.UpdateEvent, snap *ConfigSnap
 		for _, service := range services.Services {
 			u := makeUpstream(service)
 
-			err := s.watchIngressDiscoveryChain(snap, u)
+			watchOpts := discoveryChainWatchOpts{
+				id:         u.Identifier(),
+				name:       u.DestinationName,
+				namespace:  u.DestinationNamespace,
+				datacenter: s.source.Datacenter,
+			}
+			err := s.watchDiscoveryChain(snap, watchOpts)
 			if err != nil {
-				return err
+				return fmt.Errorf("failed to watch discovery chain for %s: %v", u.Identifier(), err)
 			}
 			watchedSvcs[u.Identifier()] = struct{}{}
 
@@ -1490,12 +1783,14 @@ func (s *state) handleUpdateIngressGateway(u cache.UpdateEvent, snap *ConfigSnap
 		}
 
 	default:
-		return s.handleUpdateUpstreams(u, &snap.IngressGateway.ConfigSnapshotUpstreams)
+		return s.handleUpdateUpstreams(u, snap)
 	}
 
 	return nil
 }
 
+// Note: Ingress gateways are always bound to ports and never unix sockets.
+// This means LocalBindPort is the only possibility
 func makeUpstream(g *structs.GatewayService) structs.Upstream {
 	upstream := structs.Upstream{
 		DestinationName:      g.Service.Name,
@@ -1512,25 +1807,46 @@ func makeUpstream(g *structs.GatewayService) structs.Upstream {
 	return upstream
 }
 
-func (s *state) watchIngressDiscoveryChain(snap *ConfigSnapshot, u structs.Upstream) error {
-	if _, ok := snap.IngressGateway.WatchedDiscoveryChains[u.Identifier()]; ok {
+type discoveryChainWatchOpts struct {
+	id          string
+	name        string
+	namespace   string
+	datacenter  string
+	cfg         reducedUpstreamConfig
+	meshGateway structs.MeshGatewayConfig
+}
+
+func (s *state) watchDiscoveryChain(snap *ConfigSnapshot, opts discoveryChainWatchOpts) error {
+	if _, ok := snap.ConnectProxy.WatchedDiscoveryChains[opts.id]; ok {
 		return nil
 	}
 
 	ctx, cancel := context.WithCancel(s.ctx)
 	err := s.cache.Notify(ctx, cachetype.CompiledDiscoveryChainName, &structs.DiscoveryChainRequest{
-		Datacenter:           s.source.Datacenter,
-		QueryOptions:         structs.QueryOptions{Token: s.token},
-		Name:                 u.DestinationName,
-		EvaluateInDatacenter: s.source.Datacenter,
-		EvaluateInNamespace:  u.DestinationNamespace,
-	}, "discovery-chain:"+u.Identifier(), s.ch)
+		Datacenter:             s.source.Datacenter,
+		QueryOptions:           structs.QueryOptions{Token: s.token},
+		Name:                   opts.name,
+		EvaluateInDatacenter:   opts.datacenter,
+		EvaluateInNamespace:    opts.namespace,
+		OverrideProtocol:       opts.cfg.Protocol,
+		OverrideConnectTimeout: opts.cfg.ConnectTimeout(),
+		OverrideMeshGateway:    opts.meshGateway,
+	}, "discovery-chain:"+opts.id, s.ch)
 	if err != nil {
 		cancel()
 		return err
 	}
 
-	snap.IngressGateway.WatchedDiscoveryChains[u.Identifier()] = cancel
+	switch s.kind {
+	case structs.ServiceKindIngressGateway:
+		snap.IngressGateway.WatchedDiscoveryChains[opts.id] = cancel
+	case structs.ServiceKindConnectProxy:
+		snap.ConnectProxy.WatchedDiscoveryChains[opts.id] = cancel
+	default:
+		cancel()
+		return fmt.Errorf("unsupported kind %s", s.kind)
+	}
+
 	return nil
 }
 
@@ -1640,7 +1956,7 @@ func (s *state) hostnameEndpoints(loggerName string, localDC string, nodes struc
 	)
 
 	for _, n := range nodes {
-		addr, _ := n.BestAddress(localDC != n.Node.Datacenter)
+		_, addr, _ := n.BestAddress(localDC != n.Node.Datacenter)
 		if net.ParseIP(addr) != nil {
 			hasIP = true
 			continue
