@@ -5,17 +5,22 @@ import (
 	"testing"
 	"time"
 
+	"github.com/stretchr/testify/require"
+
 	"github.com/hashicorp/consul/agent/cache"
 	cachetype "github.com/hashicorp/consul/agent/cache-types"
 	"github.com/hashicorp/consul/agent/checks"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/testrpc"
-	"github.com/stretchr/testify/require"
 )
 
 // Integration test for ServiceHTTPBasedChecks cache-type
 // Placed in agent pkg rather than cache-types to avoid circular dependency when importing agent.TestAgent
 func TestAgent_ServiceHTTPChecksNotification(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := NewTestAgent(t, "")
@@ -58,7 +63,7 @@ func TestAgent_ServiceHTTPChecksNotification(t *testing.T) {
 		},
 	}
 	// Adding TTL type should lead to a timeout, since only HTTP-based checks are watched
-	if err := a.AddService(&service, chkTypes[2:], false, "", ConfigSourceLocal); err != nil {
+	if err := a.addServiceFromSource(&service, chkTypes[2:], false, "", ConfigSourceLocal); err != nil {
 		t.Fatalf("failed to add service: %v", err)
 	}
 
@@ -70,7 +75,7 @@ func TestAgent_ServiceHTTPChecksNotification(t *testing.T) {
 	}
 
 	// Adding service with HTTP checks should lead notification for them
-	if err := a.AddService(&service, chkTypes[0:2], false, "", ConfigSourceLocal); err != nil {
+	if err := a.addServiceFromSource(&service, chkTypes[0:2], false, "", ConfigSourceLocal); err != nil {
 		t.Fatalf("failed to add service: %v", err)
 	}
 
