@@ -10,7 +10,7 @@ import (
 // Snapshot handles requests to take and restore snapshots. This uses a special
 // mechanism to make the RPC since we potentially stream large amounts of data
 // as part of these requests.
-func (s *HTTPServer) Snapshot(resp http.ResponseWriter, req *http.Request) (interface{}, error) {
+func (s *HTTPHandlers) Snapshot(resp http.ResponseWriter, req *http.Request) (interface{}, error) {
 	var args structs.SnapshotRequest
 	s.parseDC(req, &args.Datacenter)
 	s.parseToken(req, &args.Token)
