@@ -24,6 +24,7 @@ type CheckDefinition struct {
 	//
 	ScriptArgs                     []string
 	HTTP                           string
+	H2PING                         string
 	Header                         map[string][]string
 	Method                         string
 	Body                           string
@@ -34,6 +35,7 @@ type CheckDefinition struct {
 	Shell                          string
 	GRPC                           string
 	GRPCUseTLS                     bool
+	TLSServerName                  string
 	TLSSkipVerify                  bool
 	AliasNode                      string
 	AliasService                   string
@@ -63,6 +65,7 @@ func (t *CheckDefinition) UnmarshalJSON(data []byte) (err error) {
 		ScriptArgsSnake                     []string    `json:"script_args"`
 		DeregisterCriticalServiceAfterSnake interface{} `json:"deregister_critical_service_after"`
 		DockerContainerIDSnake              string      `json:"docker_container_id"`
+		TLSServerNameSnake                  string      `json:"tls_server_name"`
 		TLSSkipVerifySnake                  bool        `json:"tls_skip_verify"`
 		GRPCUseTLSSnake                     bool        `json:"grpc_use_tls"`
 		ServiceIDSnake                      string      `json:"service_id"`
@@ -89,6 +92,9 @@ func (t *CheckDefinition) UnmarshalJSON(data []byte) (err error) {
 	if t.DockerContainerID == "" {
 		t.DockerContainerID = aux.DockerContainerIDSnake
 	}
+	if t.TLSServerName == "" {
+		t.TLSServerName = aux.TLSServerNameSnake
+	}
 	if aux.TLSSkipVerifySnake {
 		t.TLSSkipVerify = aux.TLSSkipVerifySnake
 	}
@@ -155,6 +161,8 @@ func (c *CheckDefinition) HealthCheck(node string) *HealthCheck {
 		Status:         api.HealthCritical,
 		Notes:          c.Notes,
 		ServiceID:      c.ServiceID,
+		Interval:       c.Interval.String(),
+		Timeout:        c.Timeout.String(),
 		EnterpriseMeta: c.EnterpriseMeta,
 	}
 	if c.Status != "" {
@@ -177,6 +185,7 @@ func (c *CheckDefinition) CheckType() *CheckType {
 		AliasNode:                      c.AliasNode,
 		AliasService:                   c.AliasService,
 		HTTP:                           c.HTTP,
+		H2PING:                         c.H2PING,
 		GRPC:                           c.GRPC,
 		GRPCUseTLS:                     c.GRPCUseTLS,
 		Header:                         c.Header,
@@ -188,6 +197,7 @@ func (c *CheckDefinition) CheckType() *CheckType {
 		Interval:                       c.Interval,
 		DockerContainerID:              c.DockerContainerID,
 		Shell:                          c.Shell,
+		TLSServerName:                  c.TLSServerName,
 		TLSSkipVerify:                  c.TLSSkipVerify,
 		Timeout:                        c.Timeout,
 		TTL:                            c.TTL,
