@@ -88,6 +88,17 @@ func (c *CompiledDiscoveryChain) IsDefault() bool {
 	return target.Service == c.ServiceName && target.Namespace == c.Namespace
 }
 
+// ID returns an ID that encodes the service, namespace, and datacenter.
+// This ID allows us to compare a discovery chain target to the chain upstream itself.
+func (c *CompiledDiscoveryChain) ID() string {
+	return chainID("", c.ServiceName, c.Namespace, c.Datacenter)
+}
+
+func (c *CompiledDiscoveryChain) CompoundServiceName() ServiceName {
+	entMeta := NewEnterpriseMeta(c.Namespace)
+	return NewServiceName(c.ServiceName, &entMeta)
+}
+
 const (
 	DiscoveryGraphNodeTypeRouter   = "router"
 	DiscoveryGraphNodeTypeSplitter = "splitter"
@@ -205,6 +216,8 @@ type DiscoveryTarget struct {
 	MeshGateway MeshGatewayConfig     `json:",omitempty"`
 	Subset      ServiceResolverSubset `json:",omitempty"`
 
+	ConnectTimeout time.Duration `json:",omitempty"`
+
 	// External is true if this target is outside of this consul cluster.
 	External bool `json:",omitempty"`
 
@@ -218,6 +231,42 @@ type DiscoveryTarget struct {
 	Name string `json:",omitempty"`
 }
 
+func (t *DiscoveryTarget) MarshalJSON() ([]byte, error) {
+	type Alias DiscoveryTarget
+	exported := struct {
+		ConnectTimeout string `json:",omitempty"`
+		*Alias
+	}{
+		ConnectTimeout: t.ConnectTimeout.String(),
+		Alias:          (*Alias)(t),
+	}
+	if t.ConnectTimeout == 0 {
+		exported.ConnectTimeout = ""
+	}
+
+	return json.Marshal(exported)
+}
+
+func (t *DiscoveryTarget) UnmarshalJSON(data []byte) error {
+	type Alias DiscoveryTarget
+	aux := &struct {
+		ConnectTimeout string
+		*Alias
+	}{
+		Alias: (*Alias)(t),
+	}
+	if err := lib.UnmarshalJSON(data, &aux); err != nil {
+		return err
+	}
+	var err error
+	if aux.ConnectTimeout != "" {
+		if t.ConnectTimeout, err = time.ParseDuration(aux.ConnectTimeout); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 func NewDiscoveryTarget(service, serviceSubset, namespace, datacenter string) *DiscoveryTarget {
 	t := &DiscoveryTarget{
 		Service:       service,
@@ -229,13 +278,16 @@ func NewDiscoveryTarget(service, serviceSubset, namespace, datacenter string) *D
 	return t
 }
 
-func (t *DiscoveryTarget) setID() {
+func chainID(subset, service, namespace, dc string) string {
 	// NOTE: this format is similar to the SNI syntax for simplicity
-	if t.ServiceSubset == "" {
-		t.ID = fmt.Sprintf("%s.%s.%s", t.Service, t.Namespace, t.Datacenter)
-	} else {
-		t.ID = fmt.Sprintf("%s.%s.%s.%s", t.ServiceSubset, t.Service, t.Namespace, t.Datacenter)
+	if subset == "" {
+		return fmt.Sprintf("%s.%s.%s", service, namespace, dc)
 	}
+	return fmt.Sprintf("%s.%s.%s.%s", subset, service, namespace, dc)
+}
+
+func (t *DiscoveryTarget) setID() {
+	t.ID = chainID(t.ServiceSubset, t.Service, t.Namespace, t.Datacenter)
 }
 
 func (t *DiscoveryTarget) String() string {
