@@ -5,9 +5,10 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/hashicorp/consul/acl"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+
+	"github.com/hashicorp/consul/acl"
 )
 
 func TestIntention_ACLs(t *testing.T) {
@@ -453,3 +454,7 @@ func TestIntention_String(t *testing.T) {
 		})
 	}
 }
+
+func TestIntentionQueryRequest_CacheInfoKey(t *testing.T) {
+	assertCacheInfoKeyIsComplete(t, &IntentionQueryRequest{})
+}
