@@ -7,6 +7,8 @@ import (
 	"github.com/stretchr/testify/require"
 )
 
+var enterpriseMetaField = "EnterpriseMeta"
+
 func TestServiceID_String(t *testing.T) {
 	t.Run("value", func(t *testing.T) {
 		sid := NewServiceID("the-id", &EnterpriseMeta{})
@@ -39,3 +41,35 @@ func TestServiceName_String(t *testing.T) {
 		require.Equal(t, "the-id", fmt.Sprintf("%v", &sn))
 	})
 }
+
+func TestIntention_HasWildcardSource(t *testing.T) {
+	t.Run("true", func(t *testing.T) {
+		ixn := Intention{
+			SourceName: WildcardSpecifier,
+		}
+		require.True(t, ixn.HasWildcardSource())
+	})
+
+	t.Run("false", func(t *testing.T) {
+		ixn := Intention{
+			SourceName: "web",
+		}
+		require.False(t, ixn.HasWildcardSource())
+	})
+}
+
+func TestIntention_HasWildcardDestination(t *testing.T) {
+	t.Run("true", func(t *testing.T) {
+		ixn := Intention{
+			DestinationName: WildcardSpecifier,
+		}
+		require.True(t, ixn.HasWildcardDestination())
+	})
+
+	t.Run("false", func(t *testing.T) {
+		ixn := Intention{
+			DestinationName: "web",
+		}
+		require.False(t, ixn.HasWildcardDestination())
+	})
+}
