@@ -11,6 +11,8 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
+	fuzz "github.com/google/gofuzz"
+
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/cache"
 	"github.com/hashicorp/consul/api"
@@ -572,38 +574,38 @@ func TestStructs_NodeService_ValidateExposeConfig(t *testing.T) {
 	}
 	cases := map[string]testCase{
 		"valid": {
-			func(x *NodeService) {},
-			"",
+			Modify: func(x *NodeService) {},
+			Err:    "",
 		},
 		"empty path": {
-			func(x *NodeService) { x.Proxy.Expose.Paths[0].Path = "" },
-			"empty path exposed",
+			Modify: func(x *NodeService) { x.Proxy.Expose.Paths[0].Path = "" },
+			Err:    "empty path exposed",
 		},
 		"invalid port negative": {
-			func(x *NodeService) { x.Proxy.Expose.Paths[0].ListenerPort = -1 },
-			"invalid listener port",
+			Modify: func(x *NodeService) { x.Proxy.Expose.Paths[0].ListenerPort = -1 },
+			Err:    "invalid listener port",
 		},
 		"invalid port too large": {
-			func(x *NodeService) { x.Proxy.Expose.Paths[0].ListenerPort = 65536 },
-			"invalid listener port",
+			Modify: func(x *NodeService) { x.Proxy.Expose.Paths[0].ListenerPort = 65536 },
+			Err:    "invalid listener port",
 		},
-		"duplicate paths": {
-			func(x *NodeService) {
-				x.Proxy.Expose.Paths[0].Path = "/metrics"
-				x.Proxy.Expose.Paths[1].Path = "/metrics"
+		"duplicate paths are allowed": {
+			Modify: func(x *NodeService) {
+				x.Proxy.Expose.Paths[0].Path = "/healthz"
+				x.Proxy.Expose.Paths[1].Path = "/healthz"
 			},
-			"duplicate paths exposed",
+			Err: "",
 		},
-		"duplicate ports": {
-			func(x *NodeService) {
+		"duplicate listener ports are not allowed": {
+			Modify: func(x *NodeService) {
 				x.Proxy.Expose.Paths[0].ListenerPort = 21600
 				x.Proxy.Expose.Paths[1].ListenerPort = 21600
 			},
-			"duplicate listener ports exposed",
+			Err: "duplicate listener ports exposed",
 		},
 		"protocol not supported": {
-			func(x *NodeService) { x.Proxy.Expose.Paths[0].Protocol = "foo" },
-			"protocol 'foo' not supported for path",
+			Modify: func(x *NodeService) { x.Proxy.Expose.Paths[0].Protocol = "foo" },
+			Err:    "protocol 'foo' not supported for path",
 		},
 	}
 
@@ -646,6 +648,12 @@ func TestStructs_NodeService_ValidateConnectProxy(t *testing.T) {
 			"Proxy.DestinationServiceName must be",
 		},
 
+		{
+			"connect-proxy: wildcard Proxy.DestinationServiceName",
+			func(x *NodeService) { x.Proxy.DestinationServiceName = "*" },
+			"Proxy.DestinationServiceName must not be",
+		},
+
 		{
 			"connect-proxy: valid Proxy.DestinationServiceName",
 			func(x *NodeService) { x.Proxy.DestinationServiceName = "hello" },
@@ -655,7 +663,7 @@ func TestStructs_NodeService_ValidateConnectProxy(t *testing.T) {
 		{
 			"connect-proxy: no port set",
 			func(x *NodeService) { x.Port = 0 },
-			"Port must",
+			"port or socketpath must",
 		},
 
 		{
@@ -695,6 +703,28 @@ func TestStructs_NodeService_ValidateConnectProxy(t *testing.T) {
 			},
 			"upstream destination name cannot be empty",
 		},
+		{
+			"connect-proxy: upstream wildcard name",
+			func(x *NodeService) {
+				x.Proxy.Upstreams = Upstreams{{
+					DestinationType: UpstreamDestTypeService,
+					DestinationName: WildcardSpecifier,
+					LocalBindPort:   5000,
+				}}
+			},
+			"upstream destination name cannot be a wildcard",
+		},
+		{
+			"connect-proxy: upstream can have wildcard name when centrally configured",
+			func(x *NodeService) {
+				x.Proxy.Upstreams = Upstreams{{
+					DestinationType:     UpstreamDestTypeService,
+					DestinationName:     WildcardSpecifier,
+					CentrallyConfigured: true,
+				}}
+			},
+			"",
+		},
 		{
 			"connect-proxy: upstream empty bind port",
 			func(x *NodeService) {
@@ -704,7 +734,19 @@ func TestStructs_NodeService_ValidateConnectProxy(t *testing.T) {
 					LocalBindPort:   0,
 				}}
 			},
-			"upstream local bind port cannot be zero",
+			"upstream local bind port or local socket path must be defined and nonzero",
+		},
+		{
+			"connect-proxy: upstream bind port and path defined",
+			func(x *NodeService) {
+				x.Proxy.Upstreams = Upstreams{{
+					DestinationType:     UpstreamDestTypeService,
+					DestinationName:     "foo",
+					LocalBindPort:       1,
+					LocalBindSocketPath: "/tmp/socket",
+				}}
+			},
+			"only one of upstream local bind port or local socket path can be defined and nonzero",
 		},
 		{
 			"connect-proxy: Upstreams almost-but-not-quite-duplicated in various ways",
@@ -766,6 +808,24 @@ func TestStructs_NodeService_ValidateConnectProxy(t *testing.T) {
 			},
 			"upstreams cannot contain duplicates",
 		},
+		{
+			"connect-proxy: Centrally configured upstreams can have duplicate ip/port",
+			func(x *NodeService) {
+				x.Proxy.Upstreams = Upstreams{
+					{
+						DestinationType:     UpstreamDestTypeService,
+						DestinationName:     "foo",
+						CentrallyConfigured: true,
+					},
+					{
+						DestinationType:     UpstreamDestTypeService,
+						DestinationName:     "bar",
+						CentrallyConfigured: true,
+					},
+				}
+			},
+			"",
+		},
 		{
 			"connect-proxy: Upstreams duplicated by ip and port",
 			func(x *NodeService) {
@@ -1528,6 +1588,84 @@ func TestStructs_validateMetaPair(t *testing.T) {
 	}
 }
 
+func TestDCSpecificRequest_CacheInfoKey(t *testing.T) {
+	assertCacheInfoKeyIsComplete(t, &DCSpecificRequest{})
+}
+
+func TestNodeSpecificRequest_CacheInfoKey(t *testing.T) {
+	assertCacheInfoKeyIsComplete(t, &NodeSpecificRequest{})
+}
+
+func TestServiceSpecificRequest_CacheInfoKey(t *testing.T) {
+	assertCacheInfoKeyIsComplete(t, &ServiceSpecificRequest{})
+}
+
+func TestServiceDumpRequest_CacheInfoKey(t *testing.T) {
+	// ServiceKind is only included when UseServiceKind=true
+	assertCacheInfoKeyIsComplete(t, &ServiceDumpRequest{}, "ServiceKind")
+}
+
+// cacheInfoIgnoredFields are fields that can be ignored in all cache.Request types
+// because the cache itself includes these values in the cache key, or because
+// they are options used to specify the cache operation, and are not part of the
+// cache entry value.
+var cacheInfoIgnoredFields = map[string]bool{
+	// Datacenter is part of the cache key added by the cache itself.
+	"Datacenter": true,
+	// QuerySource is always the same for every request from a single agent, so it
+	// is excluded from the key.
+	"Source": true,
+	// EnterpriseMeta is an empty struct, so can not be included.
+	enterpriseMetaField: true,
+}
+
+// assertCacheInfoKeyIsComplete is an assertion to verify that all fields on a request
+// struct are considered as part of the cache key. It is used to prevent regressions
+// when new fields are added to the struct. If a field is not included in the cache
+// key it can lead to API requests or DNS requests returning the wrong value
+// because a request matches the wrong entry in the agent/cache.Cache.
+func assertCacheInfoKeyIsComplete(t *testing.T, request cache.Request, ignoredFields ...string) {
+	t.Helper()
+
+	ignored := make(map[string]bool, len(ignoredFields))
+	for _, f := range ignoredFields {
+		ignored[f] = true
+	}
+
+	fuzzer := fuzz.NewWithSeed(time.Now().UnixNano())
+	fuzzer.Funcs(randQueryOptions)
+	fuzzer.Fuzz(request)
+	requestValue := reflect.ValueOf(request).Elem()
+
+	for i := 0; i < requestValue.NumField(); i++ {
+		originalKey := request.CacheInfo().Key
+		field := requestValue.Field(i)
+		fieldName := requestValue.Type().Field(i).Name
+		originalValue := field.Interface()
+
+		if cacheInfoIgnoredFields[fieldName] || ignored[fieldName] {
+			continue
+		}
+
+		for i := 0; reflect.DeepEqual(originalValue, field.Interface()) && i < 20; i++ {
+			fuzzer.Fuzz(field.Addr().Interface())
+		}
+
+		key := request.CacheInfo().Key
+		if originalKey == key {
+			t.Fatalf("expected field %v to be represented in the CacheInfo.Key, %v change to %v (key: %v)",
+				fieldName,
+				originalValue,
+				field.Interface(),
+				key)
+		}
+	}
+}
+
+func randQueryOptions(o *QueryOptions, c fuzz.Continue) {
+	c.Fuzz(&o.Filter)
+}
+
 func TestSpecificServiceRequest_CacheInfo(t *testing.T) {
 	tests := []struct {
 		name     string
@@ -1637,6 +1775,16 @@ func TestSpecificServiceRequest_CacheInfo(t *testing.T) {
 			},
 			wantSame: false,
 		},
+		{
+			name: "with integress=true",
+			req: ServiceSpecificRequest{
+				Datacenter:  "dc1",
+				ServiceName: "my-service",
+			},
+			mutate: func(req *ServiceSpecificRequest) {
+				req.Ingress = true
+			},
+		},
 	}
 
 	for _, tc := range tests {
@@ -1887,14 +2035,18 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 		input   CheckServiceNode
 		lanAddr string
 		lanPort int
+		lanIdx  uint64
 		wanAddr string
 		wanPort int
+		wanIdx  uint64
 	}
 
 	nodeAddr := "10.1.2.3"
 	nodeWANAddr := "198.18.19.20"
+	nodeIdx := uint64(11)
 	serviceAddr := "10.2.3.4"
 	servicePort := 1234
+	serviceIdx := uint64(22)
 	serviceWANAddr := "198.19.20.21"
 	serviceWANPort := 987
 
@@ -1903,15 +2055,23 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 			input: CheckServiceNode{
 				Node: &Node{
 					Address: nodeAddr,
+					RaftIndex: RaftIndex{
+						ModifyIndex: nodeIdx,
+					},
 				},
 				Service: &NodeService{
 					Port: servicePort,
+					RaftIndex: RaftIndex{
+						ModifyIndex: serviceIdx,
+					},
 				},
 			},
 
 			lanAddr: nodeAddr,
+			lanIdx:  nodeIdx,
 			lanPort: servicePort,
 			wanAddr: nodeAddr,
+			wanIdx:  nodeIdx,
 			wanPort: servicePort,
 		},
 		"node-wan-address": {
@@ -1921,15 +2081,23 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 					TaggedAddresses: map[string]string{
 						"wan": nodeWANAddr,
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: nodeIdx,
+					},
 				},
 				Service: &NodeService{
 					Port: servicePort,
+					RaftIndex: RaftIndex{
+						ModifyIndex: serviceIdx,
+					},
 				},
 			},
 
 			lanAddr: nodeAddr,
+			lanIdx:  nodeIdx,
 			lanPort: servicePort,
 			wanAddr: nodeWANAddr,
+			wanIdx:  nodeIdx,
 			wanPort: servicePort,
 		},
 		"service-address": {
@@ -1940,16 +2108,24 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 					TaggedAddresses: map[string]string{
 						"wan": nodeWANAddr,
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: nodeIdx,
+					},
 				},
 				Service: &NodeService{
 					Address: serviceAddr,
 					Port:    servicePort,
+					RaftIndex: RaftIndex{
+						ModifyIndex: serviceIdx,
+					},
 				},
 			},
 
 			lanAddr: serviceAddr,
+			lanIdx:  serviceIdx,
 			lanPort: servicePort,
 			wanAddr: serviceAddr,
+			wanIdx:  serviceIdx,
 			wanPort: servicePort,
 		},
 		"service-wan-address": {
@@ -1960,6 +2136,9 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 					TaggedAddresses: map[string]string{
 						"wan": nodeWANAddr,
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: nodeIdx,
+					},
 				},
 				Service: &NodeService{
 					Address: serviceAddr,
@@ -1970,12 +2149,17 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 							Port:    serviceWANPort,
 						},
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: serviceIdx,
+					},
 				},
 			},
 
 			lanAddr: serviceAddr,
+			lanIdx:  serviceIdx,
 			lanPort: servicePort,
 			wanAddr: serviceWANAddr,
+			wanIdx:  serviceIdx,
 			wanPort: serviceWANPort,
 		},
 		"service-wan-address-default-port": {
@@ -1986,6 +2170,9 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 					TaggedAddresses: map[string]string{
 						"wan": nodeWANAddr,
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: nodeIdx,
+					},
 				},
 				Service: &NodeService{
 					Address: serviceAddr,
@@ -1996,12 +2183,17 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 							Port:    0,
 						},
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: serviceIdx,
+					},
 				},
 			},
 
 			lanAddr: serviceAddr,
+			lanIdx:  serviceIdx,
 			lanPort: servicePort,
 			wanAddr: serviceWANAddr,
+			wanIdx:  serviceIdx,
 			wanPort: servicePort,
 		},
 		"service-wan-address-node-lan": {
@@ -2012,6 +2204,9 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 					TaggedAddresses: map[string]string{
 						"wan": nodeWANAddr,
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: nodeIdx,
+					},
 				},
 				Service: &NodeService{
 					Port: servicePort,
@@ -2021,12 +2216,17 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 							Port:    serviceWANPort,
 						},
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: serviceIdx,
+					},
 				},
 			},
 
 			lanAddr: nodeAddr,
+			lanIdx:  nodeIdx,
 			lanPort: servicePort,
 			wanAddr: serviceWANAddr,
+			wanIdx:  serviceIdx,
 			wanPort: serviceWANPort,
 		},
 	}
@@ -2036,13 +2236,15 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 		tc := tc
 		t.Run(name, func(t *testing.T) {
 
-			addr, port := tc.input.BestAddress(false)
+			idx, addr, port := tc.input.BestAddress(false)
 			require.Equal(t, tc.lanAddr, addr)
 			require.Equal(t, tc.lanPort, port)
+			require.Equal(t, tc.lanIdx, idx)
 
-			addr, port = tc.input.BestAddress(true)
+			idx, addr, port = tc.input.BestAddress(true)
 			require.Equal(t, tc.wanAddr, addr)
 			require.Equal(t, tc.wanPort, port)
+			require.Equal(t, tc.wanIdx, idx)
 		})
 	}
 }
@@ -2298,3 +2500,38 @@ func TestGatewayService_IsSame(t *testing.T) {
 		t.Fatalf("should be equal, was %#v VS %#v", g, other)
 	}
 }
+
+// TODO(freddy): Needs enterprise test
+func TestServiceNameFromTargetID(t *testing.T) {
+	type testcase struct {
+		tid    string
+		expect ServiceName
+	}
+	run := func(t *testing.T, tc testcase) {
+		tc.expect.EnterpriseMeta.Normalize()
+
+		got := NewServiceNameFromTargetID(tc.tid)
+		require.Equal(t, tc.expect, got)
+	}
+
+	cases := map[string]testcase{
+		"with subset": {
+			tid: "v1.foo.default.dc2",
+			expect: ServiceName{
+				Name: "foo",
+			},
+		},
+		"without subset": {
+			tid: "foo.default.dc2",
+			expect: ServiceName{
+				Name: "foo",
+			},
+		},
+	}
+
+	for name, tc := range cases {
+		t.Run(name, func(t *testing.T) {
+			run(t, tc)
+		})
+	}
+}
