@@ -16,10 +16,8 @@ import (
 	"time"
 
 	metrics "github.com/armon/go-metrics"
-	"github.com/hashicorp/errwrap"
 	"github.com/hashicorp/go-hclog"
 	uuid "github.com/hashicorp/go-uuid"
-	"github.com/stretchr/testify/require"
 
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/config"
@@ -88,7 +86,9 @@ type TestAgent struct {
 // The caller is responsible for calling Shutdown() to stop the agent and remove
 // temporary directories.
 func NewTestAgent(t *testing.T, hcl string) *TestAgent {
-	return StartTestAgent(t, TestAgent{HCL: hcl})
+	a := StartTestAgent(t, TestAgent{HCL: hcl})
+	t.Cleanup(func() { a.Shutdown() })
+	return a
 }
 
 // StartTestAgent and wait for it to become available. If the agent fails to
@@ -99,6 +99,7 @@ func NewTestAgent(t *testing.T, hcl string) *TestAgent {
 func StartTestAgent(t *testing.T, a TestAgent) *TestAgent {
 	t.Helper()
 	retry.RunWith(retry.ThreeTimes(), t, func(r *retry.R) {
+		t.Helper()
 		if err := a.Start(t); err != nil {
 			r.Fatal(err)
 		}
@@ -130,7 +131,7 @@ func TestConfigHCL(nodeID string) string {
 
 // Start starts a test agent. It returns an error if the agent could not be started.
 // If no error is returned, the caller must call Shutdown() when finished.
-func (a *TestAgent) Start(t *testing.T) (err error) {
+func (a *TestAgent) Start(t *testing.T) error {
 	t.Helper()
 	if a.Agent != nil {
 		return fmt.Errorf("TestAgent already started")
@@ -167,29 +168,41 @@ func (a *TestAgent) Start(t *testing.T) (err error) {
 
 	// Create NodeID outside the closure, so that it does not change
 	testHCLConfig := TestConfigHCL(NodeID())
-	loader := func(source config.Source) (*config.RuntimeConfig, []string, error) {
-		opts := config.BuilderOpts{
-			HCL: []string{testHCLConfig, portsConfig, a.HCL, hclDataDir},
+	loader := func(source config.Source) (config.LoadResult, error) {
+		opts := config.LoadOpts{
+			DefaultConfig: source,
+			HCL:           []string{testHCLConfig, portsConfig, a.HCL, hclDataDir},
+			Overrides: []config.Source{
+				config.FileSource{
+					Name:   "test-overrides",
+					Format: "hcl",
+					Data:   a.Overrides},
+				config.DefaultConsulSource(),
+				config.DevConsulSource(),
+			},
 		}
-		overrides := []config.Source{
-			config.FileSource{
-				Name:   "test-overrides",
-				Format: "hcl",
-				Data:   a.Overrides},
-			config.DefaultConsulSource(),
-			config.DevConsulSource(),
+		result, err := config.Load(opts)
+		if result.RuntimeConfig != nil {
+			// If prom metrics need to be enabled, do not disable telemetry
+			if result.RuntimeConfig.Telemetry.PrometheusOpts.Expiration > 0 {
+				result.RuntimeConfig.Telemetry.Disable = false
+			} else {
+				result.RuntimeConfig.Telemetry.Disable = true
+			}
 		}
-		cfg, warnings, err := config.Load(opts, source, overrides...)
-		if cfg != nil {
-			cfg.Telemetry.Disable = true
-		}
-		return cfg, warnings, err
+		return result, err
 	}
 	bd, err := NewBaseDeps(loader, logOutput)
-	require.NoError(t, err)
+	if err != nil {
+		return fmt.Errorf("failed to create base deps: %w", err)
+	}
 
 	bd.Logger = logger
-	bd.MetricsHandler = metrics.NewInmemSink(1*time.Second, time.Minute)
+	// if we are not testing telemetry things, let's use a "mock" sink for metrics
+	if bd.RuntimeConfig.Telemetry.Disable {
+		bd.MetricsHandler = metrics.NewInmemSink(1*time.Second, time.Minute)
+	}
+
 	a.Config = bd.RuntimeConfig
 
 	agent, err := New(bd)
@@ -214,8 +227,8 @@ func (a *TestAgent) Start(t *testing.T) (err error) {
 
 	if err := a.waitForUp(); err != nil {
 		a.Shutdown()
-		t.Logf("Error while waiting for test agent to start: %v", err)
-		return errwrap.Wrapf(name+": {{err}}", err)
+		a.Agent = nil
+		return fmt.Errorf("error waiting for test agent to start: %w", err)
 	}
 
 	a.dns = a.dnsServers[0]
@@ -279,17 +292,6 @@ func (a *TestAgent) waitForUp() error {
 // Shutdown stops the agent and removes the data directory if it is
 // managed by the test agent.
 func (a *TestAgent) Shutdown() error {
-	/* Removed this because it was breaking persistence tests where we would
-	persist a service and load it through a new agent with the same data-dir.
-	Not sure if we still need this for other things, everywhere we manually make
-	a data dir we already do 'defer os.RemoveAll()'
-	defer func() {
-		if a.DataDir != "" {
-			os.RemoveAll(a.DataDir)
-		}
-	}()*/
-
-	// already shut down
 	if a.Agent == nil {
 		return nil
 	}
@@ -407,8 +409,7 @@ func NodeID() string {
 	return id
 }
 
-// TestConfig returns a unique default configuration for testing an
-// agent.
+// TestConfig returns a unique default configuration for testing an agent.
 func TestConfig(logger hclog.Logger, sources ...config.Source) *config.RuntimeConfig {
 	nodeID := NodeID()
 	testsrc := config.FileSource{
@@ -434,29 +435,25 @@ func TestConfig(logger hclog.Logger, sources ...config.Source) *config.RuntimeCo
 		`,
 	}
 
-	b, err := config.NewBuilder(config.BuilderOpts{})
-	if err != nil {
-		panic("NewBuilder failed: " + err.Error())
+	opts := config.LoadOpts{
+		DefaultConfig: testsrc,
+		Overrides:     sources,
 	}
-	b.Head = append(b.Head, testsrc)
-	b.Tail = append(b.Tail, config.DefaultConsulSource(), config.DevConsulSource())
-	b.Tail = append(b.Tail, sources...)
-
-	cfg, err := b.BuildAndValidate()
+	r, err := config.Load(opts)
 	if err != nil {
-		panic("Error building config: " + err.Error())
+		panic("config.Load failed: " + err.Error())
 	}
-
-	for _, w := range b.Warnings {
+	for _, w := range r.Warnings {
 		logger.Warn(w)
 	}
 
+	cfg := r.RuntimeConfig
 	// Effectively disables the delay after root rotation before requesting CSRs
 	// to make test deterministic. 0 results in default jitter being applied but a
 	// tiny delay is effectively thre same.
 	cfg.ConnectTestCALeafRootChangeSpread = 1 * time.Nanosecond
 
-	return &cfg
+	return cfg
 }
 
 // TestACLConfig returns a default configuration for testing an agent
