@@ -19,19 +19,21 @@ import (
 
 // ServiceSummary is used to summarize a service
 type ServiceSummary struct {
-	Kind              structs.ServiceKind `json:",omitempty"`
-	Name              string
-	Datacenter        string
-	Tags              []string
-	Nodes             []string
-	ExternalSources   []string
-	externalSourceSet map[string]struct{} // internal to track uniqueness
-	checks            map[string]*structs.HealthCheck
-	InstanceCount     int
-	ChecksPassing     int
-	ChecksWarning     int
-	ChecksCritical    int
-	GatewayConfig     GatewayConfig
+	Kind                structs.ServiceKind `json:",omitempty"`
+	Name                string
+	Datacenter          string
+	Tags                []string
+	Nodes               []string
+	ExternalSources     []string
+	externalSourceSet   map[string]struct{} // internal to track uniqueness
+	checks              map[string]*structs.HealthCheck
+	InstanceCount       int
+	ChecksPassing       int
+	ChecksWarning       int
+	ChecksCritical      int
+	GatewayConfig       GatewayConfig
+	TransparentProxy    bool
+	transparentProxySet bool
 
 	structs.EnterpriseMeta
 }
@@ -61,14 +63,16 @@ type ServiceListingSummary struct {
 type ServiceTopologySummary struct {
 	ServiceSummary
 
+	Source    string
 	Intention structs.IntentionDecisionSummary
 }
 
 type ServiceTopology struct {
-	Protocol       string
-	Upstreams      []*ServiceTopologySummary
-	Downstreams    []*ServiceTopologySummary
-	FilteredByACLs bool
+	Protocol         string
+	TransparentProxy bool
+	Upstreams        []*ServiceTopologySummary
+	Downstreams      []*ServiceTopologySummary
+	FilteredByACLs   bool
 }
 
 // UINodes is used to list the nodes in a given datacenter. We return a
@@ -261,7 +265,12 @@ RPC:
 	}
 
 	summaries, _ := summarizeServices(out.Dump, s.agent.config, args.Datacenter)
-	return prepSummaryOutput(summaries, false), nil
+
+	prepped := prepSummaryOutput(summaries, false)
+	if prepped == nil {
+		prepped = make([]*ServiceSummary, 0)
+	}
+	return prepped, nil
 }
 
 // UIServiceTopology returns the list of upstreams and downstreams for a Connect enabled service.
@@ -329,9 +338,25 @@ RPC:
 		sum := ServiceTopologySummary{
 			ServiceSummary: *svc,
 			Intention:      out.ServiceTopology.UpstreamDecisions[sn.String()],
+			Source:         out.ServiceTopology.UpstreamSources[sn.String()],
 		}
 		upstreamResp = append(upstreamResp, &sum)
 	}
+	for k, v := range out.ServiceTopology.UpstreamSources {
+		if v == structs.TopologySourceRoutingConfig {
+			sn := structs.ServiceNameFromString(k)
+			sum := ServiceTopologySummary{
+				ServiceSummary: ServiceSummary{
+					Datacenter:     args.Datacenter,
+					Name:           sn.Name,
+					EnterpriseMeta: sn.EnterpriseMeta,
+				},
+				Intention: out.ServiceTopology.UpstreamDecisions[sn.String()],
+				Source:    out.ServiceTopology.UpstreamSources[sn.String()],
+			}
+			upstreamResp = append(upstreamResp, &sum)
+		}
+	}
 
 	sortedDownstreams := prepSummaryOutput(downstreams, true)
 	for _, svc := range sortedDownstreams {
@@ -339,15 +364,17 @@ RPC:
 		sum := ServiceTopologySummary{
 			ServiceSummary: *svc,
 			Intention:      out.ServiceTopology.DownstreamDecisions[sn.String()],
+			Source:         out.ServiceTopology.DownstreamSources[sn.String()],
 		}
 		downstreamResp = append(downstreamResp, &sum)
 	}
 
 	topo := ServiceTopology{
-		Protocol:       out.ServiceTopology.MetricsProtocol,
-		Upstreams:      upstreamResp,
-		Downstreams:    downstreamResp,
-		FilteredByACLs: out.FilteredByACLs,
+		TransparentProxy: out.ServiceTopology.TransparentProxy,
+		Protocol:         out.ServiceTopology.MetricsProtocol,
+		Upstreams:        upstreamResp,
+		Downstreams:      downstreamResp,
+		FilteredByACLs:   out.FilteredByACLs,
 	}
 	return topo, nil
 }
@@ -405,6 +432,17 @@ func summarizeServices(dump structs.ServiceDump, cfg *config.RuntimeConfig, dc s
 				}
 				destination.checks[uid] = check
 			}
+
+			// Only consider the target service to be transparent when all its proxy instances are in that mode.
+			// This is done because the flag is used to display warnings about proxies needing to enable
+			// transparent proxy mode. If ANY instance isn't in the right mode then the warming applies.
+			if svc.Proxy.Mode == structs.ProxyModeTransparent && !destination.transparentProxySet {
+				destination.TransparentProxy = true
+			}
+			if svc.Proxy.Mode != structs.ProxyModeTransparent {
+				destination.TransparentProxy = false
+			}
+			destination.transparentProxySet = true
 		}
 		for _, tag := range svc.Tags {
 			found := false
@@ -449,6 +487,8 @@ func summarizeServices(dump structs.ServiceDump, cfg *config.RuntimeConfig, dc s
 
 func prepSummaryOutput(summaries map[structs.ServiceName]*ServiceSummary, excludeSidecars bool) []*ServiceSummary {
 	var resp []*ServiceSummary
+	// Ensure at least a zero length slice
+	resp = make([]*ServiceSummary, 0)
 
 	// Collect and sort resp for display
 	for _, sum := range summaries {
@@ -665,7 +705,11 @@ func (s *HTTPHandlers) UIMetricsProxy(resp http.ResponseWriter, req *http.Reques
 
 	// Add any configured headers
 	for _, h := range cfg.AddHeaders {
-		req.Header.Set(h.Name, h.Value)
+		if strings.ToLower(h.Name) == "host" {
+			req.Host = h.Value
+		} else {
+			req.Header.Set(h.Name, h.Value)
+		}
 	}
 
 	log.Debug("proxying request", "to", u.String())
