@@ -12,18 +12,23 @@ import (
 	"sync/atomic"
 	"testing"
 
+	cleanhttp "github.com/hashicorp/go-cleanhttp"
+	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
+
 	"github.com/hashicorp/consul/agent/config"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/sdk/testutil/retry"
 	"github.com/hashicorp/consul/testrpc"
-	cleanhttp "github.com/hashicorp/go-cleanhttp"
-	"github.com/stretchr/testify/assert"
-	"github.com/stretchr/testify/require"
 )
 
 func TestUiIndex(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	// Make a test dir to serve UI files
 	uiDir := testutil.TempDir(t, "consul")
@@ -70,6 +75,10 @@ func TestUiIndex(t *testing.T) {
 }
 
 func TestUiNodes(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := NewTestAgent(t, "")
 	defer a.Shutdown()
@@ -108,6 +117,10 @@ func TestUiNodes(t *testing.T) {
 }
 
 func TestUiNodes_Filter(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := NewTestAgent(t, "")
 	defer a.Shutdown()
@@ -150,6 +163,10 @@ func TestUiNodes_Filter(t *testing.T) {
 }
 
 func TestUiNodeInfo(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := NewTestAgent(t, "")
 	defer a.Shutdown()
@@ -198,6 +215,10 @@ func TestUiNodeInfo(t *testing.T) {
 }
 
 func TestUiServices(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := NewTestAgent(t, "")
 	defer a.Shutdown()
@@ -376,6 +397,7 @@ func TestUiServices(t *testing.T) {
 
 		// internal accounting that users don't see can be blown away
 		for _, sum := range summary {
+			sum.transparentProxySet = false
 			sum.externalSourceSet = nil
 			sum.checks = nil
 		}
@@ -546,6 +568,10 @@ func TestUiServices(t *testing.T) {
 }
 
 func TestUIGatewayServiceNodes_Terminating(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := NewTestAgent(t, "")
@@ -605,7 +631,18 @@ func TestUIGatewayServiceNodes_Terminating(t *testing.T) {
 			},
 		}
 		require.NoError(t, a.RPC("Catalog.Register", &arg, &regOutput))
+	}
 
+	{
+		// Request without having registered the config-entry, shouldn't respond with null
+		req, _ := http.NewRequest("GET", "/v1/internal/ui/gateway-services-nodes/terminating-gateway", nil)
+		resp := httptest.NewRecorder()
+		obj, err := a.srv.UIGatewayServicesNodes(resp, req)
+		require.Nil(t, err)
+		require.NotNil(t, obj)
+	}
+
+	{
 		// Register terminating-gateway config entry, linking it to db and redis (does not exist)
 		args := &structs.TerminatingGatewayConfigEntry{
 			Name: "terminating-gateway",
@@ -669,6 +706,10 @@ func TestUIGatewayServiceNodes_Terminating(t *testing.T) {
 }
 
 func TestUIGatewayServiceNodes_Ingress(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := NewTestAgent(t, `alt_domain = "alt.consul."`)
@@ -842,6 +883,10 @@ func TestUIGatewayServiceNodes_Ingress(t *testing.T) {
 }
 
 func TestUIGatewayIntentions(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := NewTestAgent(t, "")
@@ -951,6 +996,10 @@ func TestUIEndpoint_modifySummaryForGatewayService_UseRequestedDCInsteadOfConfig
 }
 
 func TestUIServiceTopology(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := NewTestAgent(t, "")
@@ -1031,12 +1080,7 @@ func TestUIServiceTopology(t *testing.T) {
 					Address: "198.18.1.2",
 					Proxy: structs.ConnectProxyConfig{
 						DestinationServiceName: "api",
-						Upstreams: structs.Upstreams{
-							{
-								DestinationName: "web",
-								LocalBindPort:   8080,
-							},
-						},
+						Mode:                   structs.ProxyModeTransparent,
 					},
 				},
 				Checks: structs.HealthChecks{
@@ -1163,12 +1207,7 @@ func TestUIServiceTopology(t *testing.T) {
 					Address: "198.18.1.40",
 					Proxy: structs.ConnectProxyConfig{
 						DestinationServiceName: "web",
-						Upstreams: structs.Upstreams{
-							{
-								DestinationName: "redis",
-								LocalBindPort:   123,
-							},
-						},
+						Mode:                   structs.ProxyModeTransparent,
 					},
 				},
 				Checks: structs.HealthChecks{
@@ -1249,8 +1288,10 @@ func TestUIServiceTopology(t *testing.T) {
 		}
 	}
 
-	// Add intentions: deny all, ingress -> api, web -> redis with L7 perms, but omit intention for api -> web
-	// Add ingress config: ingress -> api
+	// ingress -> api gateway config entry (but no intention)
+	// wildcard deny intention
+	// api -> web exact intention
+	// web -> redis exact intention
 	{
 		entries := []structs.ConfigEntryRequest{
 			{
@@ -1271,6 +1312,20 @@ func TestUIServiceTopology(t *testing.T) {
 					Protocol: "tcp",
 				},
 			},
+			{
+				Datacenter: "dc1",
+				Entry: &structs.ServiceIntentionsConfigEntry{
+					Kind: structs.ServiceIntentions,
+					Name: "*",
+					Meta: map[string]string{structs.MetaExternalSource: "nomad"},
+					Sources: []*structs.SourceIntention{
+						{
+							Name:   "*",
+							Action: structs.IntentionActionDeny,
+						},
+					},
+				},
+			},
 			{
 				Datacenter: "dc1",
 				Entry: &structs.ServiceIntentionsConfigEntry{
@@ -1295,12 +1350,11 @@ func TestUIServiceTopology(t *testing.T) {
 				Datacenter: "dc1",
 				Entry: &structs.ServiceIntentionsConfigEntry{
 					Kind: structs.ServiceIntentions,
-					Name: "*",
-					Meta: map[string]string{structs.MetaExternalSource: "nomad"},
+					Name: "web",
 					Sources: []*structs.SourceIntention{
 						{
-							Name:   "*",
-							Action: structs.IntentionActionDeny,
+							Action: structs.IntentionActionAllow,
+							Name:   "api",
 						},
 					},
 				},
@@ -1344,96 +1398,204 @@ func TestUIServiceTopology(t *testing.T) {
 		}
 	}
 
-	t.Run("request without kind", func(t *testing.T) {
-		req, _ := http.NewRequest("GET", "/v1/internal/ui/service-topology/ingress", nil)
-		resp := httptest.NewRecorder()
-		obj, err := a.srv.UIServiceTopology(resp, req)
-		require.Nil(t, err)
-		require.Nil(t, obj)
-		require.Equal(t, "Missing service kind", resp.Body.String())
-	})
+	type testCase struct {
+		name    string
+		httpReq *http.Request
+		want    *ServiceTopology
+		wantErr string
+	}
 
-	t.Run("request with unsupported kind", func(t *testing.T) {
-		req, _ := http.NewRequest("GET", "/v1/internal/ui/service-topology/ingress?kind=not-a-kind", nil)
-		resp := httptest.NewRecorder()
-		obj, err := a.srv.UIServiceTopology(resp, req)
-		require.Nil(t, err)
-		require.Nil(t, obj)
-		require.Equal(t, `Unsupported service kind "not-a-kind"`, resp.Body.String())
-	})
-
-	t.Run("ingress", func(t *testing.T) {
+	run := func(t *testing.T, tc testCase) {
 		retry.Run(t, func(r *retry.R) {
-			// Request topology for ingress
-			req, _ := http.NewRequest("GET", "/v1/internal/ui/service-topology/ingress?kind=ingress-gateway", nil)
 			resp := httptest.NewRecorder()
-			obj, err := a.srv.UIServiceTopology(resp, req)
+			obj, err := a.srv.UIServiceTopology(resp, tc.httpReq)
 			assert.Nil(r, err)
-			require.NoError(r, checkIndex(resp))
 
-			expect := ServiceTopology{
-				Protocol: "tcp",
+			if tc.wantErr != "" {
+				assert.Nil(r, tc.want) // should not define a non-nil want
+				require.Equal(r, tc.wantErr, resp.Body.String())
+				require.Nil(r, obj)
+				return
+			}
+
+			require.NoError(r, checkIndex(resp))
+			require.NotNil(r, obj)
+			result := obj.(ServiceTopology)
+			clearUnexportedFields(result)
+
+			require.Equal(r, *tc.want, result)
+		})
+	}
+
+	tcs := []testCase{
+		{
+			name: "request without kind",
+			httpReq: func() *http.Request {
+				req, _ := http.NewRequest("GET", "/v1/internal/ui/service-topology/ingress", nil)
+				return req
+			}(),
+			wantErr: "Missing service kind",
+		},
+		{
+			name: "request with unsupported kind",
+			httpReq: func() *http.Request {
+				req, _ := http.NewRequest("GET", "/v1/internal/ui/service-topology/ingress?kind=not-a-kind", nil)
+				return req
+			}(),
+			wantErr: `Unsupported service kind "not-a-kind"`,
+		},
+		{
+			name: "ingress",
+			httpReq: func() *http.Request {
+				req, _ := http.NewRequest("GET", "/v1/internal/ui/service-topology/ingress?kind=ingress-gateway", nil)
+				return req
+			}(),
+			want: &ServiceTopology{
+				Protocol:         "tcp",
+				TransparentProxy: false,
 				Upstreams: []*ServiceTopologySummary{
 					{
 						ServiceSummary: ServiceSummary{
-							Name:           "api",
-							Datacenter:     "dc1",
-							Nodes:          []string{"foo"},
-							InstanceCount:  1,
-							ChecksPassing:  3,
-							EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
+							Name:             "api",
+							Datacenter:       "dc1",
+							Nodes:            []string{"foo"},
+							InstanceCount:    1,
+							ChecksPassing:    3,
+							EnterpriseMeta:   *structs.DefaultEnterpriseMeta(),
+							TransparentProxy: true,
 						},
 						Intention: structs.IntentionDecisionSummary{
+							DefaultAllow:   true,
 							Allowed:        true,
 							HasPermissions: false,
 							HasExact:       true,
 						},
+						Source: structs.TopologySourceRegistration,
 					},
 				},
 				Downstreams:    []*ServiceTopologySummary{},
 				FilteredByACLs: false,
-			}
-			result := obj.(ServiceTopology)
-
-			// Internal accounting that is not returned in JSON response
-			for _, u := range result.Upstreams {
-				u.externalSourceSet = nil
-				u.checks = nil
-			}
-			require.Equal(r, expect, result)
-		})
-	})
-
-	t.Run("api", func(t *testing.T) {
-		retry.Run(t, func(r *retry.R) {
-			// Request topology for api
-			req, _ := http.NewRequest("GET", "/v1/internal/ui/service-topology/api?kind=", nil)
-			resp := httptest.NewRecorder()
-			obj, err := a.srv.UIServiceTopology(resp, req)
-			assert.Nil(r, err)
-			require.NoError(r, checkIndex(resp))
-
-			expect := ServiceTopology{
-				Protocol: "tcp",
+			},
+		},
+		{
+			name: "api",
+			httpReq: func() *http.Request {
+				req, _ := http.NewRequest("GET", "/v1/internal/ui/service-topology/api?kind=", nil)
+				return req
+			}(),
+			want: &ServiceTopology{
+				Protocol:         "tcp",
+				TransparentProxy: true,
 				Downstreams: []*ServiceTopologySummary{
 					{
 						ServiceSummary: ServiceSummary{
-							Name:           "ingress",
-							Kind:           structs.ServiceKindIngressGateway,
-							Datacenter:     "dc1",
-							Nodes:          []string{"edge"},
-							InstanceCount:  1,
-							ChecksPassing:  1,
-							EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
+							Name:             "ingress",
+							Kind:             structs.ServiceKindIngressGateway,
+							Datacenter:       "dc1",
+							Nodes:            []string{"edge"},
+							InstanceCount:    1,
+							ChecksPassing:    1,
+							EnterpriseMeta:   *structs.DefaultEnterpriseMeta(),
+							TransparentProxy: false,
 						},
 						Intention: structs.IntentionDecisionSummary{
+							DefaultAllow:   true,
 							Allowed:        true,
 							HasPermissions: false,
 							HasExact:       true,
 						},
+						Source: structs.TopologySourceRegistration,
 					},
 				},
 				Upstreams: []*ServiceTopologySummary{
+					{
+						ServiceSummary: ServiceSummary{
+							Name:             "web",
+							Datacenter:       "dc1",
+							Nodes:            []string{"bar", "baz"},
+							InstanceCount:    2,
+							ChecksPassing:    3,
+							ChecksWarning:    1,
+							ChecksCritical:   2,
+							EnterpriseMeta:   *structs.DefaultEnterpriseMeta(),
+							TransparentProxy: false,
+						},
+						Intention: structs.IntentionDecisionSummary{
+							DefaultAllow:   true,
+							Allowed:        true,
+							HasPermissions: false,
+							HasExact:       true,
+						},
+						Source: structs.TopologySourceSpecificIntention,
+					},
+				},
+				FilteredByACLs: false,
+			},
+		},
+		{
+			name: "web",
+			httpReq: func() *http.Request {
+				req, _ := http.NewRequest("GET", "/v1/internal/ui/service-topology/web?kind=", nil)
+				return req
+			}(),
+			want: &ServiceTopology{
+				Protocol:         "http",
+				TransparentProxy: false,
+				Upstreams: []*ServiceTopologySummary{
+					{
+						ServiceSummary: ServiceSummary{
+							Name:             "redis",
+							Datacenter:       "dc1",
+							Nodes:            []string{"zip"},
+							InstanceCount:    1,
+							ChecksPassing:    2,
+							ChecksCritical:   1,
+							EnterpriseMeta:   *structs.DefaultEnterpriseMeta(),
+							TransparentProxy: false,
+						},
+						Intention: structs.IntentionDecisionSummary{
+							DefaultAllow:   true,
+							Allowed:        false,
+							HasPermissions: true,
+							HasExact:       true,
+						},
+						Source: structs.TopologySourceRegistration,
+					},
+				},
+				Downstreams: []*ServiceTopologySummary{
+					{
+						ServiceSummary: ServiceSummary{
+							Name:             "api",
+							Datacenter:       "dc1",
+							Nodes:            []string{"foo"},
+							InstanceCount:    1,
+							ChecksPassing:    3,
+							EnterpriseMeta:   *structs.DefaultEnterpriseMeta(),
+							TransparentProxy: true,
+						},
+						Intention: structs.IntentionDecisionSummary{
+							DefaultAllow:   true,
+							Allowed:        true,
+							HasPermissions: false,
+							HasExact:       true,
+						},
+						Source: structs.TopologySourceSpecificIntention,
+					},
+				},
+				FilteredByACLs: false,
+			},
+		},
+		{
+			name: "redis",
+			httpReq: func() *http.Request {
+				req, _ := http.NewRequest("GET", "/v1/internal/ui/service-topology/redis?kind=", nil)
+				return req
+			}(),
+			want: &ServiceTopology{
+				Protocol:         "http",
+				TransparentProxy: false,
+				Upstreams:        []*ServiceTopologySummary{},
+				Downstreams: []*ServiceTopologySummary{
 					{
 						ServiceSummary: ServiceSummary{
 							Name:           "web",
@@ -1446,144 +1608,344 @@ func TestUIServiceTopology(t *testing.T) {
 							EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
 						},
 						Intention: structs.IntentionDecisionSummary{
-							Allowed:        false,
-							HasPermissions: false,
-							ExternalSource: "nomad",
-
-							// From wildcard deny
-							HasExact: false,
-						},
-					},
-				},
-				FilteredByACLs: false,
-			}
-			result := obj.(ServiceTopology)
-
-			// Internal accounting that is not returned in JSON response
-			for _, u := range result.Upstreams {
-				u.externalSourceSet = nil
-				u.checks = nil
-			}
-			for _, d := range result.Downstreams {
-				d.externalSourceSet = nil
-				d.checks = nil
-			}
-			require.Equal(r, expect, result)
-		})
-	})
-
-	t.Run("web", func(t *testing.T) {
-		retry.Run(t, func(r *retry.R) {
-			// Request topology for web
-			req, _ := http.NewRequest("GET", "/v1/internal/ui/service-topology/web?kind=", nil)
-			resp := httptest.NewRecorder()
-			obj, err := a.srv.UIServiceTopology(resp, req)
-			assert.Nil(r, err)
-			require.NoError(r, checkIndex(resp))
-
-			expect := ServiceTopology{
-				Protocol: "http",
-				Upstreams: []*ServiceTopologySummary{
-					{
-						ServiceSummary: ServiceSummary{
-							Name:           "redis",
-							Datacenter:     "dc1",
-							Nodes:          []string{"zip"},
-							InstanceCount:  1,
-							ChecksPassing:  2,
-							ChecksCritical: 1,
-							EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
-						},
-						Intention: structs.IntentionDecisionSummary{
+							DefaultAllow:   true,
 							Allowed:        false,
 							HasPermissions: true,
 							HasExact:       true,
 						},
-					},
-				},
-				Downstreams: []*ServiceTopologySummary{
-					{
-						ServiceSummary: ServiceSummary{
-							Name:           "api",
-							Datacenter:     "dc1",
-							Nodes:          []string{"foo"},
-							InstanceCount:  1,
-							ChecksPassing:  3,
-							EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
-						},
-						Intention: structs.IntentionDecisionSummary{
-							Allowed:        false,
-							HasPermissions: false,
-							ExternalSource: "nomad",
-
-							// From wildcard deny
-							HasExact: false,
-						},
+						Source: structs.TopologySourceRegistration,
 					},
 				},
 				FilteredByACLs: false,
-			}
-			result := obj.(ServiceTopology)
+			},
+		},
+	}
 
-			// Internal accounting that is not returned in JSON response
-			for _, u := range result.Upstreams {
-				u.externalSourceSet = nil
-				u.checks = nil
-			}
-			for _, d := range result.Downstreams {
-				d.externalSourceSet = nil
-				d.checks = nil
-			}
-			require.Equal(r, expect, result)
+	for _, tc := range tcs {
+		t.Run(tc.name, func(t *testing.T) {
+			run(t, tc)
 		})
-	})
+	}
+}
 
-	t.Run("redis", func(t *testing.T) {
+// clearUnexportedFields sets unexported members of ServiceTopology to their
+// type defaults, since the fields are not marshalled in the JSON response.
+func clearUnexportedFields(result ServiceTopology) {
+	for _, u := range result.Upstreams {
+		u.transparentProxySet = false
+		u.externalSourceSet = nil
+		u.checks = nil
+	}
+	for _, d := range result.Downstreams {
+		d.transparentProxySet = false
+		d.externalSourceSet = nil
+		d.checks = nil
+	}
+}
+
+func TestUIServiceTopology_RoutingConfigs(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
+	t.Parallel()
+
+	a := NewTestAgent(t, "")
+	defer a.Shutdown()
+
+	// Register dashboard -> routing-config -> { counting, counting-v2 }
+	{
+		registrations := map[string]*structs.RegisterRequest{
+			"Service dashboard": {
+				Datacenter:     "dc1",
+				Node:           "foo",
+				SkipNodeUpdate: true,
+				Service: &structs.NodeService{
+					Kind:    structs.ServiceKindTypical,
+					ID:      "dashboard",
+					Service: "dashboard",
+					Port:    9002,
+				},
+				Checks: structs.HealthChecks{
+					&structs.HealthCheck{
+						Node:        "foo",
+						CheckID:     "foo:dashboard",
+						Status:      api.HealthPassing,
+						ServiceID:   "dashboard",
+						ServiceName: "dashboard",
+					},
+				},
+			},
+			"Service dashboard-proxy": {
+				Datacenter:     "dc1",
+				Node:           "foo",
+				SkipNodeUpdate: true,
+				Service: &structs.NodeService{
+					Kind:    structs.ServiceKindConnectProxy,
+					ID:      "dashboard-sidecar-proxy",
+					Service: "dashboard-sidecar-proxy",
+					Port:    5000,
+					Address: "198.18.1.0",
+					Proxy: structs.ConnectProxyConfig{
+						DestinationServiceName: "dashboard",
+						DestinationServiceID:   "dashboard",
+						LocalServiceAddress:    "127.0.0.1",
+						LocalServicePort:       9002,
+						Upstreams: []structs.Upstream{
+							{
+								DestinationType: "service",
+								DestinationName: "routing-config",
+								LocalBindPort:   5000,
+							},
+						},
+					},
+					LocallyRegisteredAsSidecar: true,
+				},
+			},
+			"Service counting": {
+				Datacenter:     "dc1",
+				Node:           "foo",
+				SkipNodeUpdate: true,
+				Service: &structs.NodeService{
+					Kind:    structs.ServiceKindTypical,
+					ID:      "counting",
+					Service: "counting",
+					Port:    9003,
+					Address: "198.18.1.1",
+				},
+				Checks: structs.HealthChecks{
+					&structs.HealthCheck{
+						Node:        "foo",
+						CheckID:     "foo:api",
+						Status:      api.HealthPassing,
+						ServiceID:   "counting",
+						ServiceName: "counting",
+					},
+				},
+			},
+			"Service counting-proxy": {
+				Datacenter:     "dc1",
+				Node:           "foo",
+				SkipNodeUpdate: true,
+				Service: &structs.NodeService{
+					Kind:    structs.ServiceKindConnectProxy,
+					ID:      "counting-proxy",
+					Service: "counting-proxy",
+					Port:    5001,
+					Address: "198.18.1.1",
+					Proxy: structs.ConnectProxyConfig{
+						DestinationServiceName: "counting",
+					},
+					LocallyRegisteredAsSidecar: true,
+				},
+				Checks: structs.HealthChecks{
+					&structs.HealthCheck{
+						Node:        "foo",
+						CheckID:     "foo:counting-proxy",
+						Status:      api.HealthPassing,
+						ServiceID:   "counting-proxy",
+						ServiceName: "counting-proxy",
+					},
+				},
+			},
+			"Service counting-v2": {
+				Datacenter:     "dc1",
+				Node:           "foo",
+				SkipNodeUpdate: true,
+				Service: &structs.NodeService{
+					Kind:    structs.ServiceKindTypical,
+					ID:      "counting-v2",
+					Service: "counting-v2",
+					Port:    9004,
+					Address: "198.18.1.2",
+				},
+				Checks: structs.HealthChecks{
+					&structs.HealthCheck{
+						Node:        "foo",
+						CheckID:     "foo:api",
+						Status:      api.HealthPassing,
+						ServiceID:   "counting-v2",
+						ServiceName: "counting-v2",
+					},
+				},
+			},
+			"Service counting-v2-proxy": {
+				Datacenter:     "dc1",
+				Node:           "foo",
+				SkipNodeUpdate: true,
+				Service: &structs.NodeService{
+					Kind:    structs.ServiceKindConnectProxy,
+					ID:      "counting-v2-proxy",
+					Service: "counting-v2-proxy",
+					Port:    5002,
+					Address: "198.18.1.2",
+					Proxy: structs.ConnectProxyConfig{
+						DestinationServiceName: "counting-v2",
+					},
+					LocallyRegisteredAsSidecar: true,
+				},
+				Checks: structs.HealthChecks{
+					&structs.HealthCheck{
+						Node:        "foo",
+						CheckID:     "foo:counting-v2-proxy",
+						Status:      api.HealthPassing,
+						ServiceID:   "counting-v2-proxy",
+						ServiceName: "counting-v2-proxy",
+					},
+				},
+			},
+		}
+		for _, args := range registrations {
+			var out struct{}
+			require.NoError(t, a.RPC("Catalog.Register", args, &out))
+		}
+	}
+	{
+		entries := []structs.ConfigEntryRequest{
+			{
+				Datacenter: "dc1",
+				Entry: &structs.ProxyConfigEntry{
+					Kind: structs.ProxyDefaults,
+					Name: structs.ProxyConfigGlobal,
+					Config: map[string]interface{}{
+						"protocol": "http",
+					},
+				},
+			},
+			{
+				Datacenter: "dc1",
+				Entry: &structs.ServiceRouterConfigEntry{
+					Kind: structs.ServiceRouter,
+					Name: "routing-config",
+					Routes: []structs.ServiceRoute{
+						{
+							Match: &structs.ServiceRouteMatch{
+								HTTP: &structs.ServiceRouteHTTPMatch{
+									PathPrefix: "/v2",
+								},
+							},
+							Destination: &structs.ServiceRouteDestination{
+								Service: "counting-v2",
+							},
+						},
+						{
+							Match: &structs.ServiceRouteMatch{
+								HTTP: &structs.ServiceRouteHTTPMatch{
+									PathPrefix: "/",
+								},
+							},
+							Destination: &structs.ServiceRouteDestination{
+								Service: "counting",
+							},
+						},
+					},
+				},
+			},
+		}
+		for _, req := range entries {
+			out := false
+			require.NoError(t, a.RPC("ConfigEntry.Apply", &req, &out))
+		}
+	}
+
+	type testCase struct {
+		name    string
+		httpReq *http.Request
+		want    *ServiceTopology
+		wantErr string
+	}
+
+	run := func(t *testing.T, tc testCase) {
 		retry.Run(t, func(r *retry.R) {
-			// Request topology for redis
-			req, _ := http.NewRequest("GET", "/v1/internal/ui/service-topology/redis?kind=", nil)
 			resp := httptest.NewRecorder()
-			obj, err := a.srv.UIServiceTopology(resp, req)
+			obj, err := a.srv.UIServiceTopology(resp, tc.httpReq)
 			assert.Nil(r, err)
-			require.NoError(r, checkIndex(resp))
 
-			expect := ServiceTopology{
+			if tc.wantErr != "" {
+				assert.Nil(r, tc.want) // should not define a non-nil want
+				require.Equal(r, tc.wantErr, resp.Body.String())
+				require.Nil(r, obj)
+				return
+			}
+
+			require.NoError(r, checkIndex(resp))
+			require.NotNil(r, obj)
+			result := obj.(ServiceTopology)
+			clearUnexportedFields(result)
+
+			require.Equal(r, *tc.want, result)
+		})
+	}
+
+	tcs := []testCase{
+		{
+			name: "dashboard has upstream routing-config",
+			httpReq: func() *http.Request {
+				req, _ := http.NewRequest("GET", "/v1/internal/ui/service-topology/dashboard?kind=", nil)
+				return req
+			}(),
+			want: &ServiceTopology{
+				Protocol:    "http",
+				Downstreams: []*ServiceTopologySummary{},
+				Upstreams: []*ServiceTopologySummary{
+					{
+						ServiceSummary: ServiceSummary{
+							Name:             "routing-config",
+							Datacenter:       "dc1",
+							EnterpriseMeta:   *structs.DefaultEnterpriseMeta(),
+							TransparentProxy: false,
+						},
+						Intention: structs.IntentionDecisionSummary{
+							DefaultAllow: true,
+							Allowed:      true,
+						},
+						Source: structs.TopologySourceRoutingConfig,
+					},
+				},
+			},
+		},
+		{
+			name: "counting has downstream dashboard",
+			httpReq: func() *http.Request {
+				req, _ := http.NewRequest("GET", "/v1/internal/ui/service-topology/counting?kind=", nil)
+				return req
+			}(),
+			want: &ServiceTopology{
 				Protocol:  "http",
 				Upstreams: []*ServiceTopologySummary{},
 				Downstreams: []*ServiceTopologySummary{
 					{
 						ServiceSummary: ServiceSummary{
-							Name:           "web",
+							Name:           "dashboard",
 							Datacenter:     "dc1",
-							Nodes:          []string{"bar", "baz"},
-							InstanceCount:  2,
-							ChecksPassing:  3,
-							ChecksWarning:  1,
-							ChecksCritical: 2,
+							Nodes:          []string{"foo"},
 							EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
+							InstanceCount:  1,
+							ChecksPassing:  1,
 						},
+						Source: "proxy-registration",
 						Intention: structs.IntentionDecisionSummary{
-							Allowed:        false,
-							HasPermissions: true,
-							HasExact:       true,
+							Allowed:      true,
+							DefaultAllow: true,
 						},
 					},
 				},
-				FilteredByACLs: false,
-			}
-			result := obj.(ServiceTopology)
+			},
+		},
+	}
 
-			// Internal accounting that is not returned in JSON response
-			for _, d := range result.Downstreams {
-				d.externalSourceSet = nil
-				d.checks = nil
-			}
-			require.Equal(r, expect, result)
+	for _, tc := range tcs {
+		t.Run(tc.name, func(t *testing.T) {
+			run(t, tc)
 		})
-	})
+	}
 }
 
 func TestUIEndpoint_MetricsProxy(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	var lastHeadersSent atomic.Value
