@@ -8,18 +8,25 @@ import (
 	"text/template"
 	"time"
 
-	envoy "github.com/envoyproxy/go-control-plane/envoy/api/v2"
+	envoy_cluster_v3 "github.com/envoyproxy/go-control-plane/envoy/config/cluster/v3"
+
 	"github.com/golang/protobuf/ptypes/wrappers"
 	testinf "github.com/mitchellh/go-testing-interface"
 	"github.com/stretchr/testify/require"
 
+	"github.com/hashicorp/consul/agent/connect"
+	"github.com/hashicorp/consul/agent/consul/discoverychain"
 	"github.com/hashicorp/consul/agent/proxycfg"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/agent/xds/proxysupport"
+	"github.com/hashicorp/consul/lib/stringslice"
 	"github.com/hashicorp/consul/sdk/testutil"
 )
 
 func TestClustersFromSnapshot(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
 
 	tests := []struct {
 		name   string
@@ -41,20 +48,7 @@ func TestClustersFromSnapshot(t *testing.T) {
 			setup: func(snap *proxycfg.ConfigSnapshot) {
 				snap.Proxy.Config["envoy_local_cluster_json"] =
 					customAppClusterJSON(t, customClusterJSONOptions{
-						Name:        "mylocal",
-						IncludeType: false,
-					})
-			},
-		},
-		{
-			name:               "custom-local-app-typed",
-			create:             proxycfg.TestConfigSnapshot,
-			overrideGoldenName: "custom-local-app",
-			setup: func(snap *proxycfg.ConfigSnapshot) {
-				snap.Proxy.Config["envoy_local_cluster_json"] =
-					customAppClusterJSON(t, customClusterJSONOptions{
-						Name:        "mylocal",
-						IncludeType: true,
+						Name: "mylocal",
 					})
 			},
 		},
@@ -64,8 +58,7 @@ func TestClustersFromSnapshot(t *testing.T) {
 			setup: func(snap *proxycfg.ConfigSnapshot) {
 				snap.Proxy.Upstreams[0].Config["envoy_cluster_json"] =
 					customAppClusterJSON(t, customClusterJSONOptions{
-						Name:        "myservice",
-						IncludeType: false,
+						Name: "myservice",
 					})
 			},
 		},
@@ -75,21 +68,17 @@ func TestClustersFromSnapshot(t *testing.T) {
 			setup: func(snap *proxycfg.ConfigSnapshot) {
 				snap.Proxy.Upstreams[0].Config["envoy_cluster_json"] =
 					customAppClusterJSON(t, customClusterJSONOptions{
-						Name:        "myservice",
-						IncludeType: false,
-					})
-			},
-		},
-		{
-			name:               "custom-upstream-typed",
-			create:             proxycfg.TestConfigSnapshot,
-			overrideGoldenName: "custom-upstream",
-			setup: func(snap *proxycfg.ConfigSnapshot) {
-				snap.Proxy.Upstreams[0].Config["envoy_cluster_json"] =
-					customAppClusterJSON(t, customClusterJSONOptions{
-						Name:        "myservice",
-						IncludeType: true,
+						Name: "myservice",
 					})
+				snap.ConnectProxy.UpstreamConfig = map[string]*structs.Upstream{
+					"db": {
+						Config: map[string]interface{}{
+							"envoy_cluster_json": customAppClusterJSON(t, customClusterJSONOptions{
+								Name: "myservice",
+							}),
+						},
+					},
+				}
 			},
 		},
 		{
@@ -99,10 +88,9 @@ func TestClustersFromSnapshot(t *testing.T) {
 			setup: func(snap *proxycfg.ConfigSnapshot) {
 				snap.Proxy.Upstreams[0].Config["envoy_cluster_json"] =
 					customAppClusterJSON(t, customClusterJSONOptions{
-						Name:        "myservice",
-						IncludeType: true,
+						Name: "myservice",
 						// Attempt to override the TLS context should be ignored
-						TLSContext: `{"commonTlsContext": {}}`,
+						TLSContext: `"allowRenegotiation": false`,
 					})
 			},
 		},
@@ -241,6 +229,17 @@ func TestClustersFromSnapshot(t *testing.T) {
 			name:   "expose-paths-local-app-paths",
 			create: proxycfg.TestConfigSnapshotExposeConfig,
 		},
+		{
+			name:   "downstream-service-with-unix-sockets",
+			create: proxycfg.TestConfigSnapshot,
+			setup: func(snap *proxycfg.ConfigSnapshot) {
+				snap.Address = ""
+				snap.Port = 0
+				snap.Proxy.LocalServiceAddress = ""
+				snap.Proxy.LocalServicePort = 0
+				snap.Proxy.LocalServiceSocketPath = "/tmp/downstream_proxy.sock"
+			},
+		},
 		{
 			name:   "expose-paths-new-cluster-http2",
 			create: proxycfg.TestConfigSnapshotExposeConfig,
@@ -660,17 +659,106 @@ func TestClustersFromSnapshot(t *testing.T) {
 			create: proxycfg.TestConfigSnapshotIngress_MultipleListenersDuplicateService,
 			setup:  nil,
 		},
+		{
+			name:   "transparent-proxy",
+			create: proxycfg.TestConfigSnapshot,
+			setup: func(snap *proxycfg.ConfigSnapshot) {
+				snap.Proxy.Mode = structs.ProxyModeTransparent
+				snap.ConnectProxy.MeshConfigSet = true
+			},
+		},
+		{
+			name:   "transparent-proxy-catalog-destinations-only",
+			create: proxycfg.TestConfigSnapshot,
+			setup: func(snap *proxycfg.ConfigSnapshot) {
+				snap.Proxy.Mode = structs.ProxyModeTransparent
+
+				snap.ConnectProxy.MeshConfigSet = true
+				snap.ConnectProxy.MeshConfig = &structs.MeshConfigEntry{
+					TransparentProxy: structs.TransparentProxyMeshConfig{
+						MeshDestinationsOnly: true,
+					},
+				}
+			},
+		},
+		{
+			name:   "transparent-proxy-dial-instances-directly",
+			create: proxycfg.TestConfigSnapshot,
+			setup: func(snap *proxycfg.ConfigSnapshot) {
+				snap.Proxy.Mode = structs.ProxyModeTransparent
+
+				// We add a passthrough cluster for each upstream service name
+				snap.ConnectProxy.PassthroughUpstreams = map[string]map[string]map[string]struct{}{
+					"default/kafka": {
+						"kafka.default.dc1": map[string]struct{}{
+							"9.9.9.9": {},
+						},
+					},
+					"default/mongo": {
+						"mongo.default.dc1": map[string]struct{}{
+							"10.10.10.10": {},
+							"10.10.10.12": {},
+						},
+					},
+				}
+
+				// need a default chain to render kafka
+				snap.ConnectProxy.DiscoveryChain["default/kafka"] = discoverychain.TestCompileConfigEntries(t, "kafka", "default", "dc1", connect.TestClusterID+".consul", "dc1", nil)
+				snap.ConnectProxy.WatchedUpstreamEndpoints["default/kafka"] = map[string]structs.CheckServiceNodes{
+					"kafka.default.dc1": {
+						structs.CheckServiceNode{
+							Node: &structs.Node{
+								Datacenter: "dc1",
+							},
+							Service: &structs.NodeService{
+								Service: "kafka",
+								Address: "9.9.9.9",
+								Port:    9092,
+								Proxy: structs.ConnectProxyConfig{
+									TransparentProxy: structs.TransparentProxyConfig{
+										DialedDirectly: true,
+									},
+								},
+							},
+						},
+					},
+				}
+
+				// There should still be a cluster for non-passthrough requests
+				snap.ConnectProxy.DiscoveryChain["default/mongo"] = discoverychain.TestCompileConfigEntries(t, "mongo", "default", "dc1", connect.TestClusterID+".consul", "dc1", nil, &structs.ServiceResolverConfigEntry{
+					Kind:           structs.ServiceResolver,
+					Name:           "mongo",
+					ConnectTimeout: 33 * time.Second,
+				})
+				snap.ConnectProxy.WatchedUpstreamEndpoints["default/mongo"] = map[string]structs.CheckServiceNodes{
+					"mongo.default.dc1": {
+						structs.CheckServiceNode{
+							Node: &structs.Node{
+								Datacenter: "dc1",
+							},
+							Service: &structs.NodeService{
+								Service: "mongo",
+								Address: "7.7.7.7",
+								Port:    27017,
+								TaggedAddresses: map[string]structs.ServiceAddress{
+									"virtual": {Address: "6.6.6.6"},
+								},
+							},
+						},
+					},
+				}
+			},
+		},
 	}
 
 	latestEnvoyVersion := proxysupport.EnvoyVersions[0]
+	latestEnvoyVersion_v2 := proxysupport.EnvoyVersionsV2[0]
 	for _, envoyVersion := range proxysupport.EnvoyVersions {
 		sf, err := determineSupportedProxyFeaturesFromString(envoyVersion)
 		require.NoError(t, err)
 		t.Run("envoy-"+envoyVersion, func(t *testing.T) {
 			for _, tt := range tests {
 				t.Run(tt.name, func(t *testing.T) {
-					require := require.New(t)
-
 					// Sanity check default with no overrides first
 					snap := tt.create(t)
 
@@ -684,166 +772,91 @@ func TestClustersFromSnapshot(t *testing.T) {
 					}
 
 					// Need server just for logger dependency
-					s := Server{Logger: testutil.Logger(t)}
+					g := newResourceGenerator(testutil.Logger(t), nil, nil, false)
+					g.ProxyFeatures = sf
+
+					clusters, err := g.clustersFromSnapshot(snap)
+					require.NoError(t, err)
 
-					cInfo := connectionInfo{
-						Token:         "my-token",
-						ProxyFeatures: sf,
-					}
-					clusters, err := s.clustersFromSnapshot(cInfo, snap)
-					require.NoError(err)
 					sort.Slice(clusters, func(i, j int) bool {
-						return clusters[i].(*envoy.Cluster).Name < clusters[j].(*envoy.Cluster).Name
+						return clusters[i].(*envoy_cluster_v3.Cluster).Name < clusters[j].(*envoy_cluster_v3.Cluster).Name
 					})
+
 					r, err := createResponse(ClusterType, "00000001", "00000001", clusters)
-					require.NoError(err)
+					require.NoError(t, err)
 
-					gotJSON := responseToJSON(t, r)
+					t.Run("current", func(t *testing.T) {
+						gotJSON := protoToJSON(t, r)
 
-					gName := tt.name
-					if tt.overrideGoldenName != "" {
-						gName = tt.overrideGoldenName
-					}
+						gName := tt.name
+						if tt.overrideGoldenName != "" {
+							gName = tt.overrideGoldenName
+						}
 
-					require.JSONEq(goldenEnvoy(t, filepath.Join("clusters", gName), envoyVersion, latestEnvoyVersion, gotJSON), gotJSON)
+						require.JSONEq(t, goldenEnvoy(t, filepath.Join("clusters", gName), envoyVersion, latestEnvoyVersion, gotJSON), gotJSON)
+					})
+
+					t.Run("v2-compat", func(t *testing.T) {
+						if !stringslice.Contains(proxysupport.EnvoyVersionsV2, envoyVersion) {
+							t.Skip()
+						}
+						respV2, err := convertDiscoveryResponseToV2(r)
+						require.NoError(t, err)
+
+						gotJSON := protoToJSON(t, respV2)
+
+						gName := tt.name
+						if tt.overrideGoldenName != "" {
+							gName = tt.overrideGoldenName
+						}
+
+						gName += ".v2compat"
+
+						require.JSONEq(t, goldenEnvoy(t, filepath.Join("clusters", gName), envoyVersion, latestEnvoyVersion_v2, gotJSON), gotJSON)
+					})
 				})
 			}
 		})
 	}
 }
 
-func expectClustersJSONResources(snap *proxycfg.ConfigSnapshot) map[string]string {
-	return map[string]string{
-		"local_app": `
-			{
-				"@type": "type.googleapis.com/envoy.api.v2.Cluster",
-				"name": "local_app",
-				"type": "STATIC",
-				"connectTimeout": "5s",
-				"loadAssignment": {
-					"clusterName": "local_app",
-					"endpoints": [
-						{
-							"lbEndpoints": [
-								{
-									"endpoint": {
-										"address": {
-											"socketAddress": {
-												"address": "127.0.0.1",
-												"portValue": 8080
-											}
-										}
-									}
-								}
-							]
-						}
-					]
-				}
-			}`,
-		"db": `
-			{
-				"@type": "type.googleapis.com/envoy.api.v2.Cluster",
-				"name": "db.default.dc1.internal.11111111-2222-3333-4444-555555555555.consul",
-				"type": "EDS",
-				"edsClusterConfig": {
-					"edsConfig": {
-						"ads": {
-
-						}
-					}
-				},
-				"outlierDetection": {
-
-				},
-				"circuitBreakers": {
-
-				},
-				"altStatName": "db.default.dc1.internal.11111111-2222-3333-4444-555555555555.consul",
-				"commonLbConfig": {
-					"healthyPanicThreshold": {}
-				},
-				"connectTimeout": "5s",
-				"tlsContext": ` + expectedUpstreamTLSContextJSON(snap, "db.default.dc1.internal.11111111-2222-3333-4444-555555555555.consul", "db") + `
-			}`,
-		"prepared_query:geo-cache": `
-			{
-				"@type": "type.googleapis.com/envoy.api.v2.Cluster",
-				"name": "geo-cache.default.dc1.query.11111111-2222-3333-4444-555555555555.consul",
-				"type": "EDS",
-				"edsClusterConfig": {
-					"edsConfig": {
-						"ads": {
-
-						}
-					}
-				},
-				"outlierDetection": {
-
-				},
-				"circuitBreakers": {
-
-				},
-				"connectTimeout": "5s",
-				"tlsContext": ` + expectedUpstreamTLSContextJSON(snap, "geo-cache.default.dc1.query.11111111-2222-3333-4444-555555555555.consul", "geo-cache") + `
-			}`,
-	}
-}
-
-func expectClustersJSONFromResources(snap *proxycfg.ConfigSnapshot, v, n uint64, resourcesJSON map[string]string) string {
-	resJSON := ""
-
-	// Sort resources into specific order because that matters in JSONEq
-	// comparison later.
-	keyOrder := []string{"local_app"}
-	for _, u := range snap.Proxy.Upstreams {
-		keyOrder = append(keyOrder, u.Identifier())
-	}
-	for _, k := range keyOrder {
-		j, ok := resourcesJSON[k]
-		if !ok {
-			continue
-		}
-		if resJSON != "" {
-			resJSON += ",\n"
-		}
-		resJSON += j
-	}
-
-	return `{
-		"versionInfo": "` + hexString(v) + `",
-		"resources": [` + resJSON + `],
-		"typeUrl": "type.googleapis.com/envoy.api.v2.Cluster",
-		"nonce": "` + hexString(n) + `"
-		}`
-}
-
-func expectClustersJSON(snap *proxycfg.ConfigSnapshot, v, n uint64) string {
-	return expectClustersJSONFromResources(snap, v, n, expectClustersJSONResources(snap))
-}
-
 type customClusterJSONOptions struct {
-	Name        string
-	IncludeType bool
-	TLSContext  string
+	Name       string
+	TLSContext string
 }
 
 var customAppClusterJSONTpl = `{
-	{{ if .IncludeType -}}
-	"@type": "type.googleapis.com/envoy.api.v2.Cluster",
-	{{- end }}
+	"@type": "type.googleapis.com/envoy.config.cluster.v3.Cluster",
 	{{ if .TLSContext -}}
-	"tlsContext": {{ .TLSContext }},
+	"transport_socket": {
+		"name": "tls",
+		"typed_config": {
+			"@type": "type.googleapis.com/envoy.extensions.transport_sockets.tls.v3.UpstreamTlsContext",
+			{{ .TLSContext }}
+		}
+	},
 	{{- end }}
 	"name": "{{ .Name }}",
 	"connectTimeout": "15s",
-	"hosts": [
-		{
-			"socketAddress": {
-				"address": "127.0.0.1", 
-				"portValue": 8080
+	"loadAssignment": {
+		"clusterName": "{{ .Name }}",
+		"endpoints": [
+			{
+				"lbEndpoints": [
+					{
+						"endpoint": {
+							"address": {
+								"socketAddress": {
+									"address": "127.0.0.1",
+									"portValue": 8080
+								}
+							}
+						}
+					}
+				]
 			}
-		}
-	]
+		]
+	}
 }`
 
 var customAppClusterJSONTemplate = template.Must(template.New("").Parse(customAppClusterJSONTpl))
@@ -876,35 +889,35 @@ func TestEnvoyLBConfig_InjectToCluster(t *testing.T) {
 	var tests = []struct {
 		name     string
 		lb       *structs.LoadBalancer
-		expected envoy.Cluster
+		expected *envoy_cluster_v3.Cluster
 	}{
 		{
 			name: "skip empty",
 			lb: &structs.LoadBalancer{
 				Policy: "",
 			},
-			expected: envoy.Cluster{},
+			expected: &envoy_cluster_v3.Cluster{},
 		},
 		{
 			name: "round robin",
 			lb: &structs.LoadBalancer{
 				Policy: structs.LBPolicyRoundRobin,
 			},
-			expected: envoy.Cluster{LbPolicy: envoy.Cluster_ROUND_ROBIN},
+			expected: &envoy_cluster_v3.Cluster{LbPolicy: envoy_cluster_v3.Cluster_ROUND_ROBIN},
 		},
 		{
 			name: "random",
 			lb: &structs.LoadBalancer{
 				Policy: structs.LBPolicyRandom,
 			},
-			expected: envoy.Cluster{LbPolicy: envoy.Cluster_RANDOM},
+			expected: &envoy_cluster_v3.Cluster{LbPolicy: envoy_cluster_v3.Cluster_RANDOM},
 		},
 		{
 			name: "maglev",
 			lb: &structs.LoadBalancer{
 				Policy: structs.LBPolicyMaglev,
 			},
-			expected: envoy.Cluster{LbPolicy: envoy.Cluster_MAGLEV},
+			expected: &envoy_cluster_v3.Cluster{LbPolicy: envoy_cluster_v3.Cluster_MAGLEV},
 		},
 		{
 			name: "ring_hash",
@@ -915,10 +928,10 @@ func TestEnvoyLBConfig_InjectToCluster(t *testing.T) {
 					MaximumRingSize: 7,
 				},
 			},
-			expected: envoy.Cluster{
-				LbPolicy: envoy.Cluster_RING_HASH,
-				LbConfig: &envoy.Cluster_RingHashLbConfig_{
-					RingHashLbConfig: &envoy.Cluster_RingHashLbConfig{
+			expected: &envoy_cluster_v3.Cluster{
+				LbPolicy: envoy_cluster_v3.Cluster_RING_HASH,
+				LbConfig: &envoy_cluster_v3.Cluster_RingHashLbConfig_{
+					RingHashLbConfig: &envoy_cluster_v3.Cluster_RingHashLbConfig{
 						MinimumRingSize: &wrappers.UInt64Value{Value: 3},
 						MaximumRingSize: &wrappers.UInt64Value{Value: 7},
 					},
@@ -933,10 +946,10 @@ func TestEnvoyLBConfig_InjectToCluster(t *testing.T) {
 					ChoiceCount: 3,
 				},
 			},
-			expected: envoy.Cluster{
-				LbPolicy: envoy.Cluster_LEAST_REQUEST,
-				LbConfig: &envoy.Cluster_LeastRequestLbConfig_{
-					LeastRequestLbConfig: &envoy.Cluster_LeastRequestLbConfig{
+			expected: &envoy_cluster_v3.Cluster{
+				LbPolicy: envoy_cluster_v3.Cluster_LEAST_REQUEST,
+				LbConfig: &envoy_cluster_v3.Cluster_LeastRequestLbConfig_{
+					LeastRequestLbConfig: &envoy_cluster_v3.Cluster_LeastRequestLbConfig{
 						ChoiceCount: &wrappers.UInt32Value{Value: 3},
 					},
 				},
@@ -946,11 +959,11 @@ func TestEnvoyLBConfig_InjectToCluster(t *testing.T) {
 
 	for _, tc := range tests {
 		t.Run(tc.name, func(t *testing.T) {
-			var c envoy.Cluster
+			var c envoy_cluster_v3.Cluster
 			err := injectLBToCluster(tc.lb, &c)
 			require.NoError(t, err)
 
-			require.Equal(t, tc.expected, c)
+			require.Equal(t, tc.expected, &c)
 		})
 	}
 }
