@@ -6,8 +6,8 @@ import (
 	"sort"
 	"testing"
 
-	envoyrbac "github.com/envoyproxy/go-control-plane/envoy/config/rbac/v2"
-	envoymatcher "github.com/envoyproxy/go-control-plane/envoy/type/matcher"
+	envoy_rbac_v3 "github.com/envoyproxy/go-control-plane/envoy/config/rbac/v3"
+	envoy_matcher_v3 "github.com/envoyproxy/go-control-plane/envoy/type/matcher/v3"
 
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
@@ -56,12 +56,12 @@ func TestRemoveIntentionPrecedence(t *testing.T) {
 				PathPrefix: "/",
 			},
 		}
-		xdsPermSlashPrefix = &envoyrbac.Permission{
-			Rule: &envoyrbac.Permission_UrlPath{
-				UrlPath: &envoymatcher.PathMatcher{
-					Rule: &envoymatcher.PathMatcher_Path{
-						Path: &envoymatcher.StringMatcher{
-							MatchPattern: &envoymatcher.StringMatcher_Prefix{
+		xdsPermSlashPrefix = &envoy_rbac_v3.Permission{
+			Rule: &envoy_rbac_v3.Permission_UrlPath{
+				UrlPath: &envoy_matcher_v3.PathMatcher{
+					Rule: &envoy_matcher_v3.PathMatcher_Path{
+						Path: &envoy_matcher_v3.StringMatcher{
+							MatchPattern: &envoy_matcher_v3.StringMatcher_Prefix{
 								Prefix: "/",
 							},
 						},
@@ -167,7 +167,7 @@ func TestRemoveIntentionPrecedence(t *testing.T) {
 					Precedence:  8,
 					Skip:        false,
 					ComputedPrincipal: andPrincipals(
-						[]*envoyrbac.Principal{
+						[]*envoy_rbac_v3.Principal{
 							idPrincipal(nameWild),
 							notPrincipal(
 								idPrincipal(nameWeb),
@@ -239,7 +239,7 @@ func TestRemoveIntentionPrecedence(t *testing.T) {
 					Precedence:  8,
 					Skip:        false,
 					ComputedPrincipal: andPrincipals(
-						[]*envoyrbac.Principal{
+						[]*envoy_rbac_v3.Principal{
 							idPrincipal(nameWild),
 							notPrincipal(
 								idPrincipal(nameWeb),
@@ -303,7 +303,7 @@ func TestRemoveIntentionPrecedence(t *testing.T) {
 					Precedence:  8,
 					Skip:        false,
 					ComputedPrincipal: andPrincipals(
-						[]*envoyrbac.Principal{
+						[]*envoy_rbac_v3.Principal{
 							idPrincipal(nameWild),
 							notPrincipal(
 								idPrincipal(nameWeb),
@@ -358,7 +358,7 @@ func TestRemoveIntentionPrecedence(t *testing.T) {
 					Precedence:  8,
 					Skip:        false,
 					ComputedPrincipal: andPrincipals(
-						[]*envoyrbac.Principal{
+						[]*envoy_rbac_v3.Principal{
 							idPrincipal(nameWild),
 							notPrincipal(
 								idPrincipal(nameWeb),
@@ -713,17 +713,39 @@ func TestMakeRBACNetworkAndHTTPFilters(t *testing.T) {
 				filter, err := makeRBACNetworkFilter(tt.intentions, tt.intentionDefaultAllow)
 				require.NoError(t, err)
 
-				gotJSON := protoToJSON(t, filter)
+				t.Run("current", func(t *testing.T) {
+					gotJSON := protoToJSON(t, filter)
 
-				require.JSONEq(t, goldenSimple(t, filepath.Join("rbac", name), gotJSON), gotJSON)
+					require.JSONEq(t, goldenSimple(t, filepath.Join("rbac", name), gotJSON), gotJSON)
+				})
+
+				t.Run("v2-compat", func(t *testing.T) {
+					filterV2, err := convertNetFilterToV2(filter)
+					require.NoError(t, err)
+
+					gotJSON := protoToJSON(t, filterV2)
+
+					require.JSONEq(t, goldenSimple(t, filepath.Join("rbac", name+".v2compat"), gotJSON), gotJSON)
+				})
 			})
 			t.Run("http filter", func(t *testing.T) {
 				filter, err := makeRBACHTTPFilter(tt.intentions, tt.intentionDefaultAllow)
 				require.NoError(t, err)
 
-				gotJSON := protoToJSON(t, filter)
+				t.Run("current", func(t *testing.T) {
+					gotJSON := protoToJSON(t, filter)
 
-				require.JSONEq(t, goldenSimple(t, filepath.Join("rbac", name+"--httpfilter"), gotJSON), gotJSON)
+					require.JSONEq(t, goldenSimple(t, filepath.Join("rbac", name+"--httpfilter"), gotJSON), gotJSON)
+				})
+
+				t.Run("v2-compat", func(t *testing.T) {
+					filterV2, err := convertHttpFilterToV2(filter)
+					require.NoError(t, err)
+
+					gotJSON := protoToJSON(t, filterV2)
+
+					require.JSONEq(t, goldenSimple(t, filepath.Join("rbac", name+"--httpfilter.v2compat"), gotJSON), gotJSON)
+				})
 			})
 		})
 	}
