@@ -1,7 +1,9 @@
 package api
 
 import (
+	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io/ioutil"
 	"net/http"
@@ -85,7 +87,6 @@ func TestAPI_AgentReload(t *testing.T) {
 
 	// Create our initial empty config file, to be overwritten later
 	cfgDir := testutil.TempDir(t, "consul-config")
-	defer os.RemoveAll(cfgDir)
 
 	cfgFilePath := filepath.Join(cfgDir, "reload.json")
 	configFile, err := os.Create(cfgFilePath)
@@ -181,7 +182,7 @@ func TestAPI_AgentServiceAndReplaceChecks(t *testing.T) {
 		ID:   "foo",
 		Tags: []string{"bar", "baz"},
 		TaggedAddresses: map[string]ServiceAddress{
-			"lan": ServiceAddress{
+			"lan": {
 				Address: "198.18.0.1",
 				Port:    80,
 			},
@@ -197,7 +198,7 @@ func TestAPI_AgentServiceAndReplaceChecks(t *testing.T) {
 		ID:   "foo",
 		Tags: []string{"bar", "baz"},
 		TaggedAddresses: map[string]ServiceAddress{
-			"lan": ServiceAddress{
+			"lan": {
 				Address: "198.18.0.1",
 				Port:    80,
 			},
@@ -209,7 +210,9 @@ func TestAPI_AgentServiceAndReplaceChecks(t *testing.T) {
 		t.Fatalf("err: %v", err)
 	}
 
-	if err := agent.ServiceRegisterOpts(regupdate, ServiceRegisterOpts{ReplaceExistingChecks: true}); err != nil {
+	ctx := context.Background()
+	opts := ServiceRegisterOpts{ReplaceExistingChecks: true}.WithContext(ctx)
+	if err := agent.ServiceRegisterOpts(regupdate, opts); err != nil {
 		t.Fatalf("err: %v", err)
 	}
 
@@ -248,6 +251,18 @@ func TestAPI_AgentServiceAndReplaceChecks(t *testing.T) {
 	}
 }
 
+func TestAgent_ServiceRegisterOpts_WithContextTimeout(t *testing.T) {
+	c, err := NewClient(DefaultConfig())
+	require.NoError(t, err)
+
+	ctx, cancel := context.WithTimeout(context.Background(), time.Nanosecond)
+	t.Cleanup(cancel)
+
+	opts := ServiceRegisterOpts{}.WithContext(ctx)
+	err = c.Agent().ServiceRegisterOpts(&AgentServiceRegistration{}, opts)
+	require.True(t, errors.Is(err, context.DeadlineExceeded), "expected timeout")
+}
+
 func TestAPI_AgentServices(t *testing.T) {
 	t.Parallel()
 	c, s := makeClient(t)
@@ -261,7 +276,7 @@ func TestAPI_AgentServices(t *testing.T) {
 		ID:   "foo",
 		Tags: []string{"bar", "baz"},
 		TaggedAddresses: map[string]ServiceAddress{
-			"lan": ServiceAddress{
+			"lan": {
 				Address: "198.18.0.1",
 				Port:    80,
 			},
@@ -318,7 +333,7 @@ func TestAPI_AgentServices(t *testing.T) {
 	}
 }
 
-func TestAPI_AgentServicesWithFilter(t *testing.T) {
+func TestAPI_AgentServicesWithFilterOpts(t *testing.T) {
 	t.Parallel()
 	c, s := makeClient(t)
 	defer s.Stop()
@@ -347,7 +362,8 @@ func TestAPI_AgentServicesWithFilter(t *testing.T) {
 	}
 	require.NoError(t, agent.ServiceRegister(reg))
 
-	services, err := agent.ServicesWithFilter("foo in Tags")
+	opts := &QueryOptions{Namespace: defaultNamespace}
+	services, err := agent.ServicesWithFilterOpts("foo in Tags", opts)
 	require.NoError(t, err)
 	require.Len(t, services, 1)
 	_, ok := services["foo2"]
@@ -422,6 +438,7 @@ func TestAPI_AgentServices_ExternalConnectProxy(t *testing.T) {
 		Port: 8001,
 		Proxy: &AgentServiceConnectProxyConfig{
 			DestinationServiceName: "foo",
+			Mode:                   ProxyModeTransparent,
 		},
 	}
 	if err := agent.ServiceRegister(reg); err != nil {
@@ -438,6 +455,9 @@ func TestAPI_AgentServices_ExternalConnectProxy(t *testing.T) {
 	if _, ok := services["foo-proxy"]; !ok {
 		t.Fatalf("missing proxy service: %v", services)
 	}
+	if services["foo-proxy"].Proxy.Mode != ProxyModeTransparent {
+		t.Fatalf("expected transparent proxy mode to be enabled")
+	}
 
 	if err := agent.ServiceDeregister("foo"); err != nil {
 		t.Fatalf("err: %v", err)
@@ -555,11 +575,11 @@ func TestAPI_AgentServiceAddress(t *testing.T) {
 		Name: "foo2",
 		Port: 8000,
 		TaggedAddresses: map[string]ServiceAddress{
-			"lan": ServiceAddress{
+			"lan": {
 				Address: "192.168.0.43",
 				Port:    8000,
 			},
-			"wan": ServiceAddress{
+			"wan": {
 				Address: "198.18.0.1",
 				Port:    80,
 			},
@@ -602,6 +622,46 @@ func TestAPI_AgentServiceAddress(t *testing.T) {
 		t.Fatalf("err: %v", err)
 	}
 }
+func TestAPI_AgentServiceSocket(t *testing.T) {
+	t.Parallel()
+	c, s := makeClient(t)
+	defer s.Stop()
+
+	agent := c.Agent()
+
+	reg1 := &AgentServiceRegistration{
+		Name:    "foo1",
+		Port:    8000,
+		Address: "192.168.0.42",
+	}
+	reg2 := &AgentServiceRegistration{
+		Name:       "foo2",
+		SocketPath: "/tmp/foo2.sock",
+	}
+
+	if err := agent.ServiceRegister(reg1); err != nil {
+		t.Fatalf("err: %v", err)
+	}
+	if err := agent.ServiceRegister(reg2); err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	services, err := agent.Services()
+	if err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	require.Contains(t, services, "foo1", "missing service foo1")
+	require.Contains(t, services, "foo2", "missing service foo2")
+
+	require.Equal(t, "192.168.0.42", services["foo1"].Address,
+		"missing Address field in service foo1: %v", services["foo1"])
+
+	require.Equal(t, "", services["foo2"].Address,
+		"unexpected Address field in service foo1: %v", services["foo2"])
+	require.Equal(t, "/tmp/foo2.sock", services["foo2"].SocketPath,
+		"missing SocketPath field in service foo1: %v", services["foo2"])
+}
 
 func TestAPI_AgentEnableTagOverride(t *testing.T) {
 	t.Parallel()
@@ -721,14 +781,15 @@ func TestAPI_AgentService(t *testing.T) {
 		ID:          "foo",
 		Service:     "foo",
 		Tags:        []string{"bar", "baz"},
-		ContentHash: "6b13684bfe179e67",
+		ContentHash: "f72563cae6924fb5",
 		Port:        8000,
 		Weights: AgentWeights{
 			Passing: 1,
 			Warning: 1,
 		},
-		Meta:      map[string]string{},
-		Namespace: defaultNamespace,
+		Meta:       map[string]string{},
+		Namespace:  defaultNamespace,
+		Datacenter: "dc1",
 	}
 	require.Equal(expect, got)
 	require.Equal(expect.ContentHash, qm.LastContentHash)
@@ -832,6 +893,63 @@ func TestAPI_AgentSetTTLStatus(t *testing.T) {
 	}
 }
 
+func TestAPI_AgentUpdateTTLOpts(t *testing.T) {
+	t.Parallel()
+	c, s := makeClient(t)
+	defer s.Stop()
+
+	agent := c.Agent()
+	s.WaitForSerfCheck(t)
+
+	reg := &AgentServiceRegistration{
+		Name: "foo",
+		Check: &AgentServiceCheck{
+			TTL: "15s",
+		},
+	}
+	if err := agent.ServiceRegister(reg); err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	verify := func(status, output string) {
+		checks, err := agent.Checks()
+		if err != nil {
+			t.Fatalf("err: %v", err)
+		}
+		chk, ok := checks["service:foo"]
+		if !ok {
+			t.Fatalf("missing check: %v", checks)
+		}
+		if chk.Status != status {
+			t.Fatalf("Bad: %#v", chk)
+		}
+		if chk.Output != output {
+			t.Fatalf("Bad: %#v", chk)
+		}
+	}
+
+	opts := &QueryOptions{Namespace: defaultNamespace}
+
+	if err := agent.UpdateTTLOpts("service:foo", "foo", HealthWarning, opts); err != nil {
+		t.Fatalf("err: %v", err)
+	}
+	verify(HealthWarning, "foo")
+
+	if err := agent.UpdateTTLOpts("service:foo", "bar", HealthPassing, opts); err != nil {
+		t.Fatalf("err: %v", err)
+	}
+	verify(HealthPassing, "bar")
+
+	if err := agent.UpdateTTL("service:foo", "baz", HealthCritical); err != nil {
+		t.Fatalf("err: %v", err)
+	}
+	verify(HealthCritical, "baz")
+
+	if err := agent.ServiceDeregister("foo"); err != nil {
+		t.Fatalf("err: %v", err)
+	}
+}
+
 func TestAPI_AgentChecks(t *testing.T) {
 	t.Parallel()
 	c, s := makeClient(t)
@@ -867,7 +985,7 @@ func TestAPI_AgentChecks(t *testing.T) {
 	}
 }
 
-func TestAPI_AgentChecksWithFilter(t *testing.T) {
+func TestAPI_AgentChecksWithFilterOpts(t *testing.T) {
 	t.Parallel()
 	c, s := makeClient(t)
 	defer s.Stop()
@@ -885,7 +1003,8 @@ func TestAPI_AgentChecksWithFilter(t *testing.T) {
 	reg.TTL = "15s"
 	require.NoError(t, agent.CheckRegister(reg))
 
-	checks, err := agent.ChecksWithFilter("Name == foo")
+	opts := &QueryOptions{Namespace: defaultNamespace}
+	checks, err := agent.ChecksWithFilterOpts("Name == foo", opts)
 	require.NoError(t, err)
 	require.Len(t, checks, 1)
 	_, ok := checks["foo"]
