@@ -7,6 +7,7 @@ import (
 	"io/ioutil"
 	"net"
 	"net/http"
+	"net/url"
 	"os"
 	"path/filepath"
 	"reflect"
@@ -812,6 +813,79 @@ func TestAPI_SetWriteOptions(t *testing.T) {
 	}
 }
 
+func TestAPI_Headers(t *testing.T) {
+	t.Parallel()
+
+	var request *http.Request
+	c, s := makeClientWithConfig(t, func(c *Config) {
+		transport := http.DefaultTransport.(*http.Transport).Clone()
+		transport.Proxy = func(r *http.Request) (*url.URL, error) {
+			// Keep track of the last request sent
+			request = r
+			return nil, nil
+		}
+		c.Transport = transport
+	}, nil)
+	defer s.Stop()
+
+	if len(c.Headers()) != 0 {
+		t.Fatalf("expected headers to be empty: %v", c.Headers())
+	}
+
+	c.AddHeader("Hello", "World")
+	r := c.newRequest("GET", "/v1/kv/foo")
+
+	if r.header.Get("Hello") != "World" {
+		t.Fatalf("Hello header not set : %v", r.header)
+	}
+
+	c.SetHeaders(http.Header{
+		"Auth": []string{"Token"},
+	})
+
+	r = c.newRequest("GET", "/v1/kv/foo")
+	if r.header.Get("Hello") != "" {
+		t.Fatalf("Hello header should not be set: %v", r.header)
+	}
+
+	if r.header.Get("Auth") != "Token" {
+		t.Fatalf("Auth header not set: %v", r.header)
+	}
+
+	kv := c.KV()
+	_, err := kv.Put(&KVPair{Key: "test-headers", Value: []byte("foo")}, nil)
+	require.NoError(t, err)
+	require.Equal(t, "application/octet-stream", request.Header.Get("Content-Type"))
+
+	_, _, err = kv.Get("test-headers", nil)
+	require.NoError(t, err)
+	require.Equal(t, "", request.Header.Get("Content-Type"))
+
+	_, err = kv.Delete("test-headers", nil)
+	require.NoError(t, err)
+	require.Equal(t, "", request.Header.Get("Content-Type"))
+
+	err = c.Snapshot().Restore(nil, strings.NewReader("foo"))
+	require.Error(t, err)
+	require.Equal(t, "application/octet-stream", request.Header.Get("Content-Type"))
+
+	_, err = c.ACL().RulesTranslate(strings.NewReader(`
+	agent "" {
+	  policy = "read"
+	}
+	`))
+	// ACL support is disabled
+	require.Error(t, err)
+	require.Equal(t, "text/plain", request.Header.Get("Content-Type"))
+
+	_, _, err = c.Event().Fire(&UserEvent{
+		Name:    "test",
+		Payload: []byte("foo"),
+	}, nil)
+	require.NoError(t, err)
+	require.Equal(t, "application/octet-stream", request.Header.Get("Content-Type"))
+}
+
 func TestAPI_RequestToHTTP(t *testing.T) {
 	t.Parallel()
 	c, s := makeClient(t)
