@@ -20,6 +20,10 @@ func TestAgentTokensCommand_noTabs(t *testing.T) {
 }
 
 func TestAgentTokensCommand(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	assert := assert.New(t)
 
