@@ -21,6 +21,10 @@ func TestRoleDeleteCommand_noTabs(t *testing.T) {
 }
 
 func TestRoleDeleteCommand(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := agent.NewTestAgent(t, `
