@@ -35,7 +35,8 @@ func New(ui cli.Ui) *cmd {
 		versionHuman:      consulversion.GetHumanVersion(),
 		flags:             flag.NewFlagSet("", flag.ContinueOnError),
 	}
-	c.init()
+	config.AddFlags(c.flags, &c.configLoadOpts)
+	c.help = flags.Usage(help, c.flags)
 	return c
 }
 
@@ -52,16 +53,10 @@ type cmd struct {
 	version           string
 	versionPrerelease string
 	versionHuman      string
-	flagArgs          config.BuilderOpts
+	configLoadOpts    config.LoadOpts
 	logger            hclog.InterceptLogger
 }
 
-func (c *cmd) init() {
-	c.flags = flag.NewFlagSet("", flag.ContinueOnError)
-	config.AddFlags(c.flags, &c.flagArgs)
-	c.help = flags.Usage(help, c.flags)
-}
-
 func (c *cmd) Run(args []string) int {
 	code := c.run(args)
 	if c.logger != nil {
@@ -166,8 +161,9 @@ func (c *cmd) run(args []string) int {
 	// FIXME: logs should always go to stderr, but previously they were sent to
 	// stdout, so continue to use Stdout for now, and fix this in a future release.
 	logGate := &logging.GatedWriter{Writer: c.ui.Stdout()}
-	loader := func(source config.Source) (cfg *config.RuntimeConfig, warnings []string, err error) {
-		return config.Load(c.flagArgs, source)
+	loader := func(source config.Source) (config.LoadResult, error) {
+		c.configLoadOpts.DefaultConfig = source
+		return config.Load(c.configLoadOpts)
 	}
 	bd, err := agent.NewBaseDeps(loader, logGate)
 	if err != nil {
