@@ -19,6 +19,10 @@ import (
 
 // TestConfigFail should test command line flags that lead to an immediate error.
 func TestConfigFail(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	dataDir := testutil.TempDir(t, "consul")
@@ -80,6 +84,10 @@ func TestConfigFail(t *testing.T) {
 }
 
 func TestRetryJoin(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	a := agent.NewTestAgent(t, "")
 	defer a.Shutdown()
 
@@ -106,6 +114,10 @@ func TestRetryJoin(t *testing.T) {
 }
 
 func TestRetryJoinFail(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	tmpDir := testutil.TempDir(t, "consul")
 
@@ -126,6 +138,10 @@ func TestRetryJoinFail(t *testing.T) {
 }
 
 func TestRetryJoinWanFail(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	tmpDir := testutil.TempDir(t, "consul")
 
