@@ -52,6 +52,10 @@ func TestCatalogListDatacentersCommand_Validation(t *testing.T) {
 }
 
 func TestCatalogListDatacentersCommand(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
