@@ -16,6 +16,10 @@ func TestConfigRead_noTabs(t *testing.T) {
 }
 
 func TestConfigRead(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := agent.NewTestAgent(t, ``)
