@@ -1,6 +1,7 @@
 package write
 
 import (
+	"bytes"
 	"io"
 	"strings"
 	"testing"
@@ -8,11 +9,12 @@ import (
 
 	"github.com/hashicorp/consul/agent/structs"
 
+	"github.com/mitchellh/cli"
+	"github.com/stretchr/testify/require"
+
 	"github.com/hashicorp/consul/agent"
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/sdk/testutil"
-	"github.com/mitchellh/cli"
-	"github.com/stretchr/testify/require"
 )
 
 func TestConfigWrite_noTabs(t *testing.T) {
@@ -22,6 +24,10 @@ func TestConfigWrite_noTabs(t *testing.T) {
 }
 
 func TestConfigWrite(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 
 	a := agent.NewTestAgent(t, ``)
@@ -108,6 +114,36 @@ func TestConfigWrite(t *testing.T) {
 		require.NotEqual(t, 0, code)
 		require.NotEmpty(t, ui.ErrorWriter.String())
 	})
+
+	t.Run("mesh config entry", func(t *testing.T) {
+		stdin := new(bytes.Buffer)
+		stdin.WriteString(`
+kind = "mesh"
+meta {
+	"foo" = "bar"
+	"gir" = "zim"
+}
+transparent_proxy {
+	mesh_destinations_only = true
+}
+`)
+
+		ui := cli.NewMockUi()
+		c := New(ui)
+		c.testStdin = stdin
+
+		code := c.Run([]string{"-http-addr=" + a.HTTPAddr(), "-"})
+		require.Empty(t, ui.ErrorWriter.String())
+		require.Contains(t, ui.OutputWriter.String(),
+			`Config entry written: mesh/mesh`)
+		require.Equal(t, 0, code)
+
+		entry, _, err := client.ConfigEntries().Get(api.MeshConfig, api.MeshConfigMesh, nil)
+		require.NoError(t, err)
+		proxy, ok := entry.(*api.MeshConfigEntry)
+		require.True(t, ok)
+		require.Equal(t, map[string]string{"foo": "bar", "gir": "zim"}, proxy.Meta)
+	})
 }
 
 // TestParseConfigEntry is the 'api' mirror image of
@@ -177,6 +213,11 @@ func TestParseConfigEntry(t *testing.T) {
 				mesh_gateway {
 					mode = "remote"
 				}
+				mode = "direct"
+				transparent_proxy = {
+					outbound_listener_port = 10101
+					dialed_directly = true
+				}
 			`,
 			camel: `
 				Kind = "proxy-defaults"
@@ -195,6 +236,11 @@ func TestParseConfigEntry(t *testing.T) {
 				MeshGateway {
 					Mode = "remote"
 				}
+				Mode = "direct"
+				TransparentProxy = {
+					outbound_listener_port = 10101
+					dialed_directly = true
+				}
 			`,
 			snakeJSON: `
 			{
@@ -213,6 +259,11 @@ func TestParseConfigEntry(t *testing.T) {
 				},
 				"mesh_gateway": {
 					"mode": "remote"
+				},
+				"mode": "direct",
+				"transparent_proxy": {
+					"outbound_listener_port": 10101,
+					"dialed_directly": true
 				}
 			}
 			`,
@@ -233,6 +284,11 @@ func TestParseConfigEntry(t *testing.T) {
 				},
 				"MeshGateway": {
 					"Mode": "remote"
+				},
+				"Mode": "direct",
+				"TransparentProxy": {
+					"OutboundListenerPort": 10101,
+					"DialedDirectly": true
 				}
 			}
 			`,
@@ -253,6 +309,11 @@ func TestParseConfigEntry(t *testing.T) {
 				MeshGateway: api.MeshGatewayConfig{
 					Mode: api.MeshGatewayModeRemote,
 				},
+				Mode: api.ProxyModeDirect,
+				TransparentProxy: &api.TransparentProxyConfig{
+					OutboundListenerPort: 10101,
+					DialedDirectly:       true,
+				},
 			},
 			expectJSON: &api.ProxyConfigEntry{
 				Kind: "proxy-defaults",
@@ -271,6 +332,11 @@ func TestParseConfigEntry(t *testing.T) {
 				MeshGateway: api.MeshGatewayConfig{
 					Mode: api.MeshGatewayModeRemote,
 				},
+				Mode: api.ProxyModeDirect,
+				TransparentProxy: &api.TransparentProxyConfig{
+					OutboundListenerPort: 10101,
+					DialedDirectly:       true,
+				},
 			},
 		},
 		{
@@ -419,7 +485,7 @@ func TestParseConfigEntry(t *testing.T) {
 			},
 		},
 		{
-			name: "service-defaults",
+			name: "service-defaults: kitchen sink",
 			snake: `
 				kind = "service-defaults"
 				name = "main"
@@ -432,6 +498,52 @@ func TestParseConfigEntry(t *testing.T) {
 				mesh_gateway {
 					mode = "remote"
 				}
+				mode = "direct"
+				transparent_proxy = {
+					outbound_listener_port = 10101
+					dialed_directly = true
+				}
+				upstream_config {
+					overrides = [
+						{
+							name = "redis"
+							passive_health_check {
+								max_failures = 3
+								interval = "2s"
+							}
+							envoy_listener_json = "{ \"listener-foo\": 5 }"
+							envoy_cluster_json = "{ \"cluster-bar\": 5 }"
+							protocol = "grpc"
+							connect_timeout_ms = 6543
+						},
+						{
+							name = "finance--billing"
+							mesh_gateway {
+								mode = "remote"
+							}
+							limits {
+								max_connections = 1111
+								max_pending_requests = 2222
+								max_concurrent_requests = 3333
+							}
+						}
+					]
+					defaults {
+						envoy_cluster_json = "zip"
+						envoy_listener_json = "zop"
+						connect_timeout_ms = 5000
+						protocol = "http"
+						limits {
+							max_connections = 3
+							max_pending_requests = 4
+							max_concurrent_requests = 5
+						}
+						passive_health_check {
+							max_failures = 5
+							interval = "4s"
+						}
+					}
+				}
 			`,
 			camel: `
 				Kind = "service-defaults"
@@ -445,6 +557,52 @@ func TestParseConfigEntry(t *testing.T) {
 				MeshGateway {
 					Mode = "remote"
 				}
+				Mode = "direct"
+				TransparentProxy = {
+					outbound_listener_port = 10101
+					dialed_directly = true
+				}
+				UpstreamConfig {
+					Overrides = [
+						{
+							Name = "redis"
+							PassiveHealthCheck {
+								MaxFailures = 3
+								Interval = "2s"
+							}
+							EnvoyListenerJson = "{ \"listener-foo\": 5 }"
+							EnvoyClusterJson = "{ \"cluster-bar\": 5 }"
+							Protocol = "grpc"
+							ConnectTimeoutMs = 6543
+						},
+						{
+							Name = "finance--billing"
+							MeshGateway {
+								Mode = "remote"
+							}
+							Limits {
+								MaxConnections = 1111
+								MaxPendingRequests = 2222
+								MaxConcurrentRequests = 3333
+							}
+						}
+					]
+					Defaults {
+						EnvoyClusterJson = "zip"
+						EnvoyListenerJson = "zop"
+						ConnectTimeoutMs = 5000
+						Protocol = "http"
+						Limits {
+							MaxConnections = 3
+							MaxPendingRequests = 4
+							MaxConcurrentRequests = 5
+						}
+						PassiveHealthCheck {
+							MaxFailures = 5
+							Interval = "4s"
+						}
+					}
+				}
 			`,
 			snakeJSON: `
 			{
@@ -458,6 +616,52 @@ func TestParseConfigEntry(t *testing.T) {
 				"external_sni": "abc-123",
 				"mesh_gateway": {
 					"mode": "remote"
+				},
+				"mode": "direct",
+				"transparent_proxy": {
+					"outbound_listener_port": 10101,
+					"dialed_directly": true
+				},
+				"upstream_config": {
+					"overrides": [
+						{
+							"name": "redis",
+							"passive_health_check": {
+								"max_failures": 3,
+								"interval": "2s"
+							},
+							"envoy_listener_json": "{ \"listener-foo\": 5 }",
+							"envoy_cluster_json": "{ \"cluster-bar\": 5 }",
+							"protocol": "grpc",
+							"connect_timeout_ms": 6543
+						},
+						{
+							"name": "finance--billing",
+							"mesh_gateway": {
+								"mode": "remote"
+							},
+							"limits": {
+								"max_connections": 1111,
+								"max_pending_requests": 2222,
+								"max_concurrent_requests": 3333
+							}
+						}
+					],
+					"defaults": {
+						"envoy_cluster_json": "zip",
+						"envoy_listener_json": "zop",
+						"connect_timeout_ms": 5000,
+						"protocol": "http",
+						"limits": {
+							"max_connections": 3,
+							"max_pending_requests": 4,
+							"max_concurrent_requests": 5
+						},
+						"passive_health_check": {
+							"max_failures": 5,
+							"interval": "4s"
+						}
+					}
 				}
 			}
 			`,
@@ -473,6 +677,52 @@ func TestParseConfigEntry(t *testing.T) {
 				"ExternalSNI": "abc-123",
 				"MeshGateway": {
 					"Mode": "remote"
+				},
+				"Mode": "direct",
+				"TransparentProxy": {
+					"OutboundListenerPort": 10101,
+					"DialedDirectly": true
+				},
+				"UpstreamConfig": {
+					"Overrides": [
+						{
+							"Name": "redis",
+							"PassiveHealthCheck": {
+								"MaxFailures": 3,
+								"Interval": "2s"
+							},
+							"EnvoyListenerJson": "{ \"listener-foo\": 5 }",
+							"EnvoyClusterJson": "{ \"cluster-bar\": 5 }",
+							"Protocol": "grpc",
+							"ConnectTimeoutMs": 6543
+						},
+						{
+							"Name": "finance--billing",
+							"MeshGateway": {
+								"Mode": "remote"
+							},
+							"Limits": {
+								"MaxConnections": 1111,
+								"MaxPendingRequests": 2222,
+								"MaxConcurrentRequests": 3333
+							}
+						}
+					],
+					"Defaults": {
+						"EnvoyClusterJson": "zip",
+						"EnvoyListenerJson": "zop",
+						"ConnectTimeoutMs": 5000,
+						"Protocol": "http",
+						"Limits": {
+							"MaxConnections": 3,
+							"MaxPendingRequests": 4,
+							"MaxConcurrentRequests": 5
+						},
+						"PassiveHealthCheck": {
+							"MaxFailures": 5,
+							"Interval": "4s"
+						}
+					}
 				}
 			}
 			`,
@@ -488,6 +738,52 @@ func TestParseConfigEntry(t *testing.T) {
 				MeshGateway: api.MeshGatewayConfig{
 					Mode: api.MeshGatewayModeRemote,
 				},
+				Mode: api.ProxyModeDirect,
+				TransparentProxy: &api.TransparentProxyConfig{
+					OutboundListenerPort: 10101,
+					DialedDirectly:       true,
+				},
+				UpstreamConfig: &api.UpstreamConfiguration{
+					Overrides: []*api.UpstreamConfig{
+						{
+							Name: "redis",
+							PassiveHealthCheck: &api.PassiveHealthCheck{
+								MaxFailures: 3,
+								Interval:    2 * time.Second,
+							},
+							EnvoyListenerJSON: `{ "listener-foo": 5 }`,
+							EnvoyClusterJSON:  `{ "cluster-bar": 5 }`,
+							Protocol:          "grpc",
+							ConnectTimeoutMs:  6543,
+						},
+						{
+							Name: "finance--billing",
+							MeshGateway: api.MeshGatewayConfig{
+								Mode: "remote",
+							},
+							Limits: &api.UpstreamLimits{
+								MaxConnections:        intPointer(1111),
+								MaxPendingRequests:    intPointer(2222),
+								MaxConcurrentRequests: intPointer(3333),
+							},
+						},
+					},
+					Defaults: &api.UpstreamConfig{
+						EnvoyClusterJSON:  "zip",
+						EnvoyListenerJSON: "zop",
+						Protocol:          "http",
+						ConnectTimeoutMs:  5000,
+						Limits: &api.UpstreamLimits{
+							MaxConnections:        intPointer(3),
+							MaxPendingRequests:    intPointer(4),
+							MaxConcurrentRequests: intPointer(5),
+						},
+						PassiveHealthCheck: &api.PassiveHealthCheck{
+							MaxFailures: 5,
+							Interval:    4 * time.Second,
+						},
+					},
+				},
 			},
 		},
 		{
@@ -2369,6 +2665,62 @@ func TestParseConfigEntry(t *testing.T) {
 				},
 			},
 		},
+		{
+			name: "mesh",
+			snake: `
+				kind = "mesh"
+				meta {
+					"foo" = "bar"
+					"gir" = "zim"
+				}
+				transparent_proxy {
+					mesh_destinations_only = true
+				}
+			`,
+			camel: `
+				Kind = "mesh"
+				Meta {
+					"foo" = "bar"
+					"gir" = "zim"
+				}
+				TransparentProxy {
+					MeshDestinationsOnly = true
+				}
+			`,
+			snakeJSON: `
+			{
+				"kind": "mesh",
+				"meta" : {
+					"foo": "bar",
+					"gir": "zim"
+				},
+				"transparent_proxy": {
+					"mesh_destinations_only": true
+				}
+			}
+			`,
+			camelJSON: `
+			{
+				"Kind": "mesh",
+				"Meta" : {
+					"foo": "bar",
+					"gir": "zim"
+				},
+				"TransparentProxy": {
+					"MeshDestinationsOnly": true
+				}
+			}
+			`,
+			expect: &api.MeshConfigEntry{
+				Meta: map[string]string{
+					"foo": "bar",
+					"gir": "zim",
+				},
+				TransparentProxy: api.TransparentProxyMeshConfig{
+					MeshDestinationsOnly: true,
+				},
+			},
+		},
 	} {
 		tc := tc
 
@@ -2416,3 +2768,7 @@ func requireContainsLower(t *testing.T, haystack, needle string) {
 	t.Helper()
 	require.Contains(t, strings.ToLower(haystack), strings.ToLower(needle))
 }
+
+func intPointer(v int) *int {
+	return &v
+}
