@@ -4,12 +4,13 @@ import (
 	"bytes"
 	"encoding/json"
 	"fmt"
-	"github.com/hashicorp/consul/api"
 	"net"
 	"net/url"
 	"os"
 	"strings"
 	"text/template"
+
+	"github.com/hashicorp/consul/api"
 )
 
 const (
@@ -85,7 +86,7 @@ type BootstrapConfig struct {
 	// configure the aspects that Connect relies upon to work. It's recommended
 	// that this only be used if necessary, and that it be based on the default
 	// template in
-	// https://github.com/hashicorp/consul/blob/master/command/connect/envoy/bootstrap_tpl.go
+	// https://github.com/hashicorp/consul/blob/main/command/connect/envoy/bootstrap_tpl.go
 	// for the correct version of Consul and Envoy being used.
 	OverrideJSONTpl string `mapstructure:"envoy_bootstrap_json_tpl"`
 
@@ -212,19 +213,19 @@ func (c *BootstrapConfig) ConfigureArgs(args *BootstrapTplArgs, omitDeprecatedTa
 	}
 	// Setup prometheus if needed. This MUST happen after the Static*JSON is set above
 	if c.PrometheusBindAddr != "" {
-		if err := c.generateListenerConfig(args, c.PrometheusBindAddr, "envoy_prometheus_metrics", "path", "/metrics", "/stats/prometheus"); err != nil {
+		if err := c.generateListenerConfig(args, c.PrometheusBindAddr, "envoy_prometheus_metrics", "path", args.PrometheusScrapePath, "/stats/prometheus", args.PrometheusBackendPort); err != nil {
 			return err
 		}
 	}
 	// Setup /stats proxy listener if needed. This MUST happen after the Static*JSON is set above
 	if c.StatsBindAddr != "" {
-		if err := c.generateListenerConfig(args, c.StatsBindAddr, "envoy_metrics", "prefix", "/stats", "/stats"); err != nil {
+		if err := c.generateListenerConfig(args, c.StatsBindAddr, "envoy_metrics", "prefix", "/stats", "/stats", ""); err != nil {
 			return err
 		}
 	}
 	// Setup /ready proxy listener if needed. This MUST happen after the Static*JSON is set above
 	if c.ReadyBindAddr != "" {
-		if err := c.generateListenerConfig(args, c.ReadyBindAddr, "envoy_ready", "path", "/ready", "/ready"); err != nil {
+		if err := c.generateListenerConfig(args, c.ReadyBindAddr, "envoy_ready", "path", "/ready", "/ready", ""); err != nil {
 			return err
 		}
 	}
@@ -244,14 +245,22 @@ func (c *BootstrapConfig) generateStatsSinks(args *BootstrapTplArgs) error {
 	var stats_sinks []string
 
 	if c.StatsdURL != "" {
-		sinkJSON, err := c.generateStatsSinkJSON("envoy.statsd", c.StatsdURL)
+		sinkJSON, err := c.generateStatsSinkJSON(
+			"envoy.stat_sinks.statsd",
+			"type.googleapis.com/envoy.config.metrics.v3.StatsdSink",
+			c.StatsdURL,
+		)
 		if err != nil {
 			return err
 		}
 		stats_sinks = append(stats_sinks, sinkJSON)
 	}
 	if c.DogstatsdURL != "" {
-		sinkJSON, err := c.generateStatsSinkJSON("envoy.dog_statsd", c.DogstatsdURL)
+		sinkJSON, err := c.generateStatsSinkJSON(
+			"envoy.stat_sinks.dog_statsd",
+			"type.googleapis.com/envoy.config.metrics.v3.DogStatsdSink",
+			c.DogstatsdURL,
+		)
 		if err != nil {
 			return err
 		}
@@ -267,10 +276,12 @@ func (c *BootstrapConfig) generateStatsSinks(args *BootstrapTplArgs) error {
 	return nil
 }
 
-func (c *BootstrapConfig) generateStatsSinkJSON(name string, addr string) (string, error) {
+func (c *BootstrapConfig) generateStatsSinkJSON(name string, typeName string, addr string) (string, error) {
 	// Resolve address ENV var
 	if len(addr) > 2 && addr[0] == '$' {
 		addr = os.Getenv(addr[1:])
+	} else {
+		addr = os.Expand(addr, statsSinkEnvMapping)
 	}
 
 	u, err := url.Parse(addr)
@@ -300,7 +311,8 @@ func (c *BootstrapConfig) generateStatsSinkJSON(name string, addr string) (strin
 
 	return `{
 		"name": "` + name + `",
-		"config": {
+		"typedConfig": {
+			"@type": "` + typeName + `",
 			"address": {
 				` + addrJSON + `
 			}
@@ -308,6 +320,18 @@ func (c *BootstrapConfig) generateStatsSinkJSON(name string, addr string) (strin
 	}`, nil
 }
 
+func statsSinkEnvMapping(s string) string {
+	allowedStatsSinkEnvVars := map[string]bool{
+		"HOST_IP": true,
+	}
+
+	if !allowedStatsSinkEnvVars[s] {
+		// if the specified env var isn't explicitly allowed, unexpand it
+		return fmt.Sprintf("${%s}", s)
+	}
+	return os.Getenv(s)
+}
+
 // resourceTagSpecifiers returns patterns used to generate tags from cluster and filter metric names.
 func resourceTagSpecifiers(omitDeprecatedTags bool) ([]string, error) {
 	const (
@@ -328,40 +352,41 @@ func resourceTagSpecifiers(omitDeprecatedTags bool) ([]string, error) {
 		// - cluster.f8f8f8f8~pong.default.dc2.internal.e5b08d03-bfc3-c870-1833-baddb116e648.consul.bind_errors: 0
 		// - cluster.v2.pong.default.dc2.internal.e5b08d03-bfc3-c870-1833-baddb116e648.consul.bind_errors: 0
 		// - cluster.f8f8f8f8~v2.pong.default.dc2.internal.e5b08d03-bfc3-c870-1833-baddb116e648.consul.bind_errors: 0
+		// - cluster.passthrough~pong.default.dc2.internal.e5b08d03-bfc3-c870-1833-baddb116e648.consul.bind_errors: 0
 		{"consul.destination.custom_hash",
-			fmt.Sprintf(`^cluster\.((?:(%s)~)?(?:%s\.)?%s\.%s\.%s\.%s\.%s\.consul\.)`,
+			fmt.Sprintf(`^cluster\.(?:passthrough~)?((?:(%s)~)?(?:%s\.)?%s\.%s\.%s\.%s\.%s\.consul\.)`,
 				reSegment, reSegment, reSegment, reSegment, reSegment, reSegment, reSegment)},
 
 		{"consul.destination.service_subset",
-			fmt.Sprintf(`^cluster\.((?:%s~)?(?:(%s)\.)?%s\.%s\.%s\.%s\.%s\.consul\.)`,
+			fmt.Sprintf(`^cluster\.(?:passthrough~)?((?:%s~)?(?:(%s)\.)?%s\.%s\.%s\.%s\.%s\.consul\.)`,
 				reSegment, reSegment, reSegment, reSegment, reSegment, reSegment, reSegment)},
 
 		{"consul.destination.service",
-			fmt.Sprintf(`^cluster\.((?:%s~)?(?:%s\.)?(%s)\.%s\.%s\.%s\.%s\.consul\.)`,
+			fmt.Sprintf(`^cluster\.(?:passthrough~)?((?:%s~)?(?:%s\.)?(%s)\.%s\.%s\.%s\.%s\.consul\.)`,
 				reSegment, reSegment, reSegment, reSegment, reSegment, reSegment, reSegment)},
 
 		{"consul.destination.namespace",
-			fmt.Sprintf(`^cluster\.((?:%s~)?(?:%s\.)?%s\.(%s)\.%s\.%s\.%s\.consul\.)`,
+			fmt.Sprintf(`^cluster\.(?:passthrough~)?((?:%s~)?(?:%s\.)?%s\.(%s)\.%s\.%s\.%s\.consul\.)`,
 				reSegment, reSegment, reSegment, reSegment, reSegment, reSegment, reSegment)},
 
 		{"consul.destination.datacenter",
-			fmt.Sprintf(`^cluster\.((?:%s~)?(?:%s\.)?%s\.%s\.(%s)\.%s\.%s\.consul\.)`,
+			fmt.Sprintf(`^cluster\.(?:passthrough~)?((?:%s~)?(?:%s\.)?%s\.%s\.(%s)\.%s\.%s\.consul\.)`,
 				reSegment, reSegment, reSegment, reSegment, reSegment, reSegment, reSegment)},
 
 		{"consul.destination.routing_type",
-			fmt.Sprintf(`^cluster\.((?:%s~)?(?:%s\.)?%s\.%s\.%s\.(%s)\.%s\.consul\.)`,
+			fmt.Sprintf(`^cluster\.(?:passthrough~)?((?:%s~)?(?:%s\.)?%s\.%s\.%s\.(%s)\.%s\.consul\.)`,
 				reSegment, reSegment, reSegment, reSegment, reSegment, reSegment, reSegment)},
 
 		{"consul.destination.trust_domain",
-			fmt.Sprintf(`^cluster\.((?:%s~)?(?:%s\.)?%s\.%s\.%s\.%s\.(%s)\.consul\.)`,
+			fmt.Sprintf(`^cluster\.(?:passthrough~)?((?:%s~)?(?:%s\.)?%s\.%s\.%s\.%s\.(%s)\.consul\.)`,
 				reSegment, reSegment, reSegment, reSegment, reSegment, reSegment, reSegment)},
 
 		{"consul.destination.target",
-			fmt.Sprintf(`^cluster\.(((?:%s~)?(?:%s\.)?%s\.%s\.%s)\.%s\.%s\.consul\.)`,
+			fmt.Sprintf(`^cluster\.(?:passthrough~)?(((?:%s~)?(?:%s\.)?%s\.%s\.%s)\.%s\.%s\.consul\.)`,
 				reSegment, reSegment, reSegment, reSegment, reSegment, reSegment, reSegment)},
 
 		{"consul.destination.full_target",
-			fmt.Sprintf(`^cluster\.(((?:%s~)?(?:%s\.)?%s\.%s\.%s\.%s\.%s)\.consul\.)`,
+			fmt.Sprintf(`^cluster\.(?:passthrough~)?(((?:%s~)?(?:%s\.)?%s\.%s\.%s\.%s\.%s)\.consul\.)`,
 				reSegment, reSegment, reSegment, reSegment, reSegment, reSegment, reSegment)},
 
 		// Upstream listener metrics are prefixed by consul.upstream
@@ -539,25 +564,54 @@ func generateStatsTags(args *BootstrapTplArgs, initialTags []string, omitDepreca
 	return tagJSONs, nil
 }
 
-func (c *BootstrapConfig) generateListenerConfig(args *BootstrapTplArgs, bindAddr, name, matchType, matchValue, prefixRewrite string) error {
+func (c *BootstrapConfig) generateListenerConfig(args *BootstrapTplArgs, bindAddr, name, matchType, matchValue, prefixRewrite, prometheusBackendPort string) error {
 	host, port, err := net.SplitHostPort(bindAddr)
 	if err != nil {
 		return fmt.Errorf("invalid %s bind address: %s", name, err)
 	}
 
+	// If prometheusBackendPort is set (not empty string), create
+	// "prometheus_backend" cluster with the prometheusBackendPort that the
+	// listener will point to, rather than the "self_admin" cluster. This is for
+	// the merged metrics feature in consul-k8s, so the
+	// envoy_prometheus_bind_addr listener will point to the merged Envoy and
+	// service metrics endpoint rather than the Envoy admin endpoint for
+	// metrics. This cluster will only be created once since it's only created
+	// when prometheusBackendPort is set, and prometheusBackendPort is only set
+	// when calling this function if c.PrometheusBindAddr is set.
+	clusterAddress := args.AdminBindAddress
+	clusterPort := args.AdminBindPort
+	clusterName := selfAdminName
+	if prometheusBackendPort != "" {
+		clusterPort = prometheusBackendPort
+		clusterName = "prometheus_backend"
+	}
+
 	clusterJSON := `{
-		"name": "` + selfAdminName + `",
+		"name": "` + clusterName + `",
+		"ignore_health_on_host_removal": false,
 		"connect_timeout": "5s",
 		"type": "STATIC",
 		"http_protocol_options": {},
-		"hosts": [
-			{
-				"socket_address": {
-					"address": "` + args.AdminBindAddress + `",
-					"port_value": ` + args.AdminBindPort + `
+		"loadAssignment": {
+			"clusterName": "` + clusterName + `",
+			"endpoints": [
+				{
+					"lbEndpoints": [
+						{
+							"endpoint": {
+								"address": {
+									"socket_address": {
+										"address": "` + clusterAddress + `",
+										"port_value": ` + clusterPort + `
+									}
+								}
+							}
+						}
+					]
 				}
-			}
-		]
+			]
+		}
 	}`
 	listenerJSON := `{
 		"name": "` + name + `_listener",
@@ -571,8 +625,9 @@ func (c *BootstrapConfig) generateListenerConfig(args *BootstrapTplArgs, bindAdd
 			{
 				"filters": [
 					{
-						"name": "envoy.http_connection_manager",
-						"config": {
+						"name": "envoy.filters.network.http_connection_manager",
+						 "typedConfig": {
+							"@type": "type.googleapis.com/envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager",
 							"stat_prefix": "` + name + `",
 							"codec_type": "HTTP1",
 							"route_config": {
@@ -589,7 +644,7 @@ func (c *BootstrapConfig) generateListenerConfig(args *BootstrapTplArgs, bindAdd
 													"` + matchType + `": "` + matchValue + `"
 												},
 												"route": {
-													"cluster": "self_admin",
+													"cluster": "` + clusterName + `",
 													"prefix_rewrite": "` + prefixRewrite + `"
 												}
 											},
@@ -607,7 +662,7 @@ func (c *BootstrapConfig) generateListenerConfig(args *BootstrapTplArgs, bindAdd
 							},
 							"http_filters": [
 								{
-									"name": "envoy.router"
+									"name": "envoy.filters.http.router"
 								}
 							]
 						}
