@@ -13,204 +13,373 @@ import (
 
 const (
 	expectedSelfAdminCluster = `{
-		"name": "self_admin",
-		"connect_timeout": "5s",
-		"type": "STATIC",
-		"http_protocol_options": {},
-		"hosts": [
-			{
-				"socket_address": {
-					"address": "127.0.0.1",
-					"port_value": 19000
-				}
-			}
-		]
-	}`
+  "name": "self_admin",
+  "ignore_health_on_host_removal": false,
+  "connect_timeout": "5s",
+  "type": "STATIC",
+  "http_protocol_options": {},
+  "loadAssignment": {
+    "clusterName": "self_admin",
+    "endpoints": [
+      {
+        "lbEndpoints": [
+          {
+            "endpoint": {
+              "address": {
+                "socket_address": {
+                  "address": "127.0.0.1",
+                  "port_value": 19000
+                }
+              }
+            }
+          }
+        ]
+      }
+    ]
+  }
+}`
 	expectedSelfAdminClusterNonLoopbackIP = `{
-		"name": "self_admin",
-		"connect_timeout": "5s",
-		"type": "STATIC",
-		"http_protocol_options": {},
-		"hosts": [
-			{
-				"socket_address": {
-					"address": "192.0.2.10",
-					"port_value": 19002
-				}
-			}
-		]
-	}`
+  "name": "self_admin",
+  "ignore_health_on_host_removal": false,
+  "connect_timeout": "5s",
+  "type": "STATIC",
+  "http_protocol_options": {},
+  "loadAssignment": {
+    "clusterName": "self_admin",
+    "endpoints": [
+      {
+        "lbEndpoints": [
+          {
+            "endpoint": {
+              "address": {
+                "socket_address": {
+                  "address": "192.0.2.10",
+                  "port_value": 19002
+                }
+              }
+            }
+          }
+        ]
+      }
+    ]
+  }
+}`
+	expectedPrometheusBackendCluster = `{
+  "name": "prometheus_backend",
+  "ignore_health_on_host_removal": false,
+  "connect_timeout": "5s",
+  "type": "STATIC",
+  "http_protocol_options": {},
+  "loadAssignment": {
+    "clusterName": "prometheus_backend",
+    "endpoints": [
+      {
+        "lbEndpoints": [
+          {
+            "endpoint": {
+              "address": {
+                "socket_address": {
+                  "address": "127.0.0.1",
+                  "port_value": 20100
+                }
+              }
+            }
+          }
+        ]
+      }
+    ]
+  }
+}`
 	expectedPromListener = `{
-		"name": "envoy_prometheus_metrics_listener",
-		"address": {
-			"socket_address": {
-				"address": "0.0.0.0",
-				"port_value": 9000
-			}
-		},
-		"filter_chains": [
-			{
-				"filters": [
-					{
-						"name": "envoy.http_connection_manager",
-						"config": {
-							"stat_prefix": "envoy_prometheus_metrics",
-							"codec_type": "HTTP1",
-							"route_config": {
-								"name": "self_admin_route",
-								"virtual_hosts": [
-									{
-										"name": "self_admin",
-										"domains": [
-											"*"
-										],
-										"routes": [
-											{
-												"match": {
-													"path": "/metrics"
-												},
-												"route": {
-													"cluster": "self_admin",
-													"prefix_rewrite": "/stats/prometheus"
-												}
-											},
-											{
-												"match": {
-													"prefix": "/"
-												},
-												"direct_response": {
-													"status": 404
-												}
-											}
-										]
-									}
-								]
-							},
-							"http_filters": [
-								{
-									"name": "envoy.router"
-								}
-							]
+	"name": "envoy_prometheus_metrics_listener",
+	"address": {
+	  "socket_address": {
+		"address": "0.0.0.0",
+		"port_value": 9000
+	  }
+	},
+	"filter_chains": [
+	  {
+		"filters": [
+		  {
+			"name": "envoy.filters.network.http_connection_manager",
+			"typedConfig": {
+			  "@type": "type.googleapis.com/envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager",
+			  "stat_prefix": "envoy_prometheus_metrics",
+			  "codec_type": "HTTP1",
+			  "route_config": {
+				"name": "self_admin_route",
+				"virtual_hosts": [
+				  {
+					"name": "self_admin",
+					"domains": [
+					  "*"
+					],
+					"routes": [
+					  {
+						"match": {
+						  "path": "/metrics"
+						},
+						"route": {
+						  "cluster": "self_admin",
+						  "prefix_rewrite": "/stats/prometheus"
 						}
-					}
+					  },
+					  {
+						"match": {
+						  "prefix": "/"
+						},
+						"direct_response": {
+						  "status": 404
+						}
+					  }
+					]
+				  }
 				]
+			  },
+			  "http_filters": [
+				{
+				  "name": "envoy.filters.http.router"
+				}
+			  ]
 			}
+		  }
 		]
-	}`
+	  }
+	]
+  }`
+	expectedPromListenerCustomScrapePath = `{
+	"name": "envoy_prometheus_metrics_listener",
+	"address": {
+	  "socket_address": {
+		"address": "0.0.0.0",
+		"port_value": 9000
+	  }
+	},
+	"filter_chains": [
+	  {
+		"filters": [
+		  {
+			"name": "envoy.filters.network.http_connection_manager",
+			"typedConfig": {
+			  "@type": "type.googleapis.com/envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager",
+			  "stat_prefix": "envoy_prometheus_metrics",
+			  "codec_type": "HTTP1",
+			  "route_config": {
+				"name": "self_admin_route",
+				"virtual_hosts": [
+				  {
+					"name": "self_admin",
+					"domains": [
+					  "*"
+					],
+					"routes": [
+					  {
+						"match": {
+						  "path": "/scrape-path"
+						},
+						"route": {
+						  "cluster": "self_admin",
+						  "prefix_rewrite": "/stats/prometheus"
+						}
+					  },
+					  {
+						"match": {
+						  "prefix": "/"
+						},
+						"direct_response": {
+						  "status": 404
+						}
+					  }
+					]
+				  }
+				]
+			  },
+			  "http_filters": [
+				{
+				  "name": "envoy.filters.http.router"
+				}
+			  ]
+			}
+		  }
+		]
+	  }
+	]
+  }`
+	expectedPromListenerWithPrometheusBackendCluster = `{
+	"name": "envoy_prometheus_metrics_listener",
+	"address": {
+	  "socket_address": {
+		"address": "0.0.0.0",
+		"port_value": 9000
+	  }
+	},
+	"filter_chains": [
+	  {
+		"filters": [
+		  {
+			"name": "envoy.filters.network.http_connection_manager",
+			"typedConfig": {
+			  "@type": "type.googleapis.com/envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager",
+			  "stat_prefix": "envoy_prometheus_metrics",
+			  "codec_type": "HTTP1",
+			  "route_config": {
+				"name": "self_admin_route",
+				"virtual_hosts": [
+				  {
+					"name": "self_admin",
+					"domains": [
+					  "*"
+					],
+					"routes": [
+					  {
+						"match": {
+						  "path": "/metrics"
+						},
+						"route": {
+						  "cluster": "prometheus_backend",
+						  "prefix_rewrite": "/stats/prometheus"
+						}
+					  },
+					  {
+						"match": {
+						  "prefix": "/"
+						},
+						"direct_response": {
+						  "status": 404
+						}
+					  }
+					]
+				  }
+				]
+			  },
+			  "http_filters": [
+				{
+				  "name": "envoy.filters.http.router"
+				}
+			  ]
+			}
+		  }
+		]
+	  }
+	]
+  }`
 	expectedStatsListener = `{
-		"name": "envoy_metrics_listener",
-		"address": {
-			"socket_address": {
-				"address": "0.0.0.0",
-				"port_value": 9000
-			}
-		},
-		"filter_chains": [
-			{
-				"filters": [
-					{
-						"name": "envoy.http_connection_manager",
-						"config": {
-							"stat_prefix": "envoy_metrics",
-							"codec_type": "HTTP1",
-							"route_config": {
-								"name": "self_admin_route",
-								"virtual_hosts": [
-									{
-										"name": "self_admin",
-										"domains": [
-											"*"
-										],
-										"routes": [
-											{
-												"match": {
-													"prefix": "/stats"
-												},
-												"route": {
-													"cluster": "self_admin",
-													"prefix_rewrite": "/stats"
-												}
-											},
-											{
-												"match": {
-													"prefix": "/"
-												},
-												"direct_response": {
-													"status": 404
-												}
-											}
-										]
-									}
-								]
-							},
-							"http_filters": [
-								{
-									"name": "envoy.router"
-								}
-							]
-						}
-					}
-				]
-			}
-		]
-	}`
+  "name": "envoy_metrics_listener",
+  "address": {
+    "socket_address": {
+      "address": "0.0.0.0",
+      "port_value": 9000
+    }
+  },
+  "filter_chains": [
+    {
+      "filters": [
+        {
+          "name": "envoy.filters.network.http_connection_manager",
+          "typedConfig": {
+            "@type": "type.googleapis.com/envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager",
+            "stat_prefix": "envoy_metrics",
+            "codec_type": "HTTP1",
+            "route_config": {
+              "name": "self_admin_route",
+              "virtual_hosts": [
+                {
+                  "name": "self_admin",
+                  "domains": [
+                    "*"
+                  ],
+                  "routes": [
+                    {
+                      "match": {
+                        "prefix": "/stats"
+                      },
+                      "route": {
+                        "cluster": "self_admin",
+                        "prefix_rewrite": "/stats"
+                      }
+                    },
+                    {
+                      "match": {
+                        "prefix": "/"
+                      },
+                      "direct_response": {
+                        "status": 404
+                      }
+                    }
+                  ]
+                }
+              ]
+            },
+            "http_filters": [
+              {
+                "name": "envoy.filters.http.router"
+              }
+            ]
+          }
+        }
+      ]
+    }
+  ]
+}`
 	expectedReadyListener = `{
-		"name": "envoy_ready_listener",
-		"address": {
-			"socket_address": {
-				"address": "0.0.0.0",
-				"port_value": 4444
-			}
-		},
-		"filter_chains": [
-			{
-				"filters": [
-					{
-						"name": "envoy.http_connection_manager",
-						"config": {
-							"stat_prefix": "envoy_ready",
-							"codec_type": "HTTP1",
-							"route_config": {
-								"name": "self_admin_route",
-								"virtual_hosts": [
-									{
-										"name": "self_admin",
-										"domains": [
-											"*"
-										],
-										"routes": [
-											{
-												"match": {
-													"path": "/ready"
-												},
-												"route": {
-													"cluster": "self_admin",
-													"prefix_rewrite": "/ready"
-												}
-											},
-											{
-												"match": {
-													"prefix": "/"
-												},
-												"direct_response": {
-													"status": 404
-												}
-											}
-										]
-									}
-								]
-							},
-							"http_filters": [
-								{
-									"name": "envoy.router"
-								}
-							]
-						}
-					}
-				]
-			}
-		]
-	}`
+  "name": "envoy_ready_listener",
+  "address": {
+    "socket_address": {
+      "address": "0.0.0.0",
+      "port_value": 4444
+    }
+  },
+  "filter_chains": [
+    {
+      "filters": [
+        {
+          "name": "envoy.filters.network.http_connection_manager",
+          "typedConfig": {
+            "@type": "type.googleapis.com/envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager",
+            "stat_prefix": "envoy_ready",
+            "codec_type": "HTTP1",
+            "route_config": {
+              "name": "self_admin_route",
+              "virtual_hosts": [
+                {
+                  "name": "self_admin",
+                  "domains": [
+                    "*"
+                  ],
+                  "routes": [
+                    {
+                      "match": {
+                        "path": "/ready"
+                      },
+                      "route": {
+                        "cluster": "self_admin",
+                        "prefix_rewrite": "/ready"
+                      }
+                    },
+                    {
+                      "match": {
+                        "prefix": "/"
+                      },
+                      "direct_response": {
+                        "status": 404
+                      }
+                    }
+                  ]
+                }
+              ]
+            },
+            "http_filters": [
+              {
+                "name": "envoy.filters.http.router"
+              }
+            ]
+          }
+        }
+      ]
+    }
+  ]
+}`
 )
 
 func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
@@ -271,8 +440,9 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 			wantArgs: BootstrapTplArgs{
 				StatsConfigJSON: defaultStatsConfigJSON,
 				StatsSinksJSON: `[{
-					"name": "envoy.statsd",
-					"config": {
+					"name": "envoy.stat_sinks.statsd",
+					"typedConfig": {
+						"@type": "type.googleapis.com/envoy.config.metrics.v3.StatsdSink",
 						"address": {
 							"socket_address": {
 								"address": "127.0.0.1",
@@ -298,8 +468,9 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 			wantArgs: BootstrapTplArgs{
 				StatsConfigJSON: defaultStatsConfigJSON,
 				StatsSinksJSON: `[{
-					"name": "envoy.statsd",
-					"config": {
+					"name": "envoy.stat_sinks.statsd",
+					"typedConfig": {
+						"@type": "type.googleapis.com/envoy.config.metrics.v3.StatsdSink",
 						"address": {
 							"socket_address": {
 								"address": "127.0.0.1",
@@ -326,8 +497,9 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 			wantArgs: BootstrapTplArgs{
 				StatsConfigJSON: defaultStatsConfigJSON,
 				StatsSinksJSON: `[{
-					"name": "envoy.statsd",
-					"config": {
+					"name": "envoy.stat_sinks.statsd",
+					"typedConfig": {
+						"@type": "type.googleapis.com/envoy.config.metrics.v3.StatsdSink",
 						"address": {
 							"socket_address": {
 								"address": "127.0.0.1",
@@ -339,6 +511,40 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 			},
 			wantErr: false,
 		},
+		{
+			name: "simple-statsd-sink-inline-env-allowed",
+			input: BootstrapConfig{
+				StatsdURL: "udp://$HOST_IP:9125",
+			},
+			env: []string{"HOST_IP=127.0.0.1"},
+			wantArgs: BootstrapTplArgs{
+				StatsConfigJSON: defaultStatsConfigJSON,
+				StatsSinksJSON: `[{
+					"name": "envoy.stat_sinks.statsd",
+					"typedConfig": {
+						"@type": "type.googleapis.com/envoy.config.metrics.v3.StatsdSink",
+						"address": {
+							"socket_address": {
+								"address": "127.0.0.1",
+								"port_value": 9125
+							}
+						}
+					}
+				}]`,
+			},
+			wantErr: false,
+		},
+		{
+			name: "simple-statsd-sink-inline-env-disallowed",
+			input: BootstrapConfig{
+				StatsdURL: "udp://$HOST_ADDRESS:9125",
+			},
+			env: []string{"HOST_ADDRESS=127.0.0.1"},
+			wantArgs: BootstrapTplArgs{
+				StatsConfigJSON: defaultStatsConfigJSON,
+			},
+			wantErr: true,
+		},
 		{
 			name: "simple-dogstatsd-sink",
 			input: BootstrapConfig{
@@ -347,8 +553,9 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 			wantArgs: BootstrapTplArgs{
 				StatsConfigJSON: defaultStatsConfigJSON,
 				StatsSinksJSON: `[{
-					"name": "envoy.dog_statsd",
-					"config": {
+					"name": "envoy.stat_sinks.dog_statsd",
+					"typedConfig": {
+						"@type": "type.googleapis.com/envoy.config.metrics.v3.DogStatsdSink",
 						"address": {
 							"socket_address": {
 								"address": "127.0.0.1",
@@ -368,8 +575,9 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 			wantArgs: BootstrapTplArgs{
 				StatsConfigJSON: defaultStatsConfigJSON,
 				StatsSinksJSON: `[{
-					"name": "envoy.dog_statsd",
-					"config": {
+					"name": "envoy.stat_sinks.dog_statsd",
+					"typedConfig": {
+						"@type": "type.googleapis.com/envoy.config.metrics.v3.DogStatsdSink",
 						"address": {
 							"pipe": {
 								"path": "/var/run/dogstatsd.sock"
@@ -390,8 +598,9 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 			wantArgs: BootstrapTplArgs{
 				StatsConfigJSON: defaultStatsConfigJSON,
 				StatsSinksJSON: `[{
-					"name": "envoy.dog_statsd",
-					"config": {
+					"name": "envoy.stat_sinks.dog_statsd",
+					"typedConfig": {
+						"@type": "type.googleapis.com/envoy.config.metrics.v3.DogStatsdSink",
 						"address": {
 							"socket_address": {
 								"address": "127.0.0.1",
@@ -450,8 +659,9 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 				PrometheusBindAddr: "0.0.0.0:9000",
 			},
 			baseArgs: BootstrapTplArgs{
-				AdminBindAddress: "127.0.0.1",
-				AdminBindPort:    "19000",
+				AdminBindAddress:     "127.0.0.1",
+				AdminBindPort:        "19000",
+				PrometheusScrapePath: "/metrics",
 			},
 			wantArgs: BootstrapTplArgs{
 				AdminBindAddress: "127.0.0.1",
@@ -459,8 +669,9 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 				// Should add a static cluster for the self-proxy to admin
 				StaticClustersJSON: expectedSelfAdminCluster,
 				// Should add a static http listener too
-				StaticListenersJSON: expectedPromListener,
-				StatsConfigJSON:     defaultStatsConfigJSON,
+				StaticListenersJSON:  expectedPromListener,
+				StatsConfigJSON:      defaultStatsConfigJSON,
+				PrometheusScrapePath: "/metrics",
 			},
 			wantErr: false,
 		},
@@ -470,8 +681,9 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 				PrometheusBindAddr: "0.0.0.0:9000",
 			},
 			baseArgs: BootstrapTplArgs{
-				AdminBindAddress: "192.0.2.10",
-				AdminBindPort:    "19002",
+				AdminBindAddress:     "192.0.2.10",
+				AdminBindPort:        "19002",
+				PrometheusScrapePath: "/metrics",
 			},
 			wantArgs: BootstrapTplArgs{
 				AdminBindAddress: "192.0.2.10",
@@ -479,8 +691,9 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 				// Should add a static cluster for the self-proxy to admin
 				StaticClustersJSON: expectedSelfAdminClusterNonLoopbackIP,
 				// Should add a static http listener too
-				StaticListenersJSON: expectedPromListener,
-				StatsConfigJSON:     defaultStatsConfigJSON,
+				StaticListenersJSON:  expectedPromListener,
+				StatsConfigJSON:      defaultStatsConfigJSON,
+				PrometheusScrapePath: "/metrics",
 			},
 			wantErr: false,
 		},
@@ -492,8 +705,9 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 				StaticListenersJSON: `{"baz":"qux"}`,
 			},
 			baseArgs: BootstrapTplArgs{
-				AdminBindAddress: "127.0.0.1",
-				AdminBindPort:    "19000",
+				AdminBindAddress:     "127.0.0.1",
+				AdminBindPort:        "19000",
+				PrometheusScrapePath: "/scrape-path",
 			},
 			wantArgs: BootstrapTplArgs{
 				AdminBindAddress: "127.0.0.1",
@@ -501,8 +715,33 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 				// Should add a static cluster for the self-proxy to admin
 				StaticClustersJSON: `{"foo":"bar"},` + expectedSelfAdminCluster,
 				// Should add a static http listener too
-				StaticListenersJSON: `{"baz":"qux"},` + expectedPromListener,
-				StatsConfigJSON:     defaultStatsConfigJSON,
+				StaticListenersJSON:  `{"baz":"qux"},` + expectedPromListenerCustomScrapePath,
+				StatsConfigJSON:      defaultStatsConfigJSON,
+				PrometheusScrapePath: "/scrape-path",
+			},
+			wantErr: false,
+		},
+		{
+			name: "prometheus-bind-addr-with-prometheus-backend",
+			input: BootstrapConfig{
+				PrometheusBindAddr: "0.0.0.0:9000",
+			},
+			baseArgs: BootstrapTplArgs{
+				AdminBindAddress:      "127.0.0.1",
+				AdminBindPort:         "19000",
+				PrometheusBackendPort: "20100",
+				PrometheusScrapePath:  "/metrics",
+			},
+			wantArgs: BootstrapTplArgs{
+				AdminBindAddress: "127.0.0.1",
+				AdminBindPort:    "19000",
+				// Should use the "prometheus_backend" cluster instead, which
+				// uses the PrometheusBackendPort rather than Envoy admin port
+				StaticClustersJSON:    expectedPrometheusBackendCluster,
+				StaticListenersJSON:   expectedPromListenerWithPrometheusBackendCluster,
+				StatsConfigJSON:       defaultStatsConfigJSON,
+				PrometheusBackendPort: "20100",
+				PrometheusScrapePath:  "/metrics",
 			},
 			wantErr: false,
 		},
@@ -648,8 +887,9 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 				StatsBindAddr:      "0.0.0.0:9000",
 			},
 			baseArgs: BootstrapTplArgs{
-				AdminBindAddress: "127.0.0.1",
-				AdminBindPort:    "19000",
+				AdminBindAddress:     "127.0.0.1",
+				AdminBindPort:        "19000",
+				PrometheusScrapePath: "/metrics",
 			},
 			wantArgs: BootstrapTplArgs{
 				AdminBindAddress: "127.0.0.1",
@@ -661,7 +901,8 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 					[]string{expectedPromListener, expectedStatsListener, expectedReadyListener},
 					", ",
 				),
-				StatsConfigJSON: defaultStatsConfigJSON,
+				StatsConfigJSON:      defaultStatsConfigJSON,
+				PrometheusScrapePath: "/metrics",
 			},
 			wantErr: false,
 		},
@@ -673,8 +914,9 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 				StatsBindAddr:      "0.0.0.0:9000",
 			},
 			baseArgs: BootstrapTplArgs{
-				AdminBindAddress: "127.0.0.1",
-				AdminBindPort:    "19000",
+				AdminBindAddress:     "127.0.0.1",
+				AdminBindPort:        "19000",
+				PrometheusScrapePath: "/metrics",
 			},
 			omitDeprecatedTags: true,
 			wantArgs: BootstrapTplArgs{
@@ -686,7 +928,8 @@ func TestBootstrapConfig_ConfigureArgs(t *testing.T) {
 					", ",
 				),
 				// Should not have default stats config JSON when deprecated tags are omitted
-				StatsConfigJSON: updatedStatsConfigJSON,
+				StatsConfigJSON:      updatedStatsConfigJSON,
+				PrometheusScrapePath: "/metrics",
 			},
 			wantErr: false,
 		},
