@@ -38,17 +38,19 @@ type cmd struct {
 	client *api.Client
 
 	// flags
-	meshGateway          bool
-	gateway              string
-	proxyID              string
-	sidecarFor           string
-	adminAccessLogPath   string
-	adminBind            string
-	envoyBin             string
-	bootstrap            bool
-	disableCentralConfig bool
-	grpcAddr             string
-	envoyVersion         string
+	meshGateway           bool
+	gateway               string
+	proxyID               string
+	sidecarFor            string
+	adminAccessLogPath    string
+	adminBind             string
+	envoyBin              string
+	bootstrap             bool
+	disableCentralConfig  bool
+	grpcAddr              string
+	envoyVersion          string
+	prometheusBackendPort string
+	prometheusScrapePath  string
 
 	// mesh gateway registration information
 	register           bool
@@ -151,6 +153,19 @@ func (c *cmd) init() {
 			"consul.destination.[service|dc|...]. The old tags were preserved for backward compatibility,"+
 			"but can be disabled with this flag.")
 
+	c.flags.StringVar(&c.prometheusBackendPort, "prometheus-backend-port", "",
+		"Sets the backend port for the 'prometheus_backend' cluster that envoy_prometheus_bind_addr will point to. "+
+			"Without this flag, envoy_prometheus_bind_addr would point to the 'self_admin' cluster where Envoy metrics are exposed. "+
+			"The metrics merging feature in consul-k8s uses this to point to the merged metrics endpoint combining Envoy and service metrics. "+
+			"Only applicable when envoy_prometheus_bind_addr is set in proxy config.")
+
+	c.flags.StringVar(&c.prometheusScrapePath, "prometheus-scrape-path", "/metrics",
+		"Sets the path where Envoy will expose metrics on envoy_prometheus_bind_addr listener. "+
+			"For example, if envoy_prometheus_bind_addr is 0.0.0.0:20200, and this flag is "+
+			"set to /scrape-metrics, prometheus metrics would be scrapeable at "+
+			"0.0.0.0:20200/scrape-metrics. "+
+			"Only applicable when envoy_prometheus_bind_addr is set in proxy config.")
+
 	c.http = &flags.HTTPFlags{}
 	flags.Merge(c.flags, c.http.ClientFlags())
 	flags.Merge(c.flags, c.http.NamespaceFlags())
@@ -469,6 +484,8 @@ func (c *cmd) templateArgs() (*BootstrapTplArgs, error) {
 		Namespace:             httpCfg.Namespace,
 		EnvoyVersion:          c.envoyVersion,
 		Datacenter:            httpCfg.Datacenter,
+		PrometheusBackendPort: c.prometheusBackendPort,
+		PrometheusScrapePath:  c.prometheusScrapePath,
 	}, nil
 }
 
