@@ -16,6 +16,10 @@ func TestEventCommand_noTabs(t *testing.T) {
 }
 
 func TestEventCommand(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a1 := agent.NewTestAgent(t, ``)
 	defer a1.Shutdown()
