@@ -20,6 +20,10 @@ func TestKVExportCommand_noTabs(t *testing.T) {
 }
 
 func TestKVExportCommand(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
