@@ -18,6 +18,10 @@ func TestOperatorAutopilotGetConfigCommand_noTabs(t *testing.T) {
 }
 
 func TestOperatorAutopilotGetConfigCommand(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
 	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
