@@ -16,8 +16,12 @@ func TestOperatorRaftRemovePeerCommand_noTabs(t *testing.T) {
 }
 
 func TestOperatorRaftRemovePeerCommand(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	t.Parallel()
-	a := agent.NewTestAgent(t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 
 	t.Run("Test the remove-peer subcommand directly", func(t *testing.T) {
