@@ -14,8 +14,6 @@ import (
 )
 
 func TestUpstreamResolverFuncFromClient(t *testing.T) {
-	t.Parallel()
-
 	tests := []struct {
 		name string
 		cfg  UpstreamConfig
@@ -79,6 +77,10 @@ func TestUpstreamResolverFuncFromClient(t *testing.T) {
 }
 
 func TestAgentConfigWatcherSidecarProxy(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	a := agent.StartTestAgent(t, agent.TestAgent{Name: "agent_smith"})
 	defer a.Shutdown()
 
