@@ -27,31 +27,35 @@ type Config struct {
 	// SyslogFacility is the destination for syslog forwarding.
 	SyslogFacility string
 
-	//LogFilePath is the path to write the logs to the user specified file.
+	// LogFilePath is the path to write the logs to the user specified file.
 	LogFilePath string
 
-	//LogRotateDuration is the user specified time to rotate logs
+	// LogRotateDuration is the user specified time to rotate logs
 	LogRotateDuration time.Duration
 
-	//LogRotateBytes is the user specified byte limit to rotate logs
+	// LogRotateBytes is the user specified byte limit to rotate logs
 	LogRotateBytes int
 
-	//LogRotateMaxFiles is the maximum number of past archived log files to keep
+	// LogRotateMaxFiles is the maximum number of past archived log files to keep
 	LogRotateMaxFiles int
 }
 
-const (
-	// defaultRotateDuration is the default time taken by the agent to rotate logs
-	defaultRotateDuration = 24 * time.Hour
-)
-
-var (
-	logRotateDuration time.Duration
-	logRotateBytes    int
-)
+// defaultRotateDuration is the default time taken by the agent to rotate logs
+const defaultRotateDuration = 24 * time.Hour
 
 type LogSetupErrorFn func(string)
 
+// noErrorWriter is a wrapper to suppress errors when writing to w.
+type noErrorWriter struct {
+	w io.Writer
+}
+
+func (w noErrorWriter) Write(p []byte) (n int, err error) {
+	_, _ = w.w.Write(p)
+	// We purposely return n == len(p) as if write was successful
+	return len(p), nil
+}
+
 // Setup logging from Config, and return an hclog Logger.
 //
 // Logs may be written to out, and optionally to syslog, and a file.
@@ -62,7 +66,10 @@ func Setup(config Config, out io.Writer) (hclog.InterceptLogger, error) {
 			allowedLogLevels)
 	}
 
-	writers := []io.Writer{out}
+	// If out is os.Stdout and Consul is being run as a Windows Service, writes will
+	// fail silently, which may inadvertently prevent writes to other writers.
+	// noErrorWriter is used as a wrapper to suppress any errors when writing to out.
+	writers := []io.Writer{noErrorWriter{w: out}}
 
 	if config.EnableSyslog {
 		retries := 12
@@ -86,28 +93,22 @@ func Setup(config Config, out io.Writer) (hclog.InterceptLogger, error) {
 	// Create a file logger if the user has specified the path to the log file
 	if config.LogFilePath != "" {
 		dir, fileName := filepath.Split(config.LogFilePath)
-		// If a path is provided but has no fileName a default is provided.
 		if fileName == "" {
 			fileName = "consul.log"
 		}
-		// Try to enter the user specified log rotation duration first
-		if config.LogRotateDuration != 0 {
-			logRotateDuration = config.LogRotateDuration
-		} else {
-			// Default to 24 hrs if no rotation period is specified
-			logRotateDuration = defaultRotateDuration
-		}
-		// User specified byte limit for log rotation if one is provided
-		if config.LogRotateBytes != 0 {
-			logRotateBytes = config.LogRotateBytes
+		if config.LogRotateDuration == 0 {
+			config.LogRotateDuration = defaultRotateDuration
 		}
 		logFile := &LogFile{
 			fileName: fileName,
 			logPath:  dir,
-			duration: logRotateDuration,
-			MaxBytes: logRotateBytes,
+			duration: config.LogRotateDuration,
+			MaxBytes: config.LogRotateBytes,
 			MaxFiles: config.LogRotateMaxFiles,
 		}
+		if err := logFile.pruneFiles(); err != nil {
+			return nil, fmt.Errorf("Failed to prune log files: %w", err)
+		}
 		if err := logFile.openNew(); err != nil {
 			return nil, fmt.Errorf("Failed to setup logging: %w", err)
 		}
