@@ -40,6 +40,10 @@ func TestMonitor_Start(t *testing.T) {
 }
 
 func TestMonitor_Stop(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	require := require.New(t)
 
 	logger := log.NewInterceptLogger(&log.LoggerOptions{
@@ -80,6 +84,10 @@ func TestMonitor_Stop(t *testing.T) {
 }
 
 func TestMonitor_DroppedMessages(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	require := require.New(t)
 
 	logger := log.NewInterceptLogger(&log.LoggerOptions{
@@ -122,6 +130,10 @@ func TestMonitor_DroppedMessages(t *testing.T) {
 }
 
 func TestMonitor_ZeroBufSizeDefault(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
 	require := require.New(t)
 
 	logger := log.NewInterceptLogger(&log.LoggerOptions{
