@@ -37,4 +37,8 @@ for f in \
     docker cp "${container}:/out/$f" workdir/secondary/tls
 done
 
+# Private keys have 600 perms but tests are run as another user
+chmod 666 workdir/primary/tls/primary-server-consul-0-key.pem
+chmod 666 workdir/secondary/tls/secondary-server-consul-0-key.pem
+
 docker rm -f "$container" >/dev/null || true
