@@ -5,8 +5,8 @@ mv /home/ubuntu/scripts/conf.yaml /etc/datadog-agent/conf.d/consul.d/
 mv /home/ubuntu/scripts/datadog.yaml /etc/datadog-agent/
 
 ##Move Consul Config that hooks up to datadog
+mkdir -p /opt/consul/config
 mv /home/ubuntu/scripts/telemetry.json /opt/consul/config/
-chown consul:consul /opt/consul/config/telemetry.json
 
 ## Let everyone own their stuff now
 chown dd-agent:dd-agent /etc/datadog-agent/conf.d/consul.d/conf.yaml
