@@ -3,9 +3,10 @@ package testrpc
 import (
 	"testing"
 
+	"github.com/stretchr/testify/require"
+
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/sdk/testutil/retry"
-	"github.com/stretchr/testify/require"
 )
 
 type rpcFn func(string, interface{}, interface{}) error
@@ -143,6 +144,7 @@ func WaitForTestAgent(t *testing.T, rpc rpcFn, dc string, options ...waitOption)
 // raft leadership is gained so WaitForLeader isn't sufficient to be sure that
 // the CA is fully initialized.
 func WaitForActiveCARoot(t *testing.T, rpc rpcFn, dc string, expect *structs.CARoot) {
+	t.Helper()
 	retry.Run(t, func(r *retry.R) {
 		args := &structs.DCSpecificRequest{
 			Datacenter: dc,
@@ -152,13 +154,7 @@ func WaitForActiveCARoot(t *testing.T, rpc rpcFn, dc string, expect *structs.CAR
 			r.Fatalf("err: %v", err)
 		}
 
-		var root *structs.CARoot
-		for _, r := range reply.Roots {
-			if r.ID == reply.ActiveRootID {
-				root = r
-				break
-			}
-		}
+		root := reply.Active()
 		if root == nil {
 			r.Fatal("no active root")
 		}
@@ -168,6 +164,27 @@ func WaitForActiveCARoot(t *testing.T, rpc rpcFn, dc string, expect *structs.CAR
 	})
 }
 
+// WaitForServiceIntentions waits until the server can accept config entry
+// kinds of service-intentions meaning any migration bootstrapping from pre-1.9
+// intentions has completed.
+func WaitForServiceIntentions(t *testing.T, rpc rpcFn, dc string) {
+	const fakeConfigName = "Sa4ohw5raith4si0Ohwuqu3lowiethoh"
+	retry.Run(t, func(r *retry.R) {
+		args := &structs.ConfigEntryRequest{
+			Op:         structs.ConfigEntryDelete,
+			Datacenter: dc,
+			Entry: &structs.ServiceIntentionsConfigEntry{
+				Kind: structs.ServiceIntentions,
+				Name: fakeConfigName,
+			},
+		}
+		var ignored struct{}
+		if err := rpc("ConfigEntry.Delete", args, &ignored); err != nil {
+			r.Fatalf("err: %v", err)
+		}
+	})
+}
+
 func WaitForACLReplication(t *testing.T, rpc rpcFn, dc string, expectedReplicationType structs.ACLReplicationType, minPolicyIndex, minTokenIndex, minRoleIndex uint64) {
 	retry.Run(t, func(r *retry.R) {
 		args := structs.DCSpecificRequest{
