@@ -1,4 +1,5 @@
 import Route from 'consul-ui/routing/route';
+import { get } from '@ember/object';
 
 export default class IndexRoute extends Route {
   queryParams = {
@@ -17,8 +18,9 @@ export default class IndexRoute extends Route {
   async model(params) {
     return {
       dc: this.modelFor('dc').dc.Name,
-      nspace: this.modelFor('nspace').nspace.substr(1) || 'default',
+      nspace: this.optionalParams().nspace || 'default',
       slug: this.paramsFor('dc.services.show').name,
+      item: get(this.modelFor('dc.services.show').items, 'firstObject'),
       searchProperties: this.queryParams.searchproperty.empty[0],
     };
   }
