@@ -1,7 +1,7 @@
 import Route from 'consul-ui/routing/route';
 
 export default class TagsRoute extends Route {
-  model() {
+  async model() {
     const parent = this.routeName
       .split('.')
       .slice(0, -1)
