@@ -1,11 +1,11 @@
 export default {
-  Name: (item, value) => item.Name,
-  Description: (item, value) => item.Description,
-  Role: (item, value) => {
+  Name: item => item.Name,
+  Description: item => item.Description,
+  Role: item => {
     const acls = item.ACLs || {};
     return (acls.RoleDefaults || []).map(item => item.Name);
   },
-  Policy: (item, value) => {
+  Policy: item => {
     const acls = item.ACLs || {};
     return (acls.PolicyDefaults || []).map(item => item.Name);
   },
