@@ -1,7 +1,7 @@
 export default {
-  Name: (item, value) => item.Name,
-  Description: (item, value) => item.Description,
-  Policy: (item, value) => {
+  Name: item => item.Name,
+  Description: item => item.Description,
+  Policy: item => {
     return (item.Policies || [])
       .map(item => item.Name)
       .concat((item.ServiceIdentities || []).map(item => item.ServiceName))
