@@ -2,6 +2,7 @@ import { inject as service } from '@ember/service';
 import RepositoryService from 'consul-ui/services/repository';
 import { get } from '@ember/object';
 import Error from '@ember/error';
+import dataSource from 'consul-ui/decorators/data-source';
 
 const modelName = 'dc';
 export default class DcService extends RepositoryService {
@@ -12,15 +13,16 @@ export default class DcService extends RepositoryService {
     return modelName;
   }
 
-  async findAll() {
+  @dataSource('/:ns/:dc/datacenters')
+  async findAll(params, configuration = {}) {
     return this.store.query(this.getModelName(), {});
   }
 
-  async findBySlug(name, items) {
+  async findBySlug(name) {
+    const items = this.store.peekAll('dc');
     if (name != null) {
       const item = await items.findBy('Name', name);
       if (typeof item !== 'undefined') {
-        await this.settings.persist({ dc: get(item, 'Name') });
         return item;
       }
     }
@@ -30,17 +32,13 @@ export default class DcService extends RepositoryService {
   }
 
   async getActive(name, items) {
-    return Promise.all([name || this.settings.findBySlug('dc'), items || this.findAll()]).then(
-      ([name, items]) => {
-        return this.findBySlug(name, items).catch(async e => {
-          const item =
-            items.findBy('Name', this.env.var('CONSUL_DATACENTER_LOCAL')) ||
-            get(items, 'firstObject');
-          await this.settings.persist({ dc: get(item, 'Name') });
-          return item;
-        });
-      }
-    );
+    return Promise.all([name, items || this.findAll()]).then(([name, items]) => {
+      return this.findBySlug(name, items).catch(async e => {
+        return (
+          items.findBy('Name', this.env.var('CONSUL_DATACENTER_LOCAL')) || get(items, 'firstObject')
+        );
+      });
+    });
   }
 
   async clearActive() {
