@@ -1,6 +1,7 @@
 import { inject as service } from '@ember/service';
 import { get, set } from '@ember/object';
 import RepositoryService from 'consul-ui/services/repository';
+import dataSource from 'consul-ui/decorators/data-source';
 
 const modelName = 'discovery-chain';
 const ERROR_MESH_DISABLED = 'Connect must be enabled in order to use this endpoint';
@@ -12,14 +13,15 @@ export default class DiscoveryChainService extends RepositoryService {
     return modelName;
   }
 
-  findBySlug(slug, dc, nspace, configuration = {}) {
-    const datacenter = this.dcs.peekOne(dc);
+  @dataSource('/:ns/:dc/discovery-chain/:id')
+  findBySlug(params, configuration = {}) {
+    const datacenter = this.dcs.peekOne(params.dc);
     if (datacenter !== null && !get(datacenter, 'MeshEnabled')) {
       return Promise.resolve();
     }
     return super.findBySlug(...arguments).catch(e => {
       const code = get(e, 'errors.firstObject.status');
-      const body = get(e, 'errors.firstObject.detail').trim();
+      const body = (get(e, 'errors.firstObject.detail') || '').trim();
       switch (code) {
         case '500':
           if (datacenter !== null && body.endsWith(ERROR_MESH_DISABLED)) {
