@@ -3,6 +3,7 @@ import { get } from '@ember/object';
 import statusFactory from 'consul-ui/utils/acls-status';
 import isValidServerErrorFactory from 'consul-ui/utils/http/acl/is-valid-server-error';
 import { PRIMARY_KEY, SLUG_KEY } from 'consul-ui/models/policy';
+import dataSource from 'consul-ui/decorators/data-source';
 
 const isValidServerError = isValidServerErrorFactory();
 const status = statusFactory(isValidServerError, Promise);
@@ -21,6 +22,16 @@ export default class PolicyService extends RepositoryService {
     return SLUG_KEY;
   }
 
+  @dataSource('/:ns/:dc/policies')
+  async findAllByDatacenter() {
+    return super.findAllByDatacenter(...arguments);
+  }
+
+  @dataSource('/:ns/:dc/policy/:id')
+  async findBySlug() {
+    return super.findBySlug(...arguments);
+  }
+
   status(obj) {
     return status(obj);
   }
