@@ -2,6 +2,7 @@ import RepositoryService from 'consul-ui/services/repository';
 import statusFactory from 'consul-ui/utils/acls-status';
 import isValidServerErrorFactory from 'consul-ui/utils/http/acl/is-valid-server-error';
 import { PRIMARY_KEY, SLUG_KEY } from 'consul-ui/models/role';
+import dataSource from 'consul-ui/decorators/data-source';
 
 const isValidServerError = isValidServerErrorFactory();
 const status = statusFactory(isValidServerError, Promise);
@@ -20,6 +21,16 @@ export default class RoleService extends RepositoryService {
     return SLUG_KEY;
   }
 
+  @dataSource('/:ns/:dc/roles')
+  async findAllByDatacenter() {
+    return super.findAllByDatacenter(...arguments);
+  }
+
+  @dataSource('/:ns/:dc/role/:id')
+  async findBySlug() {
+    return super.findBySlug(...arguments);
+  }
+
   status(obj) {
     return status(obj);
   }
