@@ -19,7 +19,7 @@ test('findBySlug returns the correct data for item endpoint', function(assert) {
       });
     },
     function performTest(service) {
-      return service.findBySlug(id, kind, dc);
+      return service.findBySlug({ id, kind, dc });
     },
     function performAssertion(actual, expected) {
       const result = expected(function(payload) {
