@@ -21,10 +21,10 @@ export default function ConsulSubnav() {
         {
           text: 'Try Cloud',
           url:
-            'https://cloud.hashicorp.com/?utm_source=consul_io&utm_content=top_nav',
+            'https://cloud.hashicorp.com/?utm_source=consul_io&utm_content=top_nav_consul',
         },
       ]}
-      currentPath={router.pathname}
+      currentPath={router.asPath}
       menuItemsAlign="right"
       menuItems={subnavItems}
       constrainWidth
