@@ -1,5 +1,5 @@
 export default [
-  { text: 'Overview', url: '/docs/intro' },
+  { text: 'Overview', url: '/' },
   {
     text: 'Use Cases',
     submenu: [
@@ -15,6 +15,10 @@ export default [
         text: 'Multi-Platform Service Mesh',
         url: '/use-cases/multi-platform-service-mesh',
       },
+      {
+        text: 'Consul on Kubernetes',
+        url: '/consul-on-kubernetes',
+      },
     ],
   },
   {
@@ -39,10 +43,10 @@ export default [
     url: '/api-docs',
     type: 'inbound',
   },
-  { 
+  {
     text: 'CLI',
     url: '/commands',
-    type: 'inbound,'
+    type: 'inbound,',
   },
   {
     text: 'Community',
