@@ -1,6 +1,7 @@
 import VerticalTextBlockList from '@hashicorp/react-vertical-text-block-list'
 import SectionHeader from '@hashicorp/react-section-header'
 import Head from 'next/head'
+import { productSlug } from 'data/metadata'
 
 export default function CommunityPage() {
   return (
@@ -14,6 +15,7 @@ export default function CommunityPage() {
         use_h1={true}
       />
       <VerticalTextBlockList
+        product={productSlug}
         data={[
           {
             header: 'Community Forum',
