@@ -6,7 +6,7 @@ export default function MultiPlatformServiceMeshPage() {
     <UseCaseLayout
       title="Multi-Platform Service Mesh"
       description="Create a consistent platform for modern application networking and security with identity based authorization, L7 traffic management, and service-to-service encryption."
-      guideLink="https://learn.hashicorp.com/consul/gs-consul-service-mesh/understand-consul-service-mesh"
+      guideLink="https://learn.hashicorp.com/tutorials/consul/service-mesh-deploy?utm_source=WEBSITE&utm_medium=WEB_IO&utm_offer=ARTICLE_PAGE&utm_content=DOCS"
     >
       <TextSplitWithImage
         textSplit={{
@@ -24,7 +24,7 @@ export default function MultiPlatformServiceMeshPage() {
           ],
         }}
         image={{
-          url: require('./img/multi-dc-multi-region.svg?url'),
+          url: require('./img/multi-platform-service-mesh/muilti-datacenter.png'),
         }}
       />
 
@@ -44,7 +44,7 @@ export default function MultiPlatformServiceMeshPage() {
           ],
         }}
         image={{
-          url: require('./img/service-to-service.svg?url'),
+          url: require('./img/multi-platform-service-mesh/service-to-service.png'),
         }}
       />
 
@@ -64,7 +64,7 @@ export default function MultiPlatformServiceMeshPage() {
           ],
         }}
         image={{
-          url: require('./img/website-app-aware-intentions.png?url'),
+          url: require('./img/multi-platform-service-mesh/traffic_mgmt@3x.png?url'),
         }}
       />
 
@@ -83,7 +83,7 @@ export default function MultiPlatformServiceMeshPage() {
           ],
         }}
         image={{
-          url: require('./img/IntegrateAndExtend.svg?url'),
+          url: require('./img/multi-platform-service-mesh/kubernetes-extend.png'),
         }}
       />
 
@@ -95,7 +95,7 @@ export default function MultiPlatformServiceMeshPage() {
           textSide: 'right',
         }}
         image={{
-          url: require('./img/connect-and-extend.svg?url'),
+          url: require('./img/multi-platform-service-mesh/extend-mesh.svg?url'),
         }}
       />
 
@@ -107,7 +107,7 @@ export default function MultiPlatformServiceMeshPage() {
           textSide: 'left',
         }}
         image={{
-          url: require('./img/robust-ecosystem.svg?url'),
+          url: require('./img/multi-platform-service-mesh/Ecosystem.png'),
         }}
       />
 
@@ -126,7 +126,7 @@ export default function MultiPlatformServiceMeshPage() {
           ],
         }}
         image={{
-          url: require('./img/website-mesh-viz.png?url'),
+          url: require('./img/multi-platform-service-mesh/observability@3x.png?url'),
         }}
       />
 
@@ -146,7 +146,7 @@ export default function MultiPlatformServiceMeshPage() {
             ],
           }}
           image={{
-            url: require('./img/services-screenshot.png?url'),
+            url: require('./img/discovery-health-checking/consul_screenshot@2x.png?url'),
           }}
         />
       </div>
